(() => {
    'use strict';

    angular.module('noc').factory('MetadataResource', ($q, nocUtil) => {
        const metadataResource = {};

        metadataResource.stringifySelectedRouteTables = function stringifySelectedRouteTables(routeTables) {
            return _(routeTables).filter('selected').map(function (routeTable) {
                return routeTable.name ? (routeTable.name + ' (' + routeTable.id + ')') : routeTable.id;
            }).join(', ');
        };

        metadataResource.getRouteTables = function (nocHttp, region, vpcId, cloudProviderAccountId) {
            return nocHttp.get('/aws/ha/metadata/route-tables?region=' + region + '&vpcId=' + vpcId + (cloudProviderAccountId ? '&cloudProviderAccountId=' + cloudProviderAccountId : ''),
                'retrieving route-table', nocHttp.LOADING)
                .then(function (response) {
                    return _.map(response, function (row) {
                        row.tags = _.sortBy(row.tags, 'key');
                        var nameTag = _.find(row.tags, {key: 'Name'});
                        row.name = nameTag && nameTag.value;
                        row.expanded = false;
                        return row;
                    });
                });
        };

        metadataResource.validateFloatingIPAddresses = function (nocHttp, region, vpcId, routeTableIds, floatingIps, cloudProviderAccountId) {
            return nocHttp.post({
                url: '/aws/ha/metadata/validate-floating-ips',
                data: {
                    routeTablesIds: routeTableIds,
                    floatingIps: floatingIps,
                    vpcId: vpcId,
                    region: region,
                    cloudProviderAccountId
                }
            }, 'setting floating IP addresses', nocHttp.LOADING);
        };

        metadataResource.getKeyPairs = function (nocHttp, pidgeon, regionCode, singleNode, cloudProviderAccountId) {
            var uri = (singleNode ? '/vsa/metadata/key-pairs?region=' : '/aws/ha/metadata/key-pairs?region=') + regionCode + (cloudProviderAccountId ? '&cloudProviderAccountId=' + cloudProviderAccountId : '');

            pidgeon.clear('getKeyPairs');

            return nocHttp.get(uri,
                'retrieving Key Pairs', nocHttp.LOADING)
                .then(function (keyPairs) {
                    if (!keyPairs || keyPairs.length === 0) {
                        var message = 'No key pairs are available in this region. Please add Key Pair or try another region.';

                        pidgeon.error(message, null, 'getKeyPairs');
                        return $q.reject(message);
                    } else {
                        return keyPairs;
                    }
                });
        };

        metadataResource.getKeyPairsForAllRegions = function (nocHttp, pidgeon, singleNode, cloudProviderAccountId) {
            const uri = singleNode ? '/vsa/metadata/key-pairs-by-region' : '/aws/ha/metadata/key-pairs-by-region';

            pidgeon.clear('key-pairs-by-region');
            return nocHttp.get(uri + (cloudProviderAccountId ? '?cloudProviderAccountId=' + cloudProviderAccountId : ''),
                'retrieving Key Pairs', nocHttp.LOADING);
        };

        metadataResource.legacyInstanceTypes = function (nocHttp, singleNode, isAzure) {
            var uri = singleNode ? '/vsa/metadata/instance-types-not-supporting-acceleration-and-capacity-tiering' : '/aws/ha/metadata/instance-types-not-supporting-acceleration-and-capacity-tiering';
            if (isAzure) {
                uri = '/azure/vsa/metadata/instance-types-not-supporting-acceleration-and-capacity-tiering';
            }
            return nocHttp.get(uri, 'retrieving instances not supporting acceleration and capacity tiering', nocHttp.LOADING)
                .then(function (response) {
                    return response.instanceTypes;
                });
        };
        metadataResource.isRegisteredToMarketPlace = function isRegisteredToMarketPlace(nocHttp, request, type, provider) {
            //"In order to use this AWS Marketplace product you need to accept terms and subscribe. To do so please visit http://aws.amazon.com/marketplace/pp?sku=bk9ifwvjadm70j0qym0ui6msi"
            var pathPrefix = nocUtil.workingEnvironmentTypeToRequestPathPrefix(type, provider);
            var uri = pathPrefix + 'metadata/validate-subscribed-to-ontap-cloud';
            return nocHttp.post({
                url: uri,
                data: request
            }, 'checking registration', nocHttp.BLOCK)
                .then((result) => {
                    if (!result || result.isSubscribed) {
                        return {
                            subscribed: true
                        };
                    } else {
                        if (result.failureInfo) {
                            const index = result.failureInfo.indexOf('visit');
                            let url = '';
                            if (index !== -1) {
                                url = result.failureInfo.substr(index + 6);
                            }
                            return {
                                subscribed: false,
                                url: url
                            };
                        } else {
                            return {
                                subscribed: false,
                                url: ''
                            };
                        }

                    }
                });
        };

        function getIconNamePerPackage(packageName) {
            if (_.endsWith(packageName, 'poc')) return 'light-bulb';
            if (_.endsWith(packageName, 'standard')) return 'volume-star';
            if (_.endsWith(packageName, 'dr')) return 'low-cost';
            if (_.endsWith(packageName, 'fastest')) return 'rocket';
        }

        metadataResource.getPackages = function getPackages(nocHttp, type, providerName) {
            var pathPrefix = nocUtil.workingEnvironmentTypeToRequestPathPrefix(type, providerName);
            var url = pathPrefix + 'metadata/packages';

            return nocHttp.get(url, 'retrieving packages', nocHttp.LOADING)
                .then(function (packagesResponse) {
                    _.forEach(packagesResponse, function (preConfPackage) {
                        preConfPackage._iconName = getIconNamePerPackage(preConfPackage.name);
                        preConfPackage.diskSize._identifier = preConfPackage.diskSize.size + ' ' + preConfPackage.diskSize.unit;
                    });

                    return packagesResponse;
                });
        };

        //todo: Discover - need to delete this function after changing for discover with accounts in new version
        metadataResource.getAdditionalAccounts = function getAdditionalAccounts(nocHttp) {
            return nocHttp.get('/vsa/metadata/additional-accounts', nocHttp.BLOCK);
        };

        metadataResource.postAwsAccount = function postAwsAccount(nocHttp, url, accountId, role) {
            return nocHttp.post({
                url: url,
                data: {
                    accountId: accountId,
                    role: role
                }
            }, 'adding a new AWS account', nocHttp.BLOCK);
        };

        metadataResource.getInstanceProfiles = (nocHttp, prefix, cloudProviderAccountId) => {
            // eslint-disable-next-line no-nested-ternary
            return nocHttp.get(prefix + 'metadata/instance-profiles' + (cloudProviderAccountId ? '?cloudProviderAccountId=' + cloudProviderAccountId : ''), 'retrieving instance-profiles');
        };

        metadataResource.getRegions = function getRegions(nocHttp, pathPrefix, withVnets, providerAccountIdParam, cloudProviderAccountId) {
            const fields = withVnets ? '?fields=vnets' : '';

            return nocHttp.get(pathPrefix + 'metadata/regions' + fields + (fields ? '&' : '?' ) + (providerAccountIdParam ? providerAccountIdParam : '') + (providerAccountIdParam ? '&' : '' ) + 'cloudProviderAccountId=' + cloudProviderAccountId, 'retrieving regions', nocHttp.BLOCK);
        };

        metadataResource.getNetworkRequirements = function getNetworkRequirements(nocHttp, pathPrefix, weType, failoverMode) {
            return nocHttp.get(pathPrefix + 'metadata/network-requirements', 'retrieving network requirements', nocHttp.BLOCK)
                .then((response) => {
                    if (weType === 'HA') {
                        if (failoverMode === 'FloatingIP') {
                            return {
                                node: response.haNodeMinimumRequiredIps,
                                mediator: response.haMediatorMinimumRequiredIps
                            };
                        } else {
                            return {node: 2 * response.haNodeMinimumRequiredIps + response.haMediatorMinimumRequiredIps};
                        }
                    } else {
                        return {
                            node: response.nonHaNodeMinimumRequiredIps || response.vsaMinimumRequiredIps
                        };
                    }
                });
        };


        metadataResource.getTagKeys = function getTagKeys(nocHttp, pathPrefix, params) {
            return nocHttp.get(pathPrefix + 'metadata/tag-keys' + params, 'querying for all tag names', nocHttp.BLOCK);
        };

        metadataResource.getAssociatedSubscriptionsForAzure = function getAssociatedSubscriptionsForAzure(nocHttp, pidgeon, cloudProviderAccountId) {
            return nocHttp.get('/azure/vsa/metadata/associated-subscriptions?cloudProviderAccountId=' + cloudProviderAccountId, 'retrieving associated subscriptions', nocHttp.BLOCK)
                .then((subscriptions) => {
                    if (!subscriptions || subscriptions.length === 0) {
                        const message = 'Your credentials are not associated with any subscription. Please go to Azure Portal and associate a subscription';

                        pidgeon.error(message, null, 'getAssociatedSubscriptionsForAzure');

                        return $q.reject(message);
                    } else {
                        return subscriptions;
                    }
                });
        };

        metadataResource.getSupportedFeatures = function getSupportedFeatures(nocHttp, pathPrefix, region, ontapVersion, dataEncryptionType, licenseType, instanceType) {
            return nocHttp.get({
                url: pathPrefix + 'metadata/supported-features',
                params: {
                    region,
                    ontapVersion,
                    dataEncryptionType,
                    licenseType,
                    instanceType
                }
            }, 'retrieving supported features', nocHttp.LOADING);
        };

        return metadataResource;
    });
})();
