(() => {
    'use strict';

    angular.module('noc').factory('TieringMetadataResource', (nocUtil, $q, providerDiskTypes) => {
        const resource = {};
        const diskTypesAws = _.cloneDeep(providerDiskTypes.AWS);
        const diskTypesGcp = _.cloneDeep(providerDiskTypes.GCP);
        let diskTypesAzure = _.cloneDeep(providerDiskTypes.Azure);

        resource.initEbsDiskTypesResponse = function initEbsDiskTypesResponse(response, license, isHa, supportedCapacityTiers) {
            _.forEach(diskTypesAws, (diskType) => {
                diskType.supportedSizes = _(response)
                    //Remove disk types that are not supported in current license:
                    .filter((od) => {
                        return _.includes(od.supportedVolumeTypes, diskType.diskName) && _.includes(od.supportedOccmLicenses, license);
                    })
                    //Pluck the disk sizes and add string identifier:
                    .map('size')
                    .map((diskSize) => {
                        diskSize._identifier = `${diskSize.size} ${diskSize.unit}`;

                        return diskSize;
                    })
                    .value();

                const diskObjectFromSupportedCapacityTiers = _.find(supportedCapacityTiers.supportedCapacityTiersPerVolumeType, {volumeType: diskType.name});
                diskType.disabledMessage = diskObjectFromSupportedCapacityTiers ? diskObjectFromSupportedCapacityTiers.disabledMessage : '';
            });

            //Remove disk types that are not supported for HA, if relevant:
            return _.pickBy(diskTypesAws, (diskType) => {
                return (!isHa || diskType.supportedInHa);
            });
        };

        //Assuming each Azure diskType has only one availabilityType
        resource.initAzureDiskTypesResponse = function initAzureDiskTypesResponse(response, license, pathPrefix) {
            diskTypesAzure = _.cloneDeep(providerDiskTypes.Azure);
            //Remove Standard Disk for Azure HA (not supported)
            if (pathPrefix === '/azure/ha/') {
                delete diskTypesAzure.Standard_LRS;
                delete diskTypesAzure.StandardSSD_LRS;
            }

            const responseMap = _.keyBy(response, 'availabilityTypes[0]');
            _.forEach(_.keys(diskTypesAzure), (type) => {
                diskTypesAzure[type].supportedSizes =
                    //eslint-disable-next-line lodash/prefer-lodash-method
                    responseMap[type].sizes
                        .filter((diskSize) => {
                            return _.includes(diskSize.supportedOccmLicenses, license);
                        })
                        //Pluck the disk sizes and add string identifier:
                        .map((diskSize) => {
                            diskSize._identifier = `${diskSize.size.size} ${diskSize.size.unit}`;

                            return diskSize;
                        });
            });

            return diskTypesAzure;
        };

        resource.initGcpDiskTypesResponse = function initGcpDiskTypesResponse(response, license) {
            _.forEach(diskTypesGcp, (diskType) => {
                diskType.supportedSizes = _(response)
                //Remove disk types that are not supported in current license:
                    .filter((od) => {
                        return _.includes(od.supportedDiskTypes, diskType.diskName) && _.includes(od.supportedOccmLicenses, license);
                    })
                    //Pluck the disk sizes and add string identifier:
                    .map('size')
                    .map((diskSize) => {
                        diskSize._identifier = `${diskSize.size} ${diskSize.unit}`;

                        return diskSize;
                    })
                    .value();
            });

            return diskTypesGcp;
        };

        resource.getSupportedCapacityTiers = function getSupportedCapacityTiers(nocHttp, pathPrefix, region, ontapVersion, dataEncryptionType, licenseType, instanceType, subscriptionId, cloudProviderAccountId) {
            return nocHttp.get({
                url: pathPrefix + 'metadata/supported-capacity-tiers',
                params: {
                    region,
                    ontapVersion,
                    dataEncryptionType,
                    licenseType,
                    instanceType,
                    subscriptionId,
                    cloudProviderAccountId
                }
            }, 'retrieving supported capacity tiers', nocHttp.LOADING);
        };

        resource.getPossibleTiers = function getPossibleTiers(nocHttp, workingEnvironmentType, workingEnvironmentProvider, license, region, ontapVersion, dataEncryptionType, instanceType, cloudProviderAccountId) {
            const pathPrefix = nocUtil.workingEnvironmentTypeToRequestPathPrefix(workingEnvironmentType, workingEnvironmentProvider);

            if (workingEnvironmentProvider === 'AWS') {
                return $q.all([
                    nocHttp.get(pathPrefix + 'metadata/ebs-volume-types', 'retrieving AWS disk types', nocHttp.LOADING),
                    resource.getSupportedCapacityTiers(nocHttp, pathPrefix, region, ontapVersion, dataEncryptionType, license.type, instanceType, null, cloudProviderAccountId)
                ])
                    .then(_.spread((diskTypesAndSizesResponse, supportedCapacityTiersResponse) => {
                        return resource.initEbsDiskTypesResponse(diskTypesAndSizesResponse, license.name, workingEnvironmentType === 'HA', supportedCapacityTiersResponse);
                    }));
            } else if (workingEnvironmentProvider === 'Azure') {
                return nocHttp.get(pathPrefix + 'metadata/storage-account-types',
                    'retrieving Azure disk types', nocHttp.LOADING)
                    .then((response) => {
                        return resource.initAzureDiskTypesResponse(response, license.name, pathPrefix);
                    });
            } else if (workingEnvironmentProvider === 'GCP') {
                return nocHttp.get(pathPrefix + 'metadata/gcp-disk-types',
                    'retrieving GCP disk types', nocHttp.LOADING)
                    .then((response) => {
                        return resource.initGcpDiskTypesResponse(response, license.name);
                    });
            } else {
                return $q.reject('Unknown cloud provider name ' + workingEnvironmentProvider);
            }
        };

        return resource;
    });
})();
