(() => {
    'use strict';

    angular.module('noc').factory('volumesResource', (nocPoller, $rootScope, nocUtil, $filter, providerDiskTypes, $q, nocApprovalMessageBuilder, nocDialog/*, nocUIMain*/) => {
        const resource = {};

        resource.VOLUMES_GET_EVENT = 'volumesGetSuccess';
        resource.VOLUMES_GET_ERROR_EVENT = 'volumesGetError';

        function broadcastSuccess(workingEnvironment, volumes, unloading) {
            $rootScope.$broadcast(resource.VOLUMES_GET_EVENT, {
                workingEnvironment,
                volumes: resource.initVolumes(volumes),
                unloading: unloading || _.noop
            });
        }

        function broadcastError(workingEnvironment, error, unloading) {
            $rootScope.$broadcast(resource.VOLUMES_GET_ERROR_EVENT, {
                workingEnvironment,
                error,
                unloading: unloading || _.noop
            });
        }

        resource.startVolumesPolling = function startVolumesPolling(workingEnvironment, unloading) {
            return nocPoller.start({
                method: 'GET',
                url: workingEnvironment._pathPrefix + 'volumes?workingEnvironmentId=' + workingEnvironment.publicId
            }, 1 * 60 * 1000, function (volumes) {
                broadcastSuccess(workingEnvironment, volumes, unloading);
            }, function (error) {
                broadcastError(workingEnvironment, error, unloading);
            }, null, true);
        };

        resource.refreshVolumesData = function refreshVolumesData(nocHttp, workingEnvironment) {
            return nocHttp.get({
                url: workingEnvironment._pathPrefix + "volumes?workingEnvironmentId=" + workingEnvironment.publicId,
                headers: {"X-Occm-Bypass-Cache": true}
            }, null, nocHttp.LOADING)
                .then(function (volumes) {
                    broadcastSuccess(workingEnvironment, volumes);
                })
                .catch(function (error) {
                    broadcastError(workingEnvironment, error);
                });
        };

        resource.getVolumes = function getVolumes(nocHttp, workingEnvironmentId, pathPrefix, isLoadingUINeeded = true) {
            const loadingUI = isLoadingUINeeded ? nocHttp.LOADING : null;
            const opts = {
                url: pathPrefix + 'volumes',
                params: {workingEnvironmentId}
            };

            return nocHttp.get(opts, 'retrieving volumes for working environment ' + workingEnvironmentId, loadingUI)
                .then(resource.initVolumes);
        };

        function showCapacityTierUsedSizeInVolumeCards (volume) {
            const capacityTierUsedSize = _.get(volume, 'capacityTierUsedSize.size');
            return volume.tieringPolicy === 'all' || volume.tieringPolicy === 'auto' || volume.tieringPolicy === 'snapshot_only' || volume.tieringPolicy === 'backup' || (volume.tieringPolicy === 'none' && capacityTierUsedSize);
        }

        resource.initVolumes = function initVolumes(volumes) {
            _.forEach(volumes, (volume) => {
                volume._displaySize = $filter('normalizedCapacity')(volume.size, 2, true); //Add normalized size field to every volume to optimize filtering and display.
                volume._displayUsedSize = $filter('normalizedCapacity')(volume.usedSize, 2, true);
                volume._displayUsedDiskSize = volume.usedSize.size < 0 ? '0 KB' : $filter('normalizedCapacity')(volume.usedSize, 2, true);
                volume._displayCapacityTierUsedSize = $filter('normalizedCapacity')(volume.capacityTierUsedSize, 2, true);
                volume._showCapacityTierUsedSizeInVolumesCards = showCapacityTierUsedSizeInVolumeCards(volume);
                volume._shares = volume.shareNames ? volume.shareNames.join(', ') : undefined;

                const allocatedBytes = nocUtil.convertTo(volume.size, 'Byte').size;
                const capacityTierUsedSize = nocUtil.convertTo(volume.capacityTierUsedSize, 'Byte').size;
                const diskUsedSize = nocUtil.convertTo(volume.usedSize, 'Byte').size;
                const totalUsedBytes = diskUsedSize + capacityTierUsedSize;
                volume._totalUsedBytesPercent = totalUsedBytes / allocatedBytes * 100;
                volume._diskUsedPercent = (diskUsedSize / allocatedBytes) * 100;
            });

            _.remove(volumes, {volumeType: 'tmp'});

            return _.sortBy(volumes, function (vol) {
                return _.toLower(vol.name);
            });
        };

        function getDiskTypeLetters(providerVolumeType) {
            if (providerVolumeType === 'standard' || providerVolumeType === 'pd-standard') return 'std';
            else if (providerVolumeType === 'Standard_LRS') return 'hdd';
            else if (providerVolumeType === 'Premium_LRS') return 'prm';
            else if (providerVolumeType === 'StandardSSD_LRS' || providerVolumeType === 'pd-ssd') return 'ssd';
            else return providerVolumeType;
        }

        function calculateSyncFailureMessage(relationship, status) {
            var errors = [];

            if (status.status === 'FAILED') {
                errors.push(status.failureMessage);
            }
            if (relationship.relationshipStatus === 'FAILED') {
                errors.push('Relationship status: ' + _(relationship.relationshipStatus).toLower().replace(/_/g, ' '));
                errors.push(relationship.failureMessage);
            }

            return _.isEmpty(errors) ? null : errors;
        }

        function initBasicVolumeDisplay(volume, cloudProviderName) {
            const tieringPolicyNotNone = volume.tieringPolicy === 'auto' || volume.tieringPolicy === 'snapshot_only' || volume.tieringPolicy === 'backup' || volume.tieringPolicy === 'all';
            const providerToObjectStorageName = {
                AWS: 'S3',
                Azure: 'Blob',
                GCP: 'Cloud Storage'
            };

            volume._diskTypeLetters = volume.providerVolumeType ? getDiskTypeLetters(volume.providerVolumeType) : '';
            volume._capacityTier = volume._diskTypeLetters ? (volume._diskTypeLetters + ((volume.capacityTier && tieringPolicyNotNone) ? '+' + volume.capacityTier : '')) : '';
            volume._iconName = cloudProviderName ? 'volume-' + volume._diskTypeLetters : 'resource-volume';
            volume._tierDisplay = cloudProviderName ? providerDiskTypes[cloudProviderName][volume.providerVolumeType].display +
                ((volume.capacityTier && tieringPolicyNotNone) ? (' and ' + providerToObjectStorageName[cloudProviderName]) : '') : '';

            if (volume.moving) {
                volume._iconName = 'volume-move';
                volume._tierDisplay = 'Volume is moving';
            }
        }

        function initCloudSyncDisplay(volume, cloudSyncProperties) {
            if (cloudSyncProperties && cloudSyncProperties.isSynced) {
                const relationship = _.find(cloudSyncProperties.relationships, {volumeName: volume.name});

                if (relationship) {
                    volume._isCloudSynced = true;
                    volume._cloudSyncFailureMessages = calculateSyncFailureMessage(relationship, cloudSyncProperties.status);
                }
            }
        }

        resource.initVolumesForMonitoringDisplay = (cloudSyncProperties, volumes, cloudProviderName) => {
            _.forEach(volumes, (volume) => {
                initBasicVolumeDisplay(volume, cloudProviderName);
                initCloudSyncDisplay(volume, cloudSyncProperties);
            });

            return volumes;
        };

        resource.getSnapshots = function getSnapshots(nocHttp, pathPrefix, publicId, svmName, volumeName) {
            const url = pathPrefix + 'volumes/' + publicId + '/' + svmName + '/' + volumeName + '/snapshots';

            return nocHttp.get(url, 'retrieving Snapshots for volume ' + volumeName, nocHttp.LOADING)
                .then((snapshots) => {
                    _.forEach(snapshots, (snapshot) => {
                        snapshot.displayAccessTime = $filter('dateAndTimeNoSec')(snapshot.accessTime);
                    });

                    return snapshots;
                });
        };

        resource.postSnapshot = function postSnapshot(nocHttp, pathPrefix, publicId, svmName, volumeName, snapshotName) {
            const url = pathPrefix + 'volumes/' + publicId + '/' + svmName + '/' + volumeName + '/snapshot';

            return nocHttp.post({
                url,
                data: {
                    snapshotName
                }
            }, 'creating a Snapshot for volume ' + volumeName, nocHttp.LOADING);
        };

        resource.askForQuote = (nocHttp, pathPrefix, quoteRequest, actionName) => {
            return nocHttp.post({
                url: pathPrefix + 'volumes/quote',
                data: quoteRequest
            }, actionName, nocHttp.BLOCK);
        };

        resource.getUserApprovalForNewVolumeIfNeeded = (quoteResponse, cloudProviderName, diskType, isHA, volumeName) => {
            if (quoteResponse.autoVsaCapacityManagement) {
                return $q.when(quoteResponse);
            } else {
                const quoteResponseForView = _.assignIn(_.cloneDeep(quoteResponse), {
                    provider: cloudProviderName,
                    diskType,
                    isHA
                });
                if (isHA) quoteResponseForView.numOfDisks /= 2;

                return nocDialog.confirm('', 'assets/templates/dialogs/add-volume-approval-content.html', {
                    html: nocApprovalMessageBuilder.buildTitle(quoteResponseForView),
                    error: quoteResponseForView,
                    volumeName,
                    hideHint: false,
                    show: false
                })
                    .then(() => quoteResponse);
            }
        };

        return resource;
    });
})();
