(() => {
    'use strict';

    angular.module('noc').factory('nocEncryptionResourcesService', (nocUtil, $filter, Patterns) => {
        const encryptionService = {};

        encryptionService.getOccmCertificate = function (nocHttp) {
            return nocHttp.get('/occm/encryption/certificate', 'retrieving Cloud Manager certificate', nocHttp.LOADING)
                .then(function (certificate) {
                    encryptionService.formatCertificate(certificate);
                    return certificate;
                });
        };

        encryptionService.getKeyManagersCaCertificates = function (nocHttp) {
            return nocHttp.get('/occm/encryption/key-managers-ca-certificates',
                "retrieving key managers' CA certificates", nocHttp.LOADING)
                .then(function (keyManagersCaCertificates) {
                    _.forEach(keyManagersCaCertificates, function (keyManagerCaCertificate) {
                        encryptionService.formatCertificate(keyManagerCaCertificate);
                    });
                    return keyManagersCaCertificates;
                });
        };

        encryptionService.getKeyManagersCaCertificatesWithOptions = function (nocHttp) {
            return encryptionService.getKeyManagersCaCertificates(nocHttp)
                .then(function (keyManagersCaCertificates) {
                    return {
                        data: keyManagersCaCertificates,
                        select2options: {
                            formatResult: encryptionService.formatKeyManagersCaCertificate(keyManagersCaCertificates),
                            formatSelection: encryptionService.formatKeyManagersCaCertificate(keyManagersCaCertificates)
                        }
                    };
                });
        };

        encryptionService.formatCertificate = function(certificate){
            if (!certificate) return;
            //1. Calculate expiration state.
            certificate._expirationState = nocUtil.certificateExpirationState(certificate.endDate);

            //2. Add space after each comma to allow logical line-breaks.
            certificate.directIssuerName = _.replace(certificate.directIssuerName, /,/g, ', ');
            certificate.subjectName = _.replace(certificate.subjectName, /,/g, ', ');
        };

        encryptionService.stringifyKeyManagerCaCertificate = function (keyManagersCaCertificate) {
            var commonNameToEnd = keyManagersCaCertificate.subjectName.substring(keyManagersCaCertificate.subjectName.indexOf("CN="));
            var indexOfFirstComma = commonNameToEnd.indexOf(",");
            return commonNameToEnd.substring(3,indexOfFirstComma > 0 ? indexOfFirstComma : undefined);
        };

        encryptionService.formatKeyManagersCaCertificate = function (keyManagersCaCertificates) {
            return function (item) {
                if (!item.text) return "";
                var caCertificate = _.find(keyManagersCaCertificates, {publicId: item.text});
                return "<div class='select2-format-options clearfix' title='" + caCertificate.subjectName + ". " + caCertificate._expirationState.message + "'>" +
                    "<span>" + encryptionService.stringifyKeyManagerCaCertificate(caCertificate) + "</span>" +
                    "<div>" +
                    '<span class="' + caCertificate._expirationState.state + '">Expiration: ' + $filter('fullDate')(caCertificate.endDate) + '</span>' +
                    "</div>" +
                    "</div>";
            };
        };

        encryptionService.getKeyManagers = function(nocHttp){
            return nocHttp.get('/occm/encryption/key-managers', "retrieving key managers", nocHttp.LOADING);
        };

        encryptionService.getAwsEncryptionKeys = function (nocHttp, region, cloudProviderAccountId) {
            return nocHttp.get({url: "/vsa/metadata/aws-encryption-keys", params: {cloudProviderAccountId, region}}, "retrieving AWS encryption keys", nocHttp.LOADING)
                .then(function(awsEncryptionKeys){
                    var defaultKey = _.find(awsEncryptionKeys, {"default": true});
                    _.remove(awsEncryptionKeys, {"default": true});
                    if(!defaultKey) {
                        defaultKey = {
                            "alias": "Default AWS master key",
                            "keyId": "",
                            "status": "",
                            "validTo": null,
                            "origin": "",
                            "default": true,
                            "generated": true
                        };
                    }
                    awsEncryptionKeys.unshift(defaultKey);
                    return {"awsEncryptionKeys": awsEncryptionKeys, "awsEncryptionKey": defaultKey};
                });
        };

        encryptionService.formatAwsEncryptionKeyExpiration = function (key){
            var neverExpires = key.validTo === null;
            if(neverExpires || key.status !== "Enabled"){
                key.expiring = false;
                key.formattedDate = key.generated ? '' : 'None';
            } else {
                key.timeToExpirationInDays = Math.floor(moment(key.validTo).diff(moment(), "days", true));
                key.expiring = key.timeToExpirationInDays < 31;
                key.formattedDate = moment(key.validTo).format("MMM Do, YYYY");
            }
        };

        encryptionService.isKeyFromAnotherAccount = (key) => {
            const arnKeyRegex = new RegExp(Patterns['awsKmsArn'].regex);

            return arnKeyRegex.test(_.get(key, 'keyId'));
        };

        return encryptionService;
    });
})();
