(() => {
    'use strict';

    angular.module('noc').service('ManifestMapper', ($q) => {
        const manifestMapper = {
            initialized: false
        };

        //Returns only the version entries that has licenses we support
        manifestMapper.filterOutByLicenses = (permutations) => {
            return _(permutations)
                .map('license')
                .uniqBy('type')
                .value();
        };

        //Returns only the regions that a vsa can be created on
        manifestMapper.filterOutRegions = (permutations) => {
            return _(permutations)
                .map('region')
                .uniqBy('code')
                .value();
        };

        manifestMapper.isInitialized = function isInitialized() {
            return this.initialized;
        };

        manifestMapper.hasValidMapping = function hasValidMapping() {
            return this.permutations.length > 0;
        };

        manifestMapper.modes = {
            VSA_AWS: {name: 'VSA', url: '/vsa/metadata/permutations'},
            HA_AWS: {name: 'HA', url: '/aws/ha/metadata/permutations'},
            VSA_AZURE: {name: 'VSA_AZURE', url: 'azure/vsa/metadata/permutations'},
            HA_AZURE: {name: 'HA_AZURE', url: 'azure/ha/metadata/permutations'},
            VSA_GCP: {name: 'VSA_GCP', url: 'gcp/vsa/metadata/permutations'}
        };

        manifestMapper.initialize = function initialize(nocHttp, mode) {
            this.mode = this.modes[mode];
            if (_.isUndefined(this.mode)) throw new Error('ManifestMapper mode is undefined!');

            return this.setup(nocHttp);
        };

        manifestMapper.getMode = function getMode() {
            return this.mode.name;
        };

        manifestMapper.setup = function setup(nocHttp) {
            return nocHttp.get(this.mode.url + '?latest_only=true',
                '',
                nocHttp.LOADING)
                .then((permutations) => {
                    manifestMapper.permutations = permutations;
                    manifestMapper.globalRegions = manifestMapper.filterOutRegions(permutations);
                    manifestMapper.globalLicenses = manifestMapper.filterOutByLicenses(permutations);
                    manifestMapper.initialized = true;
                    if (!manifestMapper.hasValidMapping()) {
                        return $q.reject('Cannot Create a new Cloud Volumes ONTAP, Cannot find valid mapping in manifest. You can still discover an existing working environment.');
                    }
                });
        };

        manifestMapper.suicide = function suicide() {
            delete this.permutations;
            delete this.globalRegions;
            delete this.globalLicenses;
            this.initialized = false;
        };

        manifestMapper.getRegions = function getRegions() {
            return this.globalRegions;
        };

        manifestMapper.getLicenses = function getLicenses() {
            return this.globalLicenses;
        };

        manifestMapper.filterByRegionCode = function filterByRegionCode(permutations, regionCode) {
            return _.filter(permutations, {region: {code: regionCode}});
        };

        manifestMapper.filterByVersion = (permutations, version) => {
            if (!version) return [];

            return _.filter(permutations, (elem) => {
                return _.includes(elem.ontapVersion, version);
            });
        };

        manifestMapper.filterByLicense = function filterByLicense(permutations, license) {
            return _.filter(permutations, (perm) => {
                return _.includes(perm.license.type, license);
            });
        };

        manifestMapper.filterByLicenseType = function filterByLicenseType(permutations, licenseType) {
            return _.filter(permutations, {license: {type: licenseType}});
        };

        manifestMapper.filterByFeature = function filterByFeature(permutations, featureName) {
            return _.filter(permutations, (permutation) => {
                return _.includes(permutation.features, featureName);
            });
        };

        manifestMapper.filterOutByFeature = function filterOutByFeature(permutations, featureName) {
            return _.filter(permutations, (permutation) => {
                return !_.includes(permutation.features, featureName);
            });
        };

        manifestMapper.filterByRegionCodeAndVersion = function filterByRegionCodeAndVersion(permutations, regionCode, version) {
            return this.filterByVersion(this.filterByRegionCode(permutations, regionCode), version);
        };

        manifestMapper.filterByRegionCodeVersionAndLicenseType = function filterByRegionCodeVersionAndLicenseType(permutations, regionCode, version, licenseType) {
            return this.filterByLicenseType(this.filterByVersion(this.filterByRegionCode(permutations, regionCode), version), licenseType);
        };

        manifestMapper.filterByVersionAndFeature = function filterByVersionAndFeature(permutations, version, feature) {
            return this.filterByFeature(this.filterByVersion(permutations, version), feature);
        };

        manifestMapper.getAvailableVersionsByLicense = function getAvailableVersionsByLicense(regionCode, license) {
            return _(this.filterByLicense(this.filterByRegionCode(this.permutations, regionCode), license))
                .map('ontapVersion')
                .uniq()
                .value();
        };

        manifestMapper.getAvailableVersions = function getAvailableVersions(regionCode) {
            return _(this.filterByRegionCode(this.permutations, regionCode))
                .map('ontapVersion')
                .uniq()
                .value();
        };

        manifestMapper.getAvailableLicenses = function getAvailableLicenses(regionCode, version) {
            function initializeLicense(license) {
                license.availableInstances = manifestMapper.getAvailableInstances(regionCode, version, license.type);
                license._supportsOntapEncryption = _.intersection(license.availableInstances, manifestMapper.getEncryptionEnabledInstanceTypes(version)).length > 0;

                return license;
            }

            return _(this.filterByRegionCodeAndVersion(this.permutations, regionCode, version))
                .map('license')
                .uniqBy('type')
                .map(initializeLicense)
                .value();
        };

        manifestMapper.getAvailableInstances = function getAvailableInstances(regionCode, version, licenseType) {
            return _(this.filterOutByFeature(this.filterByRegionCodeVersionAndLicenseType(this.permutations, regionCode, version, licenseType), 'deprecated'))
                .map('instanceType')
                .uniq()
                .value();
        };

        manifestMapper.getDefaultInstance = function getDefaultInstance(regionCode, version, licenseType) {
            return _(this.filterOutByFeature(this.filterByRegionCodeVersionAndLicenseType(this.permutations, regionCode, version, licenseType), 'deprecated'))
                .filter('defaultInstance')
                .map('instanceType')
                .uniq()
                .head();
        };

        manifestMapper.getEncryptionEnabledInstanceTypes = function getEncryptionEnabledInstanceTypes(version) {
            return _(this.filterOutByFeature(this.filterByVersionAndFeature(this.permutations, version, 'encryption'), 'deprecated'))
                .map('instanceType')
                .uniq()
                .value();
        };

        manifestMapper.getNetworkUtilizationEnabledInstanceTypes = function getNetworkUtilizationEnabledInstanceTypes(version) {
            return _(this.filterOutByFeature(this.filterByVersionAndFeature(this.permutations, version, 'network-utilization'), 'deprecated'))
                .map('instanceType')
                .uniq()
                .value();
        };

        return manifestMapper;
    });
})();
