(() => {
    'use strict';

    angular.module('noc').factory('nocPoliciesService', () => {
        const policiesService = {};

        policiesService.SUPPORTED_POLICY_IMAGE = 'assets/images/wizard/snapmirror-policy.png';
        policiesService.UN_SUPPORTED_POLICY_IMAGE = 'assets/images/wizard/snapmirror-policy-disabled.png';
        policiesService.NO_RULES_POLICY_IMAGE = 'assets/images/wizard/snapmirror-policy-no-rules.png';

        policiesService.COMBINED_DEFAULT_RULES_DESC = 'Creates a version-flexible SnapMirror relationship which creates a new Snapshot copy and replicates all existing Snapshot copies to the destination volume';
        policiesService.SM_CREATED_DESC = 'Creates a new Snapshot copy and replicates it to the destination volume';
        policiesService.OTHER_RULES_DESC1 = 'Creates a SnapVault relationship which replicates Snapshot copies with the following labels to the destination volume';
        policiesService.OTHER_RULES_DESC2 = '(# of retained Snapshot copies in parenthesis)';

        policiesService.snapshotWarningText = 'No Snapshot copy matches the label of this policy';

        policiesService.policiesMap = {
            'DPDefault': {displayName: 'Mirror latest Snapshot copy only (legacy)', description: 'Typically used for a one-time data replication for ONTAP 8.2 and earlier'},
            'MirrorAllSnapshots': {displayName: 'Mirror', description: 'Typically used for disaster recovery'},
            'MirrorAndVault': {displayName: 'Mirror and Backup (1 year retention, weekly)', description: 'Configures disaster recovery and long-term retention of backups on the same destination volume'},
            'MirrorLatest': {displayName: 'Mirror latest Snapshot copy only', description: 'Typically used for a one-time data replication'},
            'XDPDefault': {displayName: 'Backup (1 year retention)', description: 'Configures long-term retention of backups'},
            'MirrorAndMonthlyBackup': {displayName: 'Mirror and Backup (1 month retention)', description: 'Configures disaster recovery and long-term retention of backups on the same destination volume'},
            'MonthlyBackup': {displayName: 'Backup (1 month retention)', description: 'Configures long-term retention of backups'},
            'MirrorAndYearlyBackup': {displayName: 'Mirror and Backup (1 year retention, monthly)', description: 'Configures disaster recovery and long-term retention of backups on the same destination volume'},
            'MirrorAnd7YearsBackup': {displayName: 'Mirror and Backup (7 year retention)', description: 'Configures disaster recovery and long-term retention of backups on the same destination volume'},
            'WeeklyBackup': {displayName: 'Backup (1 week retention)', description: 'Configures long-term retention of backups'},
            'Unified7year': {displayName: 'Mirror and Backup (7 year retention with more weekly)', description: 'Configures disaster recovery and long-term retention of backups on the same destination volume'}
        };

        policiesService.getPolicyInfo = (policyName) => {
            // Create a default object in case it is a user defined policy and doesn't appear in the map
            return policiesService.policiesMap[policyName] || {displayName: policyName, description: undefined};
        };

        policiesService.getPolicies = (nocHttp, sourceId, targetId) => {
            return nocHttp.get('/replication/metadata/snapmirror-policies/' + sourceId + '/' + targetId,
                'retrieving replication policies',
                nocHttp.LOADING)
                .then((policies) => {
                    _.forEach(policies, (policy) => {
                        policy._policyInfo = policiesService.getPolicyInfo(policy.name);

                        // Split the rules to two groups: default and other
                        _.defaults(policy, {_defaultRules: [], _otherRules: []});
                        _.assign(policy, _.groupBy(policy.rules, (rule) => {
                            return rule.snapMirrorLabel === 'sm_created' || rule.snapMirrorLabel === 'all_source_snapshots' ?
                                '_defaultRules' :
                                '_otherRules';
                        }));

                        policiesService.buildDefaultRulesDescription(policy);
                        policiesService.buildOtherRulesDescription(policy);
                        policy._policyImage = policy.isSupported ? policiesService.SUPPORTED_POLICY_IMAGE : policiesService.UN_SUPPORTED_POLICY_IMAGE;
                    });

                    return policies;
                });
        };

        policiesService.buildLabelsList = (policies) => {
            return _(policies)
                .flatMap((policy) => {
                    return _.map(policy._otherRules, 'snapMirrorLabel');
                })
                .uniq()
                .value();
        };

        policiesService.findExistingLabels = (rules, snapshotLabels) => {
            return _(rules)
                .flatMap('snapMirrorLabel')
                .uniq()
                .intersection(snapshotLabels)
                .size() > 0;
        };

        policiesService.snapshotWarningIfNeeded = (policy, snapshotLabels) => {
            if (_.isEmpty(policy._defaultRules)) {
                if (_.isEmpty(policy._otherRules)) {
                    return 'No rules defined for this policy. Data replication will not occur.';
                } else {
                    if (!policiesService.findExistingLabels(policy._otherRules, snapshotLabels)) {
                        return 'No Snapshot copies match the labels of this policy.';
                    }
                }
            }

            return undefined;
        };

        policiesService.getSnapshotLabels = (nocHttp, policies, sourceId, svmName, volumeName) => {
            const labelsList = policiesService.buildLabelsList(policies);

            return nocHttp.get({
                url: '/replication/metadata/snapshot-labels/' + sourceId + '/' + svmName + '/' + volumeName,
                params: {labels: labelsList}
            }, 'retrieving Snapshots labels', nocHttp.LOADING)
                .then((snapshotLabels) => {
                    _.forEach(policies, (policy) => {
                        policy._snapshotWarning = policiesService.snapshotWarningIfNeeded(policy, snapshotLabels);
                    });

                    return policies;
                });
        };

        policiesService.buildOtherRulesDescription = (policy) => {
            if (!policy._otherRules || policy._otherRules.length === 0) return;

            policy._otherRulesDescription1 = policiesService.OTHER_RULES_DESC1;
            policy._otherRulesDescription2 = policiesService.OTHER_RULES_DESC2;
            policy._labelAndRetentionSum = _.map(policy._otherRules, (rule) => {
                return rule.snapMirrorLabel + ' (' + rule.retention + ')';
            }).join(', ');
        };

        policiesService.buildDefaultRulesDescription = (policy) => {
            const defaultRules = policy._defaultRules;
            if (!defaultRules || defaultRules.length === 0) return;
            let defaultRulesDescription;

            if (defaultRules.length === 2) { // General setence for both default rules
                defaultRulesDescription = policiesService.COMBINED_DEFAULT_RULES_DESC;
            } else if (defaultRules[0].snapMirrorLabel === 'sm_created') {
                defaultRulesDescription = policiesService.SM_CREATED_DESC;
            }
            policy._defaultRulesDescription = defaultRulesDescription;
        };

        return policiesService;
    });
})();
