(() => {
    'use strict';

    angular.module('noc').service('nocPoller', ($http, $timeout, $rootScope, $state, $log) => {
        const poller = {};
        poller.delayedPolls = [];
        poller.nopoll = $state.params.nopoll;

        var paused = false;

        poller.pause = function(){
            paused = true;
        };

        poller.unpause = function(){
            paused = false;

            _.forEach(poller.delayedPolls, function(delayedPoll){
                delayedPoll();
            });
            poller.delayedPolls = [];
        };

        /*
         No poll simply slows down the polling by a significant number.
         A: It allows polling to still exist but be slower
         B: It doesn't suffer from any issue of making the initial call (keeping track of initial calls) even when nopoll is present
         */
        var determineFrequency = poller.determineFrequency = function(localFrequency){
            if(poller.nopoll) return 500000;

            return localFrequency;
        };

        poller.constructParams = function(urlOrParamsOrFn){
            var params = urlOrParamsOrFn;
            if(_.isString(urlOrParamsOrFn)){
                params = {
                    url: urlOrParamsOrFn,
                    method: "GET"
                };
            } else if (_.isFunction(urlOrParamsOrFn)) {
                params = urlOrParamsOrFn();
            }
            return params;
        };

        poller.start = function(urlOrParamsOrFn, frequency, successFn, errorFn, pollStartFn, restartOnChangedRequestStatus) {
            var stopped = false;
            var timeoutPromise;

            var doIfNotStopped = function(doFn){
                if(stopped || paused) return;
                doFn.apply(this, _.tail(arguments));
            };

            const triggerIfNotStopped = (doFn) => {
                if (stopped) return;

                if (paused) {
                    poller.delayedPolls.push(() => {
                        triggerIfNotStopped(doFn);
                    });
                } else {
                    doFn();
                }
            };

            var delayAndPoll = function () {
                //cancel previous timeouts - this could happen if requestStatusChanged event was handled while a request was pending.
                $timeout.cancel(timeoutPromise);

                timeoutPromise = $timeout(doPoll, determineFrequency(frequency));
            };

            var doPoll = function () {
                triggerIfNotStopped(function(){
                    if (_.isFunction(pollStartFn)) pollStartFn();
                    var params = poller.constructParams(urlOrParamsOrFn);

                    if(params){
                        $http(params)
                            .then(function (r) {
                                doIfNotStopped(successFn, r.data, r.status, r.headers, r.config);
                            })
                            .catch(function(r){
                                if(errorFn) doIfNotStopped(errorFn, r.data, r.status, r.headers, r.config);
                            })
                            .finally(function(){
                                delayAndPoll();
                            });
                    }else{
                        delayAndPoll();
                    }
                });
            };

            doPoll();

            var stopListeningToRequestStatusChangedEvents = _.noop;
            if (restartOnChangedRequestStatus) {
                stopListeningToRequestStatusChangedEvents = $rootScope.$on('requestStatusChanged', function () {

                    //cancel the future calls
                    $timeout.cancel(timeoutPromise);

                    //restart the poller
                    doPoll();

                    $log.debug(new Date() + "restarted " + (stopped ? 'stopped ' : '') + "polling " + JSON.stringify(poller.constructParams(urlOrParamsOrFn)));
                });
            }

            return function(){
                stopped = true;
                stopListeningToRequestStatusChangedEvents();
            };
        };

        return poller;
    });
})();
