(() => {
    'use strict';

    angular.module('noc').factory('nocUtil', ($log, $window, $filter) => {
        const nocUtil = {};

        nocUtil.sortByIp = function sortByIp(arrayOfIps, key) {
            return _.sortBy(arrayOfIps, (item) => {
                const split = _.split(item[key], '.', 4);

                return split[0] * Math.pow(256, 3) + split[1] * Math.pow(256, 2) + split[2] * 256 + split[3] * 1; //eslint-disable-line no-implicit-coercion
            });
        };

        nocUtil.fillObjectsByKeys = (arrayOfKeys, arrayOfObjects, key) => {
            if (!key) key = 'publicId';

            return _.reduce(arrayOfObjects, (array, element) => {
                if (_.includes(arrayOfKeys, element[key])) {
                    array.push(element);
                }

                return array;
            }, []);
        };

        nocUtil.floorToDecimalPoint = (number, decimalDigits) => {
            const power = Math.pow(10, decimalDigits);

            return Math.floor(number * power) / power;
        };

        nocUtil.mergeExistingObject = (oldCollection, newCollection, compareKeys) => {
            _.forEach(newCollection, (newObject, newObjectIndex) => {
                const searchObj = _.pick(newObject, compareKeys);

                let oldObject = _.find(oldCollection, searchObj);
                if (oldObject) {
                    oldObject = _.assignIn(oldObject, newCollection[newObjectIndex]);
                    newCollection[newObjectIndex] = oldObject;
                }
            });
        };

        var Unit = function (unit, denominator) {
            this.unit = unit;
            this.denominator = denominator;
        };

        Unit.prototype.minimize = function (capacity){
            return capacity.size * this.denominator;
        };

        // Byte is the base unit
        const units = [new Unit('Byte', 1), new Unit('KB', 1024), new Unit('MB', 1024 * 1024), new Unit('GB', 1024 * 1024 * 1024), new Unit('TB', 1024 * 1024 * 1024 * 1024)];
        const unitsMap = _.zipObject(_.map(units, 'unit'), units);

        nocUtil.normalizeCapacity = (capacity) => {
            if (!capacity || !_.isNumber(capacity.size)) return {size: 0, unit: 'KB'};

            if (capacity.size === 0) return capacity;

            const unit = unitsMap[capacity.unit];

            if (!unit) return {size: 0, unit: 'KB'};

            const minimumSize = unit.minimize(capacity);

            const appropriateUnit = _.findLast(units, (unit) => {
                return minimumSize >= unit.denominator;
            }) || unitsMap.Byte;

            return {size: minimumSize / appropriateUnit.denominator, unit: appropriateUnit.unit};
        };

        nocUtil.convertTo = (capacity, conversionUnit) => {
            if (!capacity || !_.isNumber(capacity.size)) return {size: 0, unit: conversionUnit};
            if (capacity.size === 0) return {size: capacity.size, unit: conversionUnit};
            const unit = unitsMap[capacity.unit];
            if (!unit) throw new Error(capacity.unit + ' units are not supported');
            const minimumSize = unit.minimize(capacity);
            const appropriateUnit = unitsMap[conversionUnit];
            if (!appropriateUnit) throw new Error(conversionUnit + ' units are not supported');

            return {size: minimumSize / appropriateUnit.denominator, unit: appropriateUnit.unit};
        };

        nocUtil.sumCapacityForDisks = (collection) => {
            const sum = {size: 0, unit: 'Byte'};
            _.forEach(collection, (item) => {
                const normalizedCapacity = nocUtil.convertTo(item.diskSize, 'Byte');
                sum.size += normalizedCapacity.size * item.numberOfDisks;
            });

            return nocUtil.normalizeCapacity(sum);
        };

        nocUtil.sumCapacityUsingField = function sumCapacityUsingField(collection, field) {
            return _.reduce(collection, (sum, item) => {
                sum.size += nocUtil.convertTo(item[field], 'Byte').size;

                return sum;
            }, {size: 0, unit: 'Byte'});
        };

        nocUtil.subtractCapacities = (a, b) => {
            if (!b || !b.size || !b.unit) return a;
            if (a.unit === b.unit) return {size: a.size - b.size, unit: a.unit};

            return {size: nocUtil.convertTo(a, 'Byte').size - nocUtil.convertTo(b, 'Byte').size, unit: 'Byte'};
        };

        //This function get searchString, object item and subset of fields.
        //Returns true if (searchString is empty) or (searchString is substring in the value of one of the fields of the object who's name is in searchFields)
        nocUtil.genericItemFilter = (searchString, item, searchFields) => {
            //Display everything if search query is empty.
            if (!searchString) return true;

            const searchRegexp = new RegExp(searchString, 'i');

            return _.some(item, (value, key) => {
                return _.includes(searchFields, key) && searchRegexp.test(value);
            });
        };

        nocUtil.filterVolumes = function filterVolumes(volumes, searchString) {
            //Filtering by:
            //  * volume name
            //  * volume tier, as shown in display
            //  * volume size, as shown in display.
            //  * volume used size, as shown in display.
            //  * volume disk type as displayed in tooltip on volume icon
            return $filter('filter')(volumes, (volume) => {
                return nocUtil.genericItemFilter(searchString, volume, ['name', '_displaySize', '_displayUsedSize', '_tierDisplay', '_diskTypeLetters', '_capacityTier']);
            });
        };

        nocUtil.getNgFilterFunc = (scope, collectionName, filteredCollectionName, searchStrName, filteredFields) => {
            const ngFilter = $filter('filter');

            return () => {
                scope[filteredCollectionName] = ngFilter(scope[collectionName], (item) => {
                    return nocUtil.genericItemFilter(scope[searchStrName], item, filteredFields);
                });
            };
        };

        nocUtil.formatSelect2 = (title, first, second, third, fourth, divClass) => {
            return "<div class='select2-format-options clearfix" + (divClass ? ' ' + divClass : '') + "' title='" + title + "'>" +
                '<span>' + first + '</span>' +
                (second ? ('<span>' + second + '</span>') : '') +
                (third ? ('<div>' +
                '<span>' + third + '</span>' +
                (fourth ? '<span>' + fourth + '</span>' : '') +
                '</div>') : '') +
                '</div>';
        };

        nocUtil.trimText = (text, size) => {
            return text.length > size ? text.substr(0, size - 3) + '...' : text;
        };

        nocUtil.isMessageable = (scope) => {
            return _.isFunction(scope.clearMessages);
        };

        nocUtil.logClientDetails = () => {
            $log.log('Client attemps login. navigator object is: ' + JSON.stringify($window.navigator,
                ['appCodeName', 'appName', 'appVersion', 'cookieEnabled', 'doNotTrack', 'hardwareConcurrency', 'language', 'maxTouchPoints', 'onLine', 'platform', 'product', 'productSub', 'userAgent', 'vendor', 'vendorSub']
            ));
            $log.log('Screen resolution: ' + $window.screen.width + ' x ' + $window.screen.height);
        };

        //This function is necessary to test certificateExpirationState
        nocUtil.getCurrentTime = () => {
            return moment();
        };

        nocUtil.certificateExpirationState = (expirationDate, startDate, warningPeriod) => {
            warningPeriod = _.isUndefined(warningPeriod) ? moment.duration(1, 'years') : moment.duration(warningPeriod, 'days'); //1 year is default
            const now = nocUtil.getCurrentTime();
            expirationDate = moment(expirationDate);
            startDate = moment(startDate);
            if (expirationDate.isBefore(now)) {
                return {state: 'certificate-expired', message: 'Certificate expired.'};
            } else if (expirationDate.diff(now) < warningPeriod) {
                return {state: 'certificate-warning', message: 'Certificate will expire in ' + moment.duration(expirationDate.diff(now)).humanize() + '.'};
            } else if (startDate.isAfter(now)) {
                return {state: 'future-activation', message: 'Certificate will be activated on ' + startDate};
            } else {
                return {state: 'certificate-valid-long', message: ''};
            }
        };

        nocUtil.workingEnvironmentTypeToRequestPathPrefix = (type, provider) => {
            switch (type) { //eslint-disable-line default-case
                case 'VSA':
                    if (provider === 'Azure') return '/azure/vsa/';
                    if (provider === 'GCP') return '/gcp/vsa/';
                    else return '/vsa/';
                case 'HA':
                    if (provider === 'Azure') return '/azure/ha/';
                    else return '/aws/ha/';
                case 'ON_PREM':
                    return '/onprem/';
                case 'NPS':
                    return '/onprem/';
            }
        };

        nocUtil.splitDataInto2Lists = (data) => {
            const lists = _.chunk(data, _.ceil(data.length / 2));

            return {leftList: lists[0], rightList: lists[1]};
        };

        nocUtil.concatArrayPropertyData = function concatArrayPropertyData(array, property) {
            let propertiesData = _.map(array, property);
            if (propertiesData.length === 1) {
                return propertiesData[0];
            }

            //There can be a case in HA where one of the nodes is down - so in order not to return weird data - the undefined will be changed to undefined text --> only if the second node has data
            if (!_.every(propertiesData, '')) {
                propertiesData = _.map(propertiesData, (property) => {
                    return property || 'not available';
                });
            }

            return propertiesData.join(', ');
        };

        nocUtil.normalizeStatus = (status) => {
            return _(status || '').toLower()
                .replace(/_/g, '-');
        };

        nocUtil.getOccmInstanceAws = function getOccmInstanceAws(workingEnvironment) {
            if (!workingEnvironment.awsProperties) {
                throw new Error('Cannot get Cloud Manager instance without awsProperties');
            }

            return _.find(workingEnvironment.awsProperties.instances, (instance) => {
                //Tag 'CloudONTAP.serial-number' is defined only on VSA\HA nodes, and filters away data-broker instances.
                return _.get(instance, ['tags', 'CloudONTAP.serial-number']) && instance.tags.WorkingEnvironmentId === workingEnvironment.publicId; //eslint-disable-line lodash/path-style
            });
        };

        nocUtil.dontOverrideObjectWithNull = function dontOverrideObjectWithNull(oldValue, newValue) {
            return newValue || oldValue;
        };

        nocUtil.getSystemTags = () => {
            return ['Tool', 'Type', 'InstanceId', 'Name', 'WorkingEnvironment', 'WorkingEnvironmentId', 'CloudONTAP.serial-number',
                'CloudONTAP.encryption', 'CloudONTAP.failover-mode', 'CloudONTAP.partner-serial-number', 'CloudONTAP.version'];
        };

        return nocUtil;
    });
})();
