describe('aggregatesResource', () => {
    'use strict';
    var j;
    beforeEach(() => {
        j = magnifispec.setup();
    });
    let aggregatesResource;

    beforeEach(inject((_aggregatesResource_) => {
        aggregatesResource = _aggregatesResource_;
    }));

    describe('getUrlForGetAggregatesApi', () => {
        it('should return the right URL for VSA_Azure', () => {
            const workingEnvironment = {
                publicId: 1,
                _typeAndProvider: 'VSA_Azure',
                _pathPrefix: j.nocUtil.workingEnvironmentTypeToRequestPathPrefix('VSA', 'Azure')
            };
            expect(aggregatesResource.getUrlForGetAggregatesApi(workingEnvironment)).toEqual('/azure/vsa/aggregates/1');
        });
        it('should return the right URL for VSA_AWA', () => {
            const workingEnvironment = {
                publicId: 1,
                _typeAndProvider: 'VSA_AWA',
                _pathPrefix: j.nocUtil.workingEnvironmentTypeToRequestPathPrefix('VSA', 'AWS')
            };
            expect(aggregatesResource.getUrlForGetAggregatesApi(workingEnvironment)).toEqual('/vsa/aggregates?workingEnvironmentId=1');
        });
        it('should return the right URL for HA', () => {
            const workingEnvironment = {
                publicId: 1,
                _typeAndProvider: 'HA',
                _pathPrefix: j.nocUtil.workingEnvironmentTypeToRequestPathPrefix('HA', 'AWS')
            };
            expect(aggregatesResource.getUrlForGetAggregatesApi(workingEnvironment)).toEqual('/aws/ha/aggregates?workingEnvironmentId=1');
        });
        it('should return the right URL for ON_PREM', () => {
            const workingEnvironment = {
                publicId: 1,
                _typeAndProvider: 'ON_PREM',
                _pathPrefix: j.nocUtil.workingEnvironmentTypeToRequestPathPrefix('ON_PREM')
            };
            expect(aggregatesResource.getUrlForGetAggregatesApi(workingEnvironment)).toEqual('/onprem/aggregates?workingEnvironmentId=1');
        });
    });

    describe('initializeAggregate', () => {
        it('should initialize the aggregate correctly', () => {
            const aggr = {
                availableCapacity: {size: 524288000, unit: 'Byte'},
                totalCapacity: {size: 686130463, unit: 'Byte'},
                providerVolumes: [{diskType: 'gp2'}, {diskType: 'gp2'}],
                usedCapacity: {size: 161842463, unit: 'Byte'},
                disks: ['disk1', 'disk2'],
                volumes: ['volume1', 'volume2', 'volume3']
            };
            aggregatesResource.initializeAggregate(aggr);

            expect(aggr).toEqual({
                availableCapacity: {size: 524288000, unit: 'Byte'},
                totalCapacity: {size: 686130463, unit: 'Byte'},
                providerVolumes: [{diskType: 'gp2'}, {diskType: 'gp2'}],
                usedCapacity: {size: 161842463, unit: 'Byte'},
                disks: ['disk1', 'disk2'],
                volumes: ['volume1', 'volume2', 'volume3'],
                _normalizedAvailableCapacity: {size: 500, unit: 'MB'},
                _normalizedTotalCapacity: Object({size: 654.3450002670288, unit: 'MB'}),
                _displayCapacity: '654.35 MB',
                _displayUsedCapacity: '154.35 MB',
                _numOfDisks: 2,
                _numOfVolumes: 3,
                _providerVolumeType: 'gp2',
                _diskAllocatedWidth: 240,
                _diskUsedWidth: 56.61050370824302,
                _tierWidth: 0
            });
        });

        it('should initialize correctly aggregate with tier', () => {
            const aggr = {
                providerVolumes: [{diskType: 'gp2'}, {diskType: 'gp2'}],
                capacityTier: 'S3',
                capacityTierUsed: {size: 129446707, unit: 'Byte'},
                disks: [],
                volumes: []
            };

            aggregatesResource.initializeAggregate(aggr);

            expect(aggr).toEqual(jasmine.objectContaining({
                _providerVolumeType: 'gp2',
                _displayUsedCapacityTier: '123.45 MB'
            }));
        });

        it('should initialize correctly onPrem aggregate', () => {
            const aggr = {
                availableCapacity: {size: 524288000, unit: 'Byte'},
                name: 'aggr1',
                state: 'online',
                totalCapacity: {size: 1024, unit: 'GB'}
            };

            aggregatesResource.initializeAggregate(aggr);

            expect(aggr).toEqual({
                availableCapacity: {size: 524288000, unit: 'Byte'},
                name: 'aggr1',
                state: 'online',
                totalCapacity: {size: 1024, unit: 'GB'},
                _normalizedAvailableCapacity: {size: 500, unit: 'MB'},
                _normalizedTotalCapacity: Object({size: 1, unit: 'TB'}),
                _displayCapacity: '1 TB',
                _displayUsedCapacity: '0 KB',
                _numOfDisks: undefined,
                _numOfVolumes: undefined,
                _diskAllocatedWidth: 240,
                _diskUsedWidth: 0,
                _tierWidth: 0
            });
        });
    });

    describe('Start aggregates polling', () => {
        let stopFunction;
        beforeEach(() => {
            stopFunction = jest.fn();
            jest.spyOn(j.nocPoller, 'start').mockReturnValue(stopFunction);
            jest.spyOn(aggregatesResource, 'initializeAggregate').mockImplementation(() => {
            });
        });

        it('should start the poller with the correct parameters', () => {
            const workingEnvironment = {
                _pathPrefix: '/vsa/',
                publicId: 'the working environment',
                _cloudProviderName: 'AWS'
            };
            const successCallback = jest.fn();
            const errorCallback = jest.fn();
            aggregatesResource.startAggregatesPolling(workingEnvironment, successCallback, errorCallback);

            expect(j.nocPoller.start).toHaveBeenCalled();
            const args = mostRecentCall(j.nocPoller.start);

            expect(args[0]).toEqual({
                method: 'GET',
                url: '/vsa/aggregates?workingEnvironmentId=the working environment'
            });
            expect(args[1]).toEqual(1 * 60 * 1000);

            jest.spyOn(j.$rootScope, '$broadcast').mockImplementation(() => {
            });

            args[2]([
                {name: 'aggr1'},
                {name: 'aggr2'}
            ]);
            expect(successCallback).toHaveBeenCalledWith([
                {name: 'aggr1'},
                {name: 'aggr2'}
            ]);

            expect(aggregatesResource.initializeAggregate.mock.calls.length).toEqual(2);

            args[3]({message: 'error'});
            expect(errorCallback).toHaveBeenCalledWith({message: 'error'});
        });

        it('should start the poller with vsa-ha for HA WEs', () => {
            const workingEnvironment = {
                isHA: true,
                _pathPrefix: '/aws/ha/',
                publicId: 'the working environment',
                _cloudProviderName: 'AWS'
            };

            aggregatesResource.startAggregatesPolling(workingEnvironment);

            expect(j.nocPoller.start).toHaveBeenCalled();
            const args = mostRecentCall(j.nocPoller.start);
            expect(args[0]).toEqual({
                method: 'GET',
                url: '/aws/ha/aggregates?workingEnvironmentId=the working environment'
            });
            expect(args[1]).toEqual(1 * 60 * 1000);
        });
    });

    describe('getAggregates', () => {
        let workingEnvironment;
        describe('should work for VSA and initialize _providerVolumeType', () => {
            beforeEach(() => {
                jest.spyOn(aggregatesResource, 'getUrlForGetAggregatesApi').mockReturnValue('url');
                workingEnvironment = {
                    publicId: 'pId1',
                    workingEnvironmentType: 'VSA',
                    _typeAndProvider: 'VSA_Azure',
                    _pathPrefix: '/azure_vsa/',
                    name: 'weName'
                };
            });
            it('success case', () => {
                jest.spyOn(j.nocHttp, 'get').mockReturnValue(j.$q.when([{
                    name: 'aggr1',
                    providerVolumes: [{diskType: 'premium_lrs'}]
                }]));

                expect(aggregatesResource.getAggregates(j.nocHttp, workingEnvironment)).toBeResolvedWith([
                    {
                        name: 'aggr1',
                        providerVolumes: [{diskType: 'premium_lrs'}],
                        _providerVolumeType: 'premium_lrs'
                    }
                ]);
            });
            it('error case', () => {
                jest.spyOn(j.nocHttp, 'get').mockReturnValue(j.$q.reject(j.errorResponse));

                expect(aggregatesResource.getAggregates(j.nocHttp, workingEnvironment)).toBeRejectedWith(j.errorResponse);
            });
            afterEach(() => {
                expect(j.nocHttp.get).toHaveBeenCalledWith('url', 'retrieving aggregates for working environment weName', j.nocHttp.LOADING);
            });
        });

        it('should work for ON_PREM', () => {
            workingEnvironment = {
                publicId: 'pId1',
                workingEnvironmentType: 'ON_PREM',
                _typeAndProvider: 'ON_PREM',
                _pathPrefix: '/on_prem/',
                name: 'weName'
            };
            jest.spyOn(j.nocHttp, 'get').mockReturnValue(j.$q.when([{name: 'aggr1'}]));

            expect(aggregatesResource.getAggregates(j.nocHttp, workingEnvironment)).toBeResolvedWith([{name: 'aggr1'}]);
        });
    });

    describe('createAggregate', () => {
        const workingEnvironment = {_pathPrefix: '/vsa/'};
        it('should work upon success', () => {
            jest.spyOn(j.nocHttp, 'post').mockReturnValue(j.$q.when('API_RESPONSE'));
            expect(aggregatesResource.createAggregate(j.nocHttp, workingEnvironment, {a: 'b'}))
                .toBeResolvedWith('API_RESPONSE');
        });

        it('error case', () => {
            jest.spyOn(j.nocHttp, 'post').mockReturnValue(j.$q.reject(j.errorResponse));
            expect(aggregatesResource.createAggregate(j.nocHttp, workingEnvironment, {a: 'b'}))
                .toBeRejectedWith(j.errorResponse);
        });

        afterEach(() => {
            expect(j.nocHttp.post)
                .toHaveBeenCalledWith({url: '/vsa/aggregates', data: {a: 'b'}}, 'creating aggregate', j.nocHttp.BLOCK);
        });
    });

    describe('addDisksToAggregate', () => {
        const workingEnvironment = {
            publicId: 'id1',
            _pathPrefix: '/vsa/'
        };
        const aggregate = {name: 'aggr10'};
        it('should work upon success', () => {
            jest.spyOn(j.nocHttp, 'post').mockReturnValue(j.$q.when('API_RESPONSE'));
            expect(aggregatesResource.addDisksToAggregate(j.nocHttp, workingEnvironment, aggregate, {a: 'b'}))
                .toBeResolvedWith('API_RESPONSE');
        });

        it('error case', () => {
            jest.spyOn(j.nocHttp, 'post').mockReturnValue(j.$q.reject(j.errorResponse));
            expect(aggregatesResource.addDisksToAggregate(j.nocHttp, workingEnvironment, aggregate, {a: 'b'}))
                .toBeRejectedWith(j.errorResponse);
        });

        afterEach(() => {
            expect(j.nocHttp.post).toHaveBeenCalledWith({
                url: '/vsa/aggregates/id1/aggr10/disks',
                data: {a: 'b'}
            }, 'adding disks to aggregate aggr10', j.nocHttp.BLOCK);
        });
    });

    describe('deleteAggregate', () => {
        const workingEnvironment = {
            publicId: 'id1',
            _pathPrefix: '/vsa/'
        };
        const aggregate = {name: 'aggr10'};
        it('should work upon success', () => {
            jest.spyOn(j.nocHttp, 'delete').mockReturnValue(j.$q.when('API_RESPONSE'));
            expect(aggregatesResource.deleteAggregate(j.nocHttp, workingEnvironment, aggregate))
                .toBeResolvedWith('API_RESPONSE');
        });

        it('error case', () => {
            jest.spyOn(j.nocHttp, 'delete').mockReturnValue(j.$q.reject(j.errorResponse));
            expect(aggregatesResource.deleteAggregate(j.nocHttp, workingEnvironment, aggregate))
                .toBeRejectedWith(j.errorResponse);
        });

        afterEach(() => {
            expect(j.nocHttp.delete)
                .toHaveBeenCalledWith('/vsa/aggregates/id1/aggr10', 'deleting aggregate aggr10', j.nocHttp.BLOCK);
        });
    });
});
