describe("Replication Resource", function () {
    "use strict";

    var j;    beforeEach(() => {        j = magnifispec.setup();    });
    var replicationResource, constants;

    beforeEach(inject(function (_replicationResource_, Constants) {
        replicationResource = _replicationResource_;
        constants = Constants;
    }));

    describe("init relationships", function () {
        it("should put empty/default values in all the missing fields and update the relationship allowed operations for on state", function () {
            var relationship = {
                source: {workingEnvironmentStatus: "on"},
                destination: {workingEnvironmentStatus: "on"},
                policyType: "async_mirror"
            };
            replicationResource.initRelationships([relationship]);
            expect(relationship._lagDuration).toEqual({viewValue: "", sortValue: ""});
            expect(relationship._health).toEqual("Healthy");
            expect(relationship.lastTransferInfo._transferEnded).toEqual("");
            expect(relationship.lastTransferInfo._transferSize).toEqual("");
            expect(relationship.lastTransferInfo._transferDuration).toEqual("");
            expect(relationship._maxTransferRate).toEqual("Unlimited");
            expect(relationship._totalTransferTime).toEqual({viewValue: "", sortValue: undefined});
            expect(relationship._totalTransferSize).toEqual({viewValue: "", sortValue: 0});
            expect(relationship._newestSnapshotCreated).toEqual({viewValue: "", sortValue: undefined});
            expect(relationship._relationshipProgress).toEqual("");
            expect(relationship._schedule).toEqual(constants.emptyReplicationScheduleDescription);
            expect(relationship._volumeUsedSize).toEqual('0 KB');
            expect(relationship._volumeCapacityTier).toEqual('0 KB');
            expect(relationship._allowedRelationshipOperations).toEqual({
                "break-relationship": true,
                "resync-relationship": true,
                "reverse-relationship": true,
                "edit-relationship": true,
                "delete-relationship": true,
                'initialize-relationship': true,
                'policy-info': true,
                'update-snapmirror': true
            });
        });

        it("should format all fields correctly", function () {
            var relationship = {
                source: {workingEnvironmentStatus: "on"},
                destination: {workingEnvironmentStatus: "on"},
                policyType: "async_mirror",
                "lastTransferInfo": {
                    "transferType": "resync",
                    "transferSize": {
                        "size": 10,
                        "unit": "GB"
                    },
                    "transferDuration": {
                        "length": 10,
                        "unit": "SECONDS"
                    },
                    "transferEnded": 484397100,
                    "transferError": null
                },
                "totalTransferSize": {
                    "size": 50,
                    "unit": "GB"
                },
                "totalTransferTime": {
                    "length": 20,
                    "unit": "SECONDS"
                },
                "maxTransferRate": {
                    "size": 45,
                    "unit": "KB"
                },
                "newestSnapshotCreated": 981165906,
                "relationshipProgress": {
                    "size": 321,
                    "unit": "GB"
                },
                "schedule": "daily",
                "volumeUsedSize": {
                    "size": 222,
                    "unit": "MB"
                },
                volumeCapacityTier: {
                    size: 333,
                    unit: 'MB'
                }
            };
            replicationResource.initRelationships([relationship]);
            expect(relationship._lagDuration).toEqual({viewValue: "", sortValue: ""});
            expect(relationship._health).toEqual("Healthy");
            expect(relationship.lastTransferInfo._transferEnded).toEqual('May 8, 1985 01:45:00 pm');
            expect(relationship.lastTransferInfo._transferSize).toEqual('10 GB');
            expect(relationship.lastTransferInfo._transferDuration).toEqual('A few seconds');
            expect(relationship._maxTransferRate).toEqual('45 KB/s');
            expect(relationship._totalTransferTime).toEqual({viewValue: 'A few seconds', sortValue: 20});
            expect(relationship._totalTransferSize).toEqual({viewValue: '50 GB', sortValue: 53687091200});
            expect(relationship._newestSnapshotCreated).toEqual({
                viewValue: 'Feb 3, 2001 04:05:06 am',
                sortValue: 981165906
            });
            expect(relationship._relationshipProgress).toEqual('321 GB');
            expect(relationship._schedule).toEqual('daily');
            expect(relationship._volumeUsedSize).toEqual('222 MB');
            expect(relationship._volumeCapacityTier).toEqual('333 MB');
            expect(relationship._allowedRelationshipOperations).toEqual({
                "break-relationship": true,
                "resync-relationship": true,
                "reverse-relationship": true,
                "edit-relationship": true,
                "delete-relationship": true,
                'initialize-relationship': true,
                'policy-info': true,
                'update-snapmirror': true
            });
        });

        it("should not allow break-relationship and resync-relationship if the working environments are in the wrong state", function () {
            var relationship = {
                source: {workingEnvironmentStatus: "off"},
                destination: {workingEnvironmentStatus: "on"}
            };
            replicationResource.initRelationships([relationship]);
            expect(relationship._allowedRelationshipOperations).toEqual({
                "break-relationship": false,
                "resync-relationship": false,
                "reverse-relationship": false,
                "edit-relationship": false,
                "delete-relationship": false,
                'initialize-relationship': false,
                'policy-info': false,
                'update-snapmirror': false
            });
        });

        it("should show empty when no schedule but status of target is not on", function () {
            var relationship = {
                source: {workingEnvironmentStatus: "on"},
                destination: {workingEnvironmentStatus: "off"}
            };
            replicationResource.initRelationships([relationship]);
            expect(relationship._schedule).toEqual("");
        });

        it("should display lag duration, Total Transfer Time and Last Transfer Duration in sentence case", function () {
            var relationship = {
                source: {workingEnvironmentStatus: "on"},
                destination: {workingEnvironmentStatus: "on"},
                lagTime: {length: 123456, unit: "SECONDS"},
                lastTransferInfo: {transferDuration: {length: 10, unit: "SECONDS"}},
                totalTransferTime: {length: 6, unit: "SECONDS"}
            };
            replicationResource.initRelationships([relationship]);
            expect(relationship._lagDuration.viewValue).toEqual("A day");
            expect(relationship.lastTransferInfo._transferDuration).toEqual("A few seconds");
            expect(relationship._totalTransferTime.viewValue).toEqual("A few seconds");
        });
    });

    describe("getReplicationStatus", function () {
        var workingEnvironment;
        beforeEach(function () {
            workingEnvironment = {publicId: "1", name: "we1"};
            spyOn(replicationResource, "initRelationships").and.returnValue(j.$q.when("initializedReplicationStatus"));
        });
        describe("case where working environment is supplied", function () {
            it("should call the right API and resolve upon success with right values", function () {
                spyOn(j.nocHttp, "get").and.returnValue(j.$q.when("replicationStatus"));

                expect(replicationResource.getReplicationStatus(j.nocHttp, null, workingEnvironment)).toBeResolvedWith("initializedReplicationStatus");
                expect(replicationResource.initRelationships).toHaveBeenCalledWith("replicationStatus");
            });
            it("should call the right API and reject upon error with right values", function () {
                spyOn(j.nocHttp, "get").and.returnValue(j.$q.reject(j.errorResponse));

                expect(replicationResource.getReplicationStatus(j.nocHttp, null, workingEnvironment)).toBeRejectedWith(j.errorResponse);
                expect(replicationResource.initRelationships).not.toHaveBeenCalled();
            });

            afterEach(function () {
                expect(j.nocHttp.get).toHaveBeenCalledWith({
                    url: '/replication/status/1',
                    params: {},
                    headers: {
                        "X-Occm-Bypass-Cache": true
                    }
                }, 'retrieving replication status for working environment we1', j.nocHttp.LOADING);
            });
        });
        describe("case where tenantId is supplied", function () {
            it("should call the right API and resolve upon success with right values", function () {
                spyOn(j.nocHttp, "get").and.returnValue(j.$q.when("replicationStatus"));

                expect(replicationResource.getReplicationStatus(j.nocHttp, "tenant1")).toBeResolvedWith("initializedReplicationStatus");
                expect(replicationResource.initRelationships).toHaveBeenCalledWith("replicationStatus");
            });
            it("should call the right API and reject upon error with right values", function () {
                spyOn(j.nocHttp, "get").and.returnValue(j.$q.reject(j.errorResponse));

                expect(replicationResource.getReplicationStatus(j.nocHttp, "tenant1")).toBeRejectedWith(j.errorResponse);
                expect(replicationResource.initRelationships).not.toHaveBeenCalled();
            });

            afterEach(function () {
                expect(j.nocHttp.get).toHaveBeenCalledWith({
                    url: '/replication/status',
                    params: {tenantId: 'tenant1'},
                    headers: {
                        "X-Occm-Bypass-Cache": true
                    }
                }, 'retrieving replication status', j.nocHttp.LOADING);
            });
        });
    });

    describe("startReplicationStatusPolling", () => {
        it("should call nocPoller with the right parameters", () => {
            spyOn(j.nocPoller, "start").and.returnValue('return value');

            expect(replicationResource.startReplicationStatusPolling({publicId: 'id1'})).toEqual('return value');

            expect(j.nocPoller.start).toHaveBeenCalledWith(
                {method: "GET", url: "/replication/status/id1", params: {}},
                5 * 60 * 1000,
                jasmine.any(Function), jasmine.any(Function),
                null, true
            );

            const successFunction = j.nocPoller.start.calls.mostRecent().args[2];
            const errorFunction = j.nocPoller.start.calls.mostRecent().args[3];

            spyOn(replicationResource, 'initRelationships');
            spyOn(j.$rootScope, '$broadcast');
            successFunction('replicationStatusResponse');

            expect(replicationResource.initRelationships).toHaveBeenCalledWith('replicationStatusResponse');
            expect(j.$rootScope.$broadcast).toHaveBeenCalledWith(replicationResource.REPLICATION_STATUS_GET_EVENT, {
                workingEnvironment: {publicId: 'id1'},
                replicationStatusResponse: 'replicationStatusResponse'
            });

            errorFunction('errorResponse');
            expect(j.$rootScope.$broadcast).toHaveBeenCalledWith(replicationResource.REPLICATION_STATUS_GET_ERROR_EVENT, {
                workingEnvironment: {publicId: 'id1'},
                error: 'errorResponse'
            });
        });
    });

    describe("startReplicationStatusAllPolling", () => {
        it("should call nocPoller with the right parameters", () => {
            spyOn(j.nocPoller, "start").and.returnValue('return value');

            expect(replicationResource.startReplicationStatusAllPolling('tenantId1')).toEqual('return value');

            expect(j.nocPoller.start).toHaveBeenCalledWith(
                {method: "GET", url: "/replication/status", params: {tenantId: 'tenantId1'}},
                5 * 60 * 1000,
                jasmine.any(Function), jasmine.any(Function),
                null, true
            );

            const successFunction = j.nocPoller.start.calls.mostRecent().args[2];
            const errorFunction = j.nocPoller.start.calls.mostRecent().args[3];

            spyOn(replicationResource, 'initRelationships');
            spyOn(j.$rootScope, '$broadcast');
            successFunction('replicationStatusResponse', null, null, {params: {tenantId: 'tenantId1'}});

            expect(replicationResource.initRelationships).toHaveBeenCalledWith('replicationStatusResponse');
            expect(j.$rootScope.$broadcast).toHaveBeenCalledWith(replicationResource.REPLICATION_STATUS_GET_ALL_EVENT, {
                replicationStatusResponse: 'replicationStatusResponse',
                tenantId: 'tenantId1'
            });

            errorFunction('errorResponse', null, null, {params: {tenantId: 'tenantId1'}});
            expect(j.$rootScope.$broadcast).toHaveBeenCalledWith(replicationResource.REPLICATION_STATUS_GET_ALL_ERROR_EVENT, {
                error: 'errorResponse',
                tenantId: 'tenantId1'
            });
        });
    });

    describe("getInterclusterLifs", () => {
        it("should resolve with LIFS upon success", () => {
            spyOn(j.nocHttp, "get").and.returnValue(j.$q.when('LIFS_LIFS_LIFS'));
            expect(replicationResource.getInterclusterLifs(j.nocHttp, 'id1', 'id2')).toBeResolvedWith('LIFS_LIFS_LIFS');
        });

        it("error case", () => {
            spyOn(j.nocHttp, "get").and.returnValue(j.$q.reject(j.errorResponse));
            expect(replicationResource.getInterclusterLifs(j.nocHttp, 'id1', 'id2')).toBeRejectedWith(j.errorResponse);
        });

        afterEach(() => {
            expect(j.nocHttp.get).toHaveBeenCalledWith({
                url: "/replication/intercluster-lifs",
                params: {
                    "workingEnvironmentId": 'id1',
                    "peerWorkingEnvironmentId": 'id2'
                }
            });
        });
    });

    describe("getSchedules", () => {
        it("should resolve with schedules upon success", () => {
            spyOn(j.nocHttp, "get").and.returnValue(j.$q.when('SCHEDULES_RESPONSE'));
            expect(replicationResource.getSchedules(j.nocHttp, 'id1')).toBeResolvedWith('SCHEDULES_RESPONSE');
        });

        it("error case", () => {
            spyOn(j.nocHttp, "get").and.returnValue(j.$q.reject(j.errorResponse));
            expect(replicationResource.getSchedules(j.nocHttp, 'id1')).toBeRejectedWith(j.errorResponse);
        });

        afterEach(() => {
            expect(j.nocHttp.get).toHaveBeenCalledWith("/replication/schedules/id1",
                "retrieving schedules for target volume", j.nocHttp.LOADING
            );
        });
    });

    describe("updateRelationship", () => {
        it("should resolve upon success", () => {
            spyOn(j.nocHttp, "put").and.returnValue(j.$q.when('SUCCESS_RESPONSE'));
            expect(replicationResource.updateRelationship(j.nocHttp, {
                workingEnvironmentId: 'id1',
                svmName: 'svm1',
                volumeName: 'volume1'
            }, {newData: 'lalala'}, 'updating new data')).toBeResolvedWith('SUCCESS_RESPONSE');
        });

        it("error case", () => {
            spyOn(j.nocHttp, "put").and.returnValue(j.$q.reject(j.errorResponse));
            expect(replicationResource.updateRelationship(j.nocHttp, {
                workingEnvironmentId: 'id1',
                svmName: 'svm1',
                volumeName: 'volume1'
            }, {newData: 'lalala'}, 'updating new data')).toBeRejectedWith(j.errorResponse);
        });

        afterEach(() => {
            expect(j.nocHttp.put).toHaveBeenCalledWith({
                    url: "/replication/id1/svm1/volume1",
                    data: {newData: 'lalala'}
                }, 'updating new data', j.nocHttp.BLOCK
            );
        });
    });
});