describe('Working Environment Resource', () => {
    'use strict';
    var j;    beforeEach(() => {        j = magnifispec.setup();    });

    let workingEnvironmentResource;

    beforeEach(inject((_workingEnvironmentResource_) => {
        workingEnvironmentResource = _workingEnvironmentResource_;
    }));

    describe('VPCs', () => {
        const workingEnvironment = {
            _pathPrefix: '/vsa/',
            regionName: 'region1',
            publicId: 'we1',
            vpcId: 'vpc1',
            _cloudProviderName: 'AWS'
        };
        const vpcsUrl = '/vsa/working-environments/we1?fields=awsProperties';

        it('should make a get request and return the promise (test when promise succeeds)', () => {
            j.$httpBackend.expectGET(j.nocBaseUrl + vpcsUrl).respond({});

            const successFunction = jest.fn();
            workingEnvironmentResource.getVpcOfWorkingEnvironment(j.nocHttp, workingEnvironment)
                .then(successFunction);

            j.$httpBackend.flush();

            expect(successFunction).toHaveBeenCalled();
        });

        it('should make a get request and return the promise (failure)', () => {
            j.$httpBackend.expectGET(j.nocBaseUrl + vpcsUrl).respond(400, {});

            const failFunction = jest.fn();
            workingEnvironmentResource.getVpcOfWorkingEnvironment(j.nocHttp, workingEnvironment)
                .catch(failFunction);

            j.$httpBackend.flush();

            expect(failFunction).toHaveBeenCalled();
        });
    });

    describe('Instances', () => {
        describe('Start instances polling', () => {
            let stopFunction;
            beforeEach(() => {
                stopFunction = jest.fn();
                jest.spyOn(j.nocPoller, 'start').mockReturnValue(stopFunction);
            });

            it('should start the poller with the correct parameters', () => {
                jest.spyOn(workingEnvironmentResource, 'initInstances').mockImplementation(_.identity);
                const workingEnvironment = {
                    publicId: 'VsaWorkingEnvironment-1',
                    _pathPrefix: '/vsa/',
                    _cloudProviderName: 'AWS',
                    awsProperties: {
                        instances: [
                            {
                                name: 'instance 1',
                                privateIpAddress: '1.1.1.1',
                                isVsaInstance: true,
                                isOCCMInstance: false
                            },
                            {
                                name: 'instance 2',
                                privateIpAddress: '2.2.2.2',
                                isVsaInstance: false,
                                isOCCMInstance: false
                            }
                        ]
                    }
                };
                workingEnvironmentResource.startInstancesPolling(workingEnvironment);

                expect(j.nocPoller.start).toHaveBeenCalled();
                const args = mostRecentCall(j.nocPoller.start);

                expect(args[0]).toEqual({
                    method: 'GET',
                    url: '/vsa/working-environments/VsaWorkingEnvironment-1?fields=supportedFeatures,k8sProperties,awsProperties'
                });
                expect(args[1]).toEqual(5 * 60 * 1000);

                jest.spyOn(j.$rootScope, '$broadcast').mockImplementation(() => {});

                args[2](workingEnvironment);
                expect(j.$rootScope.$broadcast).toHaveBeenCalledWith(workingEnvironmentResource.INSTANCES_GET_EVENT, {
                    workingEnvironment,
                    instances: [
                        {name: 'instance 1', privateIpAddress: '1.1.1.1', isVsaInstance: true, isOCCMInstance: false},
                        {name: 'instance 2', privateIpAddress: '2.2.2.2', isVsaInstance: false, isOCCMInstance: false}
                    ]
                });
                expect(workingEnvironmentResource.initInstances).toHaveBeenCalled();

                args[3]({message: 'error'});
                expect(j.$rootScope.$broadcast).toHaveBeenCalledWith(workingEnvironmentResource.INSTANCES_GET_ERROR_EVENT, {
                    workingEnvironment,
                    error: {message: 'error'}
                });
            });

            it('should return a function which stops the poller when called', () => {
                const stopper = workingEnvironmentResource.startInstancesPolling({});
                stopper();

                expect(stopFunction).toHaveBeenCalled();
            });
        });
        describe('sortInstances', () => {
            it('should sort by ip work when there is just one type', () => {
                const sortInstances = workingEnvironmentResource.sortInstances([
                    {
                        isVsaInstance: false,
                        isOCCMInstance: false,
                        _isDataBroker: false,
                        privateIpAddress: '1.1.1.32'
                    },
                    {
                        isVsaInstance: false,
                        isOCCMInstance: false,
                        _isDataBroker: false,
                        privateIpAddress: '1.1.1.0'
                    },
                    {
                        isVsaInstance: false,
                        isOCCMInstance: false,
                        _isDataBroker: false,
                        privateIpAddress: '1.1.1.2'
                    }
                ]);

                expect(_.map(sortInstances, 'privateIpAddress')).toEqual(['1.1.1.0', '1.1.1.2', '1.1.1.32']);
            });

            it('should work with no instances', () => {
                expect(workingEnvironmentResource.sortInstances([])).toEqual([]);
            });

            it('should sort by group: OCCM, Cloud ONTAP, Data brokers and then the rest. Inside each group should sort by ip', () => {
                const sortInstances = workingEnvironmentResource.sortInstances([
                    {
                        isVsaInstance: false,
                        isOCCMInstance: false,
                        _isDataBroker: false,
                        privateIpAddress: '1.1.1.11'
                    },
                    {
                        isVsaInstance: false,
                        isOCCMInstance: false,
                        _isDataBroker: false,
                        privateIpAddress: '1.1.1.10'
                    },
                    {
                        isVsaInstance: true,
                        isOCCMInstance: false,
                        _isDataBroker: false,
                        privateIpAddress: '1.1.1.0'
                    },
                    {
                        isVsaInstance: false,
                        isOCCMInstance: false,
                        _isDataBroker: false,
                        privateIpAddress: '1.1.1.31'
                    },
                    {
                        isVsaInstance: false,
                        isOCCMInstance: false,
                        _isDataBroker: false,
                        privateIpAddress: '1.1.1.30'
                    },
                    {
                        isVsaInstance: false,
                        isOCCMInstance: true,
                        _isDataBroker: false,
                        privateIpAddress: '1.1.1.20'
                    },
                    {
                        isVsaInstance: false,
                        isOCCMInstance: false,
                        _isDataBroker: true,
                        privateIpAddress: '1.1.3.1'
                    }
                ]);

                expect(_.map(sortInstances, 'privateIpAddress')).toEqual(['1.1.1.20', '1.1.1.0', '1.1.3.1', '1.1.1.10', '1.1.1.11', '1.1.1.30', '1.1.1.31']);
            });
        });

        describe('initResources', () => {
            it('should initialize and return them sorted', () => {
                jest.spyOn(workingEnvironmentResource, 'sortInstances').mockImplementation(_.identity);
                const instances = [{
                    name: 'vsa1',
                    isVsaInstance: true,
                    isOCCMInstance: false,
                    privateIpAddress: '1.1.1.11'
                }, {
                    name: 'occm1',
                    isVsaInstance: false,
                    isOCCMInstance: true,
                    privateIpAddress: '1.1.1.10'
                }, {
                    name: 'generalInstance',
                    isVsaInstance: false,
                    isOCCMInstance: false,
                    privateIpAddress: '1.1.1.0'
                }, {
                    name: 'vsa1-ha-mediator',
                    isVsaInstance: true,
                    isOCCMInstance: false,
                    privateIpAddress: '1.1.1.31'
                }, {
                    name: 'vsa1-dataBroker-sdkf4skld',
                    isVsaInstance: false,
                    isOCCMInstance: false,
                    privateIpAddress: '1.1.1.30'
                }];

                expect(workingEnvironmentResource.initInstances(instances)).toEqual([{
                    name: 'vsa1',
                    isVsaInstance: true,
                    isOCCMInstance: false,
                    privateIpAddress: '1.1.1.11',
                    _isMediator: false,
                    _isDataBroker: false
                }, {
                    name: 'occm1',
                    isVsaInstance: false,
                    isOCCMInstance: true,
                    privateIpAddress: '1.1.1.10',
                    _isMediator: false,
                    _isDataBroker: false
                }, {
                    name: 'generalInstance',
                    isVsaInstance: false,
                    isOCCMInstance: false,
                    privateIpAddress: '1.1.1.0',
                    _isMediator: false,
                    _isDataBroker: false
                }, {
                    name: 'vsa1-ha-mediator',
                    isVsaInstance: true,
                    isOCCMInstance: false,
                    privateIpAddress: '1.1.1.31',
                    _isMediator: true,
                    _isDataBroker: false
                }, {
                    name: 'vsa1-dataBroker-sdkf4skld',
                    isVsaInstance: false,
                    isOCCMInstance: false,
                    privateIpAddress: '1.1.1.30',
                    _isMediator: false,
                    _isDataBroker: true
                }]);
            });
        });
    });
    //
    // describe('Working Environments:', () => {
    //     let stopFunction;
    //     beforeEach(inject((workingEnvironmentStatuses) => {
    //         stopFunction = jasmine.createSpy('stopFunction');
    //         spyOn(j.nocPoller, 'start').mockReturnValue(stopFunction);
    //         workingEnvironmentStatuses.stateInfoMap = {'on': {allowedOperations: {'turn-off': true}}};
    //         workingEnvironmentStatuses.getAllowedOperations = jasmine.createSpy('getAllowedOperations').mockImplementation((typeAndProvider, state) => {
    //             return 'mocked allowed operations list for type ' + typeAndProvider + ' and state ' + state;
    //         });
    //     }));
    //
    //     describe('Initializing Working Environments', () => {
    //         describe('general initialization', () => {
    //             it('should set the type long name of the working environment', () => {
    //                 const workingEnvironment = {workingEnvironmentType: 'VSA', _typeAndProvider: 'VSA_AWS'};
    //
    //                 workingEnvironmentResource.initWorkingEnvironment(workingEnvironment);
    //
    //                 expect(workingEnvironment._typeLongName).toEqual('Cloud Volumes ONTAP');
    //             });
    //
    //             describe('Enabled Schedules', () => {
    //                 it('should be false when no schedule is set', () => {
    //                     const we1 = {
    //                         schedules: [
    //                             {name: 'schedule1', enabled: false},
    //                             {name: 'schedule2', enabled: false},
    //                             {name: 'schedule3', enabled: false}
    //                         ]
    //                     };
    //                     workingEnvironmentResource.initWorkingEnvironment(we1);
    //
    //                     expect(we1._hasEnabledSchedules).toBe(false);
    //                 });
    //
    //                 it('should be true when a schedule is set', () => {
    //                     const we1 = {
    //                         schedules: [
    //                             {name: 'schedule1', enabled: false},
    //                             {name: 'schedule2', enabled: true},
    //                             {name: 'schedule3', enabled: false}
    //                         ]
    //                     };
    //                     workingEnvironmentResource.initWorkingEnvironment(we1);
    //
    //                     expect(we1._hasEnabledSchedules).toBe(true);
    //                 });
    //             });
    //         });
    //
    //         describe('initStatusField', () => {
    //             it('should prettify the status of the working environment', () => {
    //                 const workingEnvironment = {status: {status: 'NOT_FOUND'}, workingEnvironmentType: 'VSA'};
    //
    //                 workingEnvironmentResource.initWorkingEnvironment(workingEnvironment);
    //
    //                 expect(workingEnvironment.status.status).toEqual('not-found');
    //             });
    //
    //             it('should init the status to "unknown" when no other status available', () => {
    //                 const workingEnvironment = {workingEnvironmentType: 'VSA'};
    //
    //                 workingEnvironmentResource.initWorkingEnvironment(workingEnvironment);
    //
    //                 expect(workingEnvironment.status.status).toEqual('unknown');
    //             });
    //
    //             it('should clone a stateInfo from the stateInfo map', () => {
    //                 const workingEnvironment = {status: {status: 'on'}};
    //
    //                 workingEnvironmentResource.initWorkingEnvironment(workingEnvironment);
    //
    //                 expect(workingEnvironment._stateInfo).toEqual({allowedOperations: {'turn-off': true}});
    //             });
    //
    //             it('should set an empty object in case the status is bad', () => {
    //                 const workingEnvironment = {status: 'oldStatus'};
    //
    //                 workingEnvironmentResource.initWorkingEnvironment(workingEnvironment);
    //
    //                 expect(workingEnvironment._stateInfo).toEqual({});
    //             });
    //
    //             it('should bring appropriate allowedOperations', () => {
    //                 const workingEnvironment = {
    //                     status: {status: 'on'},
    //                     workingEnvironmentType: 'VSA',
    //                     cloudProviderName: 'Amazon'
    //                 };
    //
    //                 workingEnvironmentResource.initWorkingEnvironment(workingEnvironment);
    //
    //                 expect(workingEnvironment._allowedOperations).toEqual('mocked allowed operations list for type VSA_AWS and state on');
    //             });
    //         });
    //
    //         describe('initEncryptionProperties', () => {
    //             describe('encryption String', () => {
    //                 it('should set the right string for awsEncryption', () => {
    //                     const workingEnvironment = {
    //                         workingEnvironmentType: 'VSA',
    //                         encryptionProperties: {awsVolumeEncryption: true},
    //                         status: {status: 'on'}
    //                     };
    //                     workingEnvironmentResource.initWorkingEnvironment(workingEnvironment);
    //                     expect(workingEnvironment._encryptionType).toEqual('AWS');
    //                 });
    //
    //                 it('should set the right string for azureEncryption', () => {
    //                     const workingEnvironment = {
    //                         workingEnvironmentType: 'VSA',
    //                         encryptionProperties: {azureVolumeEncryption: true},
    //                         status: {status: 'on'}
    //                     };
    //                     workingEnvironmentResource.initWorkingEnvironment(workingEnvironment);
    //                     expect(workingEnvironment._encryptionType).toEqual('Azure');
    //                 });
    //
    //                 it('should set the right string for ontapEncryption', () => {
    //                     const workingEnvironment = {
    //                         workingEnvironmentType: 'VSA',
    //                         _typeAndProvider: 'VSA_AWS',
    //                         encryptionProperties: {ontapEncryption: true},
    //                         status: {status: 'off'}
    //                     };
    //                     workingEnvironmentResource.initWorkingEnvironment(workingEnvironment);
    //                     expect(workingEnvironment._encryptionType).toEqual('Cloud Volumes ONTAP');
    //                 });
    //
    //                 it('should set the right string for no encryption', () => {
    //                     const workingEnvironment = {
    //                         workingEnvironmentType: 'VSA',
    //                         encryptionProperties: {ontapEncryption: false, awsVolumeEncryption: false},
    //                         status: {status: 'on'}
    //                     };
    //                     workingEnvironmentResource.initWorkingEnvironment(workingEnvironment);
    //                     expect(workingEnvironment._encryptionType).toEqual('None');
    //                 });
    //
    //                 it('should set the right string for starting', () => {
    //                     const workingEnvironment = {
    //                         workingEnvironmentType: 'VSA',
    //                         _typeAndProvider: 'VSA_AWS',
    //                         encryptionProperties: {ontapEncryption: true},
    //                         status: 'starting'
    //                     };
    //                     workingEnvironmentResource.initWorkingEnvironment(workingEnvironment);
    //                     expect(workingEnvironment._encryptionType).toEqual('N/A');
    //                 });
    //
    //                 it('should set the right string for stopping', () => {
    //                     const workingEnvironment = {
    //                         workingEnvironmentType: 'VSA',
    //                         _typeAndProvider: 'VSA_AWS',
    //                         encryptionProperties: {awsVolumeEncryption: true},
    //                         status: 'stopping'
    //                     };
    //                     workingEnvironmentResource.initWorkingEnvironment(workingEnvironment);
    //                     expect(workingEnvironment._encryptionType).toEqual('N/A');
    //                 });
    //             });
    //
    //             it('should init awsEncryptionKey properly', inject((nocEncryptionResourcesService) => {
    //                 spyOn(nocEncryptionResourcesService, 'formatAwsEncryptionKeyExpiration');
    //                 const workingEnvironment = {
    //                     encryptionProperties: {
    //                         awsVolumeEncryption: true,
    //                         awsEncryptionKey: {
    //                             status: 'PendingImport'
    //                         }
    //                     }
    //                 };
    //
    //                 workingEnvironmentResource.initWorkingEnvironment(workingEnvironment);
    //
    //                 expect(workingEnvironment.encryptionProperties.awsEncryptionKey.status).toEqual('Pending Import');
    //                 expect(nocEncryptionResourcesService.formatAwsEncryptionKeyExpiration)
    //                     .toHaveBeenCalledWith(workingEnvironment.encryptionProperties.awsEncryptionKey);
    //                 expect(workingEnvironment._awsEncryptionKey).toEqual(workingEnvironment.encryptionProperties.awsEncryptionKey);
    //             }));
    //         });
    //
    //         describe('initAwsProperties', () => {
    //             it('should flatten awsProperties correctly', () => {
    //                 const awsProperties = {
    //                     regionName: 'us-east-1',
    //                     vpc: {
    //                         vpcId: 'vpc-1',
    //                         cidrBlock: '10.20.0.0/16'
    //                     },
    //                     availabilityZones: ['us-east-1b'],
    //                     instances: [{
    //                         instanceId: 'instanceId',
    //                         tags: {WorkingEnvironmentId: '1'}
    //                     }]
    //                 };
    //                 const workingEnvironment = {publicId: '1', awsProperties};
    //
    //                 workingEnvironmentResource.initWorkingEnvironment(workingEnvironment);
    //
    //                 expect(workingEnvironment.regionName).toEqual(awsProperties.regionName);
    //                 expect(workingEnvironment._regionName).toEqual('US East (N. Virginia)');
    //                 expect(workingEnvironment.vpcId).toEqual(awsProperties.vpc.vpcId);
    //                 expect(workingEnvironment.cidrBlock).toEqual(awsProperties.vpc.cidrBlock);
    //                 expect(workingEnvironment.instances).toEqual(awsProperties.instances);
    //                 expect(workingEnvironment.availabilityZones).toEqual(awsProperties.availabilityZones);
    //             });
    //         });
    //
    //         describe('initProviderProperties', () => {
    //             it('should init fields', () => {
    //                 const providerProperties = {
    //                     regionName: 'westus',
    //                     resourceGroup: {name: 'We4745rg', location: 'westus', tags: {}},
    //                     vnetCidr: '10.0.0.0/24',
    //                     tags: {},
    //                     subscriptionId: '1',
    //                     deploymentId: 'depId',
    //                     creationTime: 1495630750129,
    //                     instanceType: 'Standard_DS4_v2',
    //                     numOfNics: 6
    //                 };
    //                 const workingEnvironment = {publicId: '1', providerProperties};
    //
    //                 workingEnvironmentResource.initWorkingEnvironment(workingEnvironment);
    //
    //                 expect(workingEnvironment.regionName).toEqual(providerProperties.regionName);
    //                 expect(workingEnvironment._regionName).toEqual('West US (California)');
    //                 expect(workingEnvironment._instanceType).toEqual(providerProperties.instanceType);
    //             });
    //         });
    //
    //         describe('initReplicationProperties', () => {
    //             it('should flatten replicationProperties correctly', () => {
    //                 const replicationProperties = {
    //                     'peers': ['VsaWorkingEnvironment-id1'],
    //                     'replicationTargets': ['VsaWorkingEnvironment-id1']
    //                 };
    //                 const workingEnvironment = {
    //                     workingEnvironmentType: 'VSA',
    //                     status: {status: 'on'},
    //                     name: 'name1',
    //                     vpc: {vpcId: 'vpcId', cidrBlock: '10.20.0.0/16'},
    //                     replicationProperties
    //                 };
    //
    //                 workingEnvironmentResource.initWorkingEnvironment(workingEnvironment);
    //
    //                 _.forEach(replicationProperties, (value, key) => {
    //                     expect(workingEnvironment[key]).toEqual(value);
    //                 });
    //             });
    //         });
    //
    //         describe('initCloudSyncProperties', () => {
    //             let workingEnvironment;
    //
    //             beforeEach(() => {
    //                 workingEnvironment = {
    //                     cloudSyncProperties: {
    //                         isSynced: true,
    //                         status: {status: 'COMPLETE'},
    //                         relationships: [{relationshipStatus: 'DONE'}]
    //                     }
    //                 };
    //             });
    //
    //             describe('init _failedRelationships', () => {
    //                 it('should count the failed relationships', () => {
    //                     workingEnvironment.cloudSyncProperties.relationships = [{relationshipStatus: 'DONE'}, {relationshipStatus: 'FAILED'}, {relationshipStatus: 'LULU'}, {relationshipStatus: 'FAILED'}];
    //
    //                     workingEnvironmentResource.initWorkingEnvironment(workingEnvironment);
    //
    //                     expect(workingEnvironment.cloudSyncProperties._failedRelationships).toEqual(2);
    //                 });
    //
    //                 it('should be falsy when all relationships are good', () => {
    //                     workingEnvironmentResource.initWorkingEnvironment(workingEnvironment);
    //
    //                     expect(workingEnvironment.cloudSyncProperties._failedRelationships).toEqual(0);
    //                 });
    //             });
    //
    //             describe('init _displayAsFailed', () => {
    //                 it('should be truthy when sync id failed', () => {
    //                     workingEnvironment.cloudSyncProperties.status = {status: 'FAILED'};
    //
    //                     workingEnvironmentResource.initWorkingEnvironment(workingEnvironment);
    //
    //                     expect(workingEnvironment.cloudSyncProperties._displayAsFailed).toBeTruthy();
    //                 });
    //
    //                 it('should be truthy when relationship is failed', () => {
    //                     workingEnvironment.cloudSyncProperties.relationships = [{relationshipStatus: 'DONE'}, {relationshipStatus: 'FAILED'}];
    //
    //                     workingEnvironmentResource.initWorkingEnvironment(workingEnvironment);
    //
    //                     expect(workingEnvironment.cloudSyncProperties._displayAsFailed).toBeTruthy();
    //                 });
    //
    //                 it('should be falsy when everything is OK', () => {
    //                     workingEnvironmentResource.initWorkingEnvironment(workingEnvironment);
    //
    //                     expect(workingEnvironment.cloudSyncProperties._displayAsFailed).toBeFalsy();
    //                 });
    //
    //                 it('should be falsy when WE is not synced', () => {
    //                     workingEnvironment.cloudSyncProperties.isSynced = false;
    //
    //                     workingEnvironmentResource.initWorkingEnvironment(workingEnvironment);
    //
    //                     expect(workingEnvironment.cloudSyncProperties._displayAsFailed).toBeFalsy();
    //                 });
    //             });
    //
    //             it('should do nothing if no cloudSyncProperties', () => {
    //                 const workingEnvironment = {workingEnvironmentType: 'VSA'};
    //
    //                 workingEnvironmentResource.initWorkingEnvironment(workingEnvironment);
    //
    //                 expect(workingEnvironment.cloudSyncProperties).not.toBeDefined();
    //             });
    //         });
    //
    //         describe('initCapacityFeatures', () => {
    //             it('should init provider volumes', () => {
    //                 const we1 = {
    //                     cloudProviderName: 'Amazon',
    //                     capacityFeatures: {
    //                         providerVolumesType: [
    //                             {
    //                                 size: {size: 100, unit: 'GB'},
    //                                 supportedVolumeTypes: ['gp2'],
    //                                 maxDisksAllow: {numOfDisks: 6, reason: null}
    //                             }, {
    //                                 size: {size: 500, unit: 'GB'},
    //                                 supportedVolumeTypes: ['gp2', 'st1'],
    //                                 maxDisksAllow: {numOfDisks: 6, reason: null}
    //                             }, {
    //                                 size: {size: 1, unit: 'TB'},
    //                                 supportedVolumeTypes: ['gp2', 'st1'],
    //                                 maxDisksAllow: {numOfDisks: 6, reason: null}
    //                             }, {
    //                                 size: {size: 2, unit: 'TB'},
    //                                 supportedVolumeTypes: ['gp2', 'st1'],
    //                                 maxDisksAllow: {numOfDisks: 0, reason: 'license problem'}
    //                             }
    //                         ],
    //                         defaultProviderVolumeType: {
    //                             size: {size: 1, unit: 'TB'},
    //                             diskType: 'gp2'
    //                         },
    //                         supportedCapacityTiers: {
    //                             supportedCapacityTiersPerVolumeType: [
    //                                 {volumeType: 'standard', supportedCapacityTiers: []},
    //                                 {volumeType: 'io1', supportedCapacityTiers: []},
    //                                 {volumeType: 'gp2', supportedCapacityTiers: []},
    //                                 {volumeType: 'st1', supportedCapacityTiers: ['S3']},
    //                                 {volumeType: 'sc1', supportedCapacityTiers: []}
    //                             ],
    //                             capacityTiersDisableReasons: ['reason1', 'reason2']
    //                         }
    //                     }
    //                 };
    //
    //                 workingEnvironmentResource.initWorkingEnvironment(we1);
    //
    //                 expect(we1._providerVolumeTypes).toEqual({
    //                     gp2: jasmine.objectContaining({
    //                         name: 'gp2',
    //                         supportedSizes: [{
    //                             size: 100, unit: 'GB', _identifier: '100 GB',
    //                             maxDisksAllow: {numOfDisks: 6, reason: null}
    //                         }, {
    //                             size: 500, unit: 'GB', _identifier: '500 GB',
    //                             maxDisksAllow: {numOfDisks: 6, reason: null}
    //                         }, {
    //                             size: 1, unit: 'TB', _identifier: '1 TB',
    //                             maxDisksAllow: {numOfDisks: 6, reason: null}
    //                         }, {
    //                             size: 2, unit: 'TB', _identifier: '2 TB',
    //                             maxDisksAllow: {numOfDisks: 0, reason: 'license problem'}
    //                         }]
    //                     }),
    //                     st1: jasmine.objectContaining({
    //                         name: 'st1',
    //                         supportedSizes: [{
    //                             size: 500, unit: 'GB', _identifier: '500 GB',
    //                             maxDisksAllow: {numOfDisks: 6, reason: null}
    //                         }, {
    //                             size: 1, unit: 'TB', _identifier: '1 TB',
    //                             maxDisksAllow: {numOfDisks: 6, reason: null}
    //                         }, {
    //                             size: 2, unit: 'TB', _identifier: '2 TB',
    //                             maxDisksAllow: {numOfDisks: 0, reason: 'license problem'}
    //                         }]
    //                     })
    //                 });
    //             });
    //
    //             it('should not mix together two different working environments', () => {
    //                 const we1 = {
    //                     cloudProviderName: 'Amazon',
    //                     capacityFeatures: {
    //                         providerVolumesType: [{
    //                             size: {size: 1, unit: 'TB'},
    //                             supportedVolumeTypes: ['gp2', 'st1'],
    //                             maxDisksAllow: {numOfDisks: 6, reason: null}
    //                         }],
    //                         supportedCapacityTiers: {
    //                             supportedCapacityTiersPerVolumeType: [
    //                                 {volumeType: 'gp2', supportedCapacityTiers: []},
    //                                 {volumeType: 'st1', supportedCapacityTiers: ['S3']}
    //                             ],
    //                             capacityTiersDisableReasons: ['reason1', 'reason2'] // <-- main difference
    //                         }
    //                     }
    //                 };
    //                 const we2 = {
    //                     cloudProviderName: 'Amazon',
    //                     capacityFeatures: {
    //                         providerVolumesType: [{
    //                             size: {size: 1, unit: 'TB'},
    //                             supportedVolumeTypes: ['gp2', 'st1'],
    //                             maxDisksAllow: {numOfDisks: 6, reason: null}
    //                         }],
    //                         supportedCapacityTiers: {
    //                             supportedCapacityTiersPerVolumeType: [
    //                                 {volumeType: 'gp2', supportedCapacityTiers: []},
    //                                 {volumeType: 'st1', supportedCapacityTiers: ['S3']}
    //                             ],
    //                             capacityTiersDisableReasons: [] // <-- main difference
    //                         }
    //                     }
    //                 };
    //
    //                 workingEnvironmentResource.initWorkingEnvironment(we1);
    //                 workingEnvironmentResource.initWorkingEnvironment(we2);
    //
    //                 expect(we1._providerVolumeTypes['st1'].disabledMessage).toEqual('reason1, reason2');
    //             });
    //         });
    //
    //         describe('initializing working environments for the table view', () => {
    //             beforeEach(() => {
    //                 workingEnvironmentResource.workingEnvironments = [
    //                     {
    //                         _pathPrefix: '/vsa/',
    //                         publicId: 1,
    //                         name: 'we1',
    //                         status: 'on',
    //                         peers: [2, 3],
    //                         reservedSize: {size: 1443434, unit: 'Byte'},
    //                         workingEnvironmentType: 'VSA'
    //                     },
    //                     {
    //                         _pathPrefix: '/vsa/',
    //                         publicId: 2,
    //                         name: 'we2',
    //                         status: 'on',
    //                         peers: [3],
    //                         reservedSize: {size: 1443434, unit: 'Byte'},
    //                         workingEnvironmentType: 'VSA'
    //                     },
    //                     {
    //                         _pathPrefix: '/vsa/',
    //                         publicId: 3,
    //                         name: 'we3',
    //                         status: 'not-found',
    //                         peers: [],
    //                         reservedSize: {size: 1443434, unit: 'Byte'},
    //                         workingEnvironmentType: 'VSA'
    //                     }
    //                 ];
    //             });
    //             it('should calculate all the required fields for the table', () => {
    //                 const workingEnvironment = {
    //                     publicId: 1,
    //                     reservedSize: {size: 1443434, unit: 'Byte'},
    //                     peers: [2, 3],
    //                     regionName: 'us-west-1',
    //                     workingEnvironmentType: 'VSA',
    //                     _typeAndProvider: 'VSA_AWS'
    //                 };
    //
    //                 workingEnvironmentResource.initWorkingEnvironment(workingEnvironment);
    //
    //                 expect(workingEnvironment._normalizedReservedSize).toEqual('< 0.01 TB');
    //                 expect(workingEnvironment._peers).toEqual('we2, we3');
    //                 expect(workingEnvironment._regionName).toEqual('US West (N. California)');
    //             });
    //
    //             it('should get peers also for not on working environments when possible', () => {
    //                 const workingEnvironment = workingEnvironmentResource.workingEnvironments[2];
    //                 workingEnvironmentResource.initWorkingEnvironment(workingEnvironment);
    //                 expect(workingEnvironment._peers).toEqual('we1, we2');
    //             });
    //
    //             it('should put undefined in the __normalizedReservedSize if the reserved size is 0', () => {
    //                 const workingEnvironment = {
    //                     publicId: 1,
    //                     reservedSize: {size: 0, unit: 'Byte'},
    //                     workingEnvironmentType: 'VSA'
    //                 };
    //                 workingEnvironmentResource.initWorkingEnvironment(workingEnvironment);
    //                 expect(workingEnvironment._normalizedReservedSize).toBeUndefined();
    //             });
    //         });
    //     });
    //
    //     describe('deleteWorkingEnvironment', () => {
    //         it('should work upon success for vsa that is on', () => {
    //             spyOn(workingEnvironmentResource, 'updateWorkingEnvironmentStatus');
    //             spyOn(j.$state, 'go');
    //             const workingEnvironment = {
    //                 workingEnvironmentType: 'VSA',
    //                 publicId: 'id1',
    //                 _pathPrefix: 'vsa/',
    //                 status: {status: 'on'}
    //             };
    //             spyOn(j.nocHttp, 'delete').mockReturnValue(j.$q.when());
    //
    //             expect(workingEnvironmentResource.deleteWorkingEnvironment(j.nocHttp, workingEnvironment, false)).toBeResolved();
    //
    //             expect(j.nocHttp.delete).toHaveBeenCalledWith(
    //                 {
    //                     url: 'vsa/working-environments/id1',
    //                     params: {localDelete: false, forceDelete: false}
    //                 },
    //                 'deleting working environment',
    //                 j.nocHttp.BLOCK);
    //             expect(workingEnvironmentResource.updateWorkingEnvironmentStatus).toHaveBeenCalledWith(workingEnvironment, 'deleting');
    //         });
    //
    //         it('should work upon success for vsa in failed-to-create status and perform local delete', () => {
    //             spyOn(workingEnvironmentResource, 'updateWorkingEnvironmentStatus');
    //             spyOn(j.$state, 'go');
    //             const workingEnvironment = {
    //                 workingEnvironmentType: 'VSA',
    //                 publicId: 'id1',
    //                 _pathPrefix: 'vsa/',
    //                 status: {status: 'failed-to-create'}
    //             };
    //             spyOn(j.nocHttp, 'delete').mockReturnValue(j.$q.when());
    //
    //             expect(workingEnvironmentResource.deleteWorkingEnvironment(j.nocHttp, workingEnvironment, false)).toBeResolved();
    //
    //             expect(j.nocHttp.delete).toHaveBeenCalledWith(
    //                 {
    //                     url: 'vsa/working-environments/id1',
    //                     params: {localDelete: true, forceDelete: false}
    //                 },
    //                 'deleting working environment',
    //                 j.nocHttp.BLOCK);
    //             expect(workingEnvironmentResource.updateWorkingEnvironmentStatus).toHaveBeenCalledWith(workingEnvironment, 'deleting');
    //         });
    //
    //         it('should work upon success for on-prem', () => {
    //             spyOn(workingEnvironmentResource, 'updateWorkingEnvironmentStatus');
    //             const workingEnvironment = {
    //                 workingEnvironmentType: 'ON_PREM',
    //                 publicId: 'id1',
    //                 _pathPrefix: 'onprem/',
    //                 status: {status: 'on'}
    //             };
    //             spyOn(j.nocHttp, 'delete').mockReturnValue(j.$q.when());
    //
    //             expect(workingEnvironmentResource.deleteWorkingEnvironment(j.nocHttp, workingEnvironment, false)).toBeResolved();
    //
    //             expect(j.nocHttp.delete).toHaveBeenCalledWith(
    //                 {
    //                     url: 'onprem/working-environments/id1',
    //                     params: {localDelete: false, forceDelete: false}
    //                 },
    //                 'deleting working environment',
    //                 j.nocHttp.BLOCK);
    //             expect(workingEnvironmentResource.updateWorkingEnvironmentStatus).toHaveBeenCalledWith(workingEnvironment, 'deleted');
    //         });
    //
    //         it('should work upon error', () => {
    //             spyOn(workingEnvironmentResource, 'updateWorkingEnvironmentStatus');
    //             const workingEnvironment = {
    //                 workingEnvironmentType: 'ON_PREM',
    //                 publicId: 'id1',
    //                 _pathPrefix: 'onprem/',
    //                 status: {status: 'on'}
    //             };
    //             spyOn(j.nocHttp, 'delete').mockReturnValue(j.$q.reject());
    //
    //             expect(workingEnvironmentResource.deleteWorkingEnvironment(j.nocHttp, workingEnvironment, false)).toBeRejected();
    //
    //             expect(j.nocHttp.delete).toHaveBeenCalledWith(
    //                 {
    //                     url: 'onprem/working-environments/id1',
    //                     params: {localDelete: false, forceDelete: false}
    //                 },
    //                 'deleting working environment',
    //                 j.nocHttp.BLOCK);
    //             expect(workingEnvironmentResource.updateWorkingEnvironmentStatus).not.toHaveBeenCalledWith();
    //         });
    //
    //         it('should force delete when there are replications', () => {
    //             spyOn(workingEnvironmentResource, 'updateWorkingEnvironmentStatus');
    //             const workingEnvironment = {
    //                 workingEnvironmentType: 'ON_PREM',
    //                 publicId: 'id1',
    //                 _pathPrefix: 'vsa/',
    //                 status: {status: 'on'},
    //                 forceDeleteCheckbox: true
    //             };
    //             spyOn(j.nocHttp, 'delete').mockReturnValue(j.$q.reject());
    //
    //             expect(workingEnvironmentResource.deleteWorkingEnvironment(j.nocHttp, workingEnvironment, true)).toBeRejected();
    //
    //             expect(j.nocHttp.delete).toHaveBeenCalledWith(
    //                 {
    //                     url: 'vsa/working-environments/id1',
    //                     params: {localDelete: false, forceDelete: true}
    //                 },
    //                 'deleting working environment',
    //                 j.nocHttp.BLOCK);
    //             expect(workingEnvironmentResource.updateWorkingEnvironmentStatus).not.toHaveBeenCalledWith();
    //         });
    //     });
    //
    //     describe('Polling Working Environments Success Function', () => {
    //         it('should perform broadcast with the modified working environments', () => {
    //             spyOn(j.$rootScope, '$broadcast');
    //             spyOn(workingEnvironmentResource, 'processAggregateWorkingEnvironmentResponse').mockReturnValue('test');
    //
    //             const we1 = {name: 'we1'};
    //             const we2 = {name: 'we2'};
    //             const workingEnvironments = [we1, we2];
    //             workingEnvironmentResource.workingEnvironmentsPollingSuccessFunction(workingEnvironments, {}, {}, {});
    //
    //             expect(workingEnvironmentResource.processAggregateWorkingEnvironmentResponse.calls.count()).toEqual(1);
    //             expect(workingEnvironmentResource.processAggregateWorkingEnvironmentResponse.calls.argsFor(0)).toEqual([workingEnvironments]);
    //
    //             expect(j.$rootScope.$broadcast).toHaveBeenCalledWith(workingEnvironmentResource.WORKING_ENVIRONMENTS_GET_EVENT, {
    //                 workingEnvironments: 'test',
    //                 tenantId: undefined
    //             });
    //         });
    //
    //         it('should pass the tenant id', () => {
    //             spyOn(j.$rootScope, '$broadcast');
    //             spyOn(workingEnvironmentResource, 'processAggregateWorkingEnvironmentResponse').mockReturnValue('test');
    //
    //             const we1 = {name: 'we1'};
    //             const we2 = {name: 'we2'};
    //             const workingEnvironments = [we1, we2];
    //             workingEnvironmentResource.workingEnvironmentsPollingSuccessFunction(workingEnvironments, {}, {}, {params: {tenantId: '1'}});
    //
    //             expect(j.$rootScope.$broadcast).toHaveBeenCalledWith(workingEnvironmentResource.WORKING_ENVIRONMENTS_GET_EVENT, {
    //                 workingEnvironments: 'test',
    //                 tenantId: '1'
    //             });
    //         });
    //     });
    //
    //     describe('Polling Working Environments Error Function', () => {
    //         it('should perform broadcast with the error', () => {
    //             spyOn(j.$rootScope, '$broadcast');
    //             const error = {message: 'error'};
    //             workingEnvironmentResource.workingEnvironmentsPollingErrorFunction(error, {}, {}, {});
    //
    //             expect(j.$rootScope.$broadcast).toHaveBeenCalledWith(workingEnvironmentResource.WORKING_ENVIRONMENTS_GET_ERROR_EVENT, {
    //                 error,
    //                 tenantId: undefined
    //             });
    //         });
    //
    //         it('should perform broadcast with the tenantId', () => {
    //             spyOn(j.$rootScope, '$broadcast');
    //             const error = {message: 'error'};
    //             workingEnvironmentResource.workingEnvironmentsPollingErrorFunction(error, {}, {}, {params: {tenantId: '1'}});
    //
    //             expect(j.$rootScope.$broadcast).toHaveBeenCalledWith(workingEnvironmentResource.WORKING_ENVIRONMENTS_GET_ERROR_EVENT, {
    //                 error,
    //                 tenantId: '1'
    //             });
    //         });
    //     });
    //
    //     describe('Start polling', () => {
    //         it('should start the poller if it is the first call to start polling', () => {
    //             workingEnvironmentResource.startWorkingEnvironmentsPolling('tenantId');
    //             expect(j.nocPoller.start).toHaveBeenCalledWith({
    //                 method: 'GET',
    //                 url: '/working-environments',
    //                 params: {tenantId: 'tenantId'}
    //             }, 1 * 60 * 1000, workingEnvironmentResource.workingEnvironmentsPollingSuccessFunction, workingEnvironmentResource.workingEnvironmentsPollingErrorFunction, null, true);
    //         });
    //
    //         it('should start the poller if it is the first call to start polling (case with peering)', () => {
    //             workingEnvironmentResource.startWorkingEnvironmentsPolling('id', 'status');
    //             expect(j.nocPoller.start).toHaveBeenCalledWith({
    //                 method: 'GET',
    //                 url: '/working-environments',
    //                 params: {tenantId: 'id', fields: 'status'}
    //             }, 1 * 60 * 1000, workingEnvironmentResource.workingEnvironmentsPollingSuccessFunction, workingEnvironmentResource.workingEnvironmentsPollingErrorFunction, null, true);
    //         });
    //
    //         it('should not start the poller if it is not the first call to start polling', () => {
    //             workingEnvironmentResource.startWorkingEnvironmentsPolling();
    //             workingEnvironmentResource.startWorkingEnvironmentsPolling();
    //
    //             expect(j.nocPoller.start).toHaveBeenCalledWith({
    //                 method: 'GET',
    //                 url: '/working-environments',
    //                 params: {}
    //             }, 1 * 60 * 1000, workingEnvironmentResource.workingEnvironmentsPollingSuccessFunction, workingEnvironmentResource.workingEnvironmentsPollingErrorFunction, null, true);
    //
    //             expect(j.nocPoller.start.calls.count()).toEqual(1);
    //         });
    //     });
    //
    //     describe('Stop polling', () => {
    //         it('should not stop the poller if something else is listening', () => {
    //             workingEnvironmentResource.startWorkingEnvironmentsPolling();
    //             const stopper = workingEnvironmentResource.startWorkingEnvironmentsPolling();
    //             stopper();
    //             expect(stopFunction).not.toHaveBeenCalled();
    //         });
    //
    //         it('should stop the poller if nothing else is listening', () => {
    //             const stopper = workingEnvironmentResource.startWorkingEnvironmentsPolling();
    //             stopper();
    //             expect(stopFunction).toHaveBeenCalled();
    //         });
    //     });
    //
    //     describe('Start status polling', () => {
    //         it('should start the poller with the correct parameters', () => {
    //             const workingEnvironment = {
    //                 publicId: 'VsaWorkingEnvironment-1',
    //                 _pathPrefix: '/vsa/',
    //                 status: 'someStatus'
    //             };
    //             workingEnvironmentResource.startStatusPolling(workingEnvironment);
    //
    //             expect(j.nocPoller.start).toHaveBeenCalled();
    //             const args = mostRecentCall(j.nocPoller.start).args;
    //
    //             expect(args[0]).toEqual({
    //                 method: 'GET',
    //                 url: '/vsa/working-environments/VsaWorkingEnvironment-1',
    //                 params: {fields: 'status'}
    //             });
    //
    //             spyOn(j.$rootScope, '$broadcast');
    //
    //             args[2](workingEnvironment);
    //             expect(j.$rootScope.$broadcast).toHaveBeenCalledWith('VsaWorkingEnvironment-1' + workingEnvironmentResource.WE_STATUS_GET_EVENT, 'someStatus');
    //
    //             args[3]({message: 'error'}, 400);
    //             expect(j.$rootScope.$broadcast).toHaveBeenCalledWith('VsaWorkingEnvironment-1' + workingEnvironmentResource.WE_STATUS_ERROR_EVENT, {
    //                 error: {message: 'error'},
    //                 status: 400
    //             });
    //         });
    //
    //         it('should return a function which stops the poller when called', () => {
    //             const stopper = workingEnvironmentResource.startStatusPolling({});
    //             stopper();
    //
    //             expect(stopFunction).toHaveBeenCalled();
    //         });
    //     });
    // });
    //
    // describe('VSA', () => {
    //     describe('Start VSA', () => {
    //         let workingEnvironment;
    //         const vsaUrl = '/occm/api/vsa/working-environments/we1/start',
    //             haUrl = '/occm/api/aws/ha/working-environments/we1/start';
    //
    //         beforeEach(() => {
    //             workingEnvironment = {_pathPrefix: '/vsa/', publicId: 'we1'};
    //         });
    //
    //         it('should make a get request and return the promise (test when promise succeeds)', () => {
    //             j.$httpBackend.expectPOST('/occm/api/vsa/working-environments/we1/start').respond({});
    //
    //             const successFunction = jasmine.createSpy('successFunction');
    //             workingEnvironmentResource.startVsa(j.nocHttp, workingEnvironment, false).then(successFunction);
    //
    //             j.$httpBackend.flush();
    //
    //             expect(successFunction).toHaveBeenCalled();
    //         });
    //
    //         it('should make a get request and return the promise (failure)', () => {
    //             j.$httpBackend.expectPOST(vsaUrl).respond(400, {});
    //
    //             const failFunction = jasmine.createSpy('failFunction');
    //             workingEnvironmentResource.startVsa(j.nocHttp, workingEnvironment, false).catch(failFunction);
    //
    //             j.$httpBackend.flush();
    //
    //             expect(failFunction).toHaveBeenCalled();
    //         });
    //
    //         it('should change the api according to the we type', () => {
    //             workingEnvironment._pathPrefix = '/aws/ha/';
    //
    //             j.$httpBackend.expectPOST(haUrl).respond(400, {});
    //
    //             const failFunction = jasmine.createSpy('failFunction');
    //             workingEnvironmentResource.startVsa(j.nocHttp, workingEnvironment, false).catch(failFunction);
    //
    //             j.$httpBackend.flush();
    //
    //             expect(failFunction).toHaveBeenCalled();
    //         });
    //     });
    //
    //     describe('Stop VSA', () => {
    //         const workingEnvironment = {_pathPrefix: '/vsa/', publicId: 'we1'};
    //         const stopUrl = '/occm/api/vsa/working-environments/we1/stop?takeSnapshots=false';
    //
    //         it('should make a get request and return the promise (test when promise succeeds)', () => {
    //             j.$httpBackend.expectPOST(stopUrl).respond({});
    //
    //             const successFunction = jasmine.createSpy('successFunction');
    //             workingEnvironmentResource.stopVsa(j.nocHttp, workingEnvironment, false).then(successFunction);
    //
    //             j.$httpBackend.flush();
    //
    //             expect(successFunction).toHaveBeenCalled();
    //         });
    //
    //         it('should make a get request and return the promise (failure)', () => {
    //             j.$httpBackend.expectPOST(stopUrl).respond(400, {});
    //
    //             const failFunction = jasmine.createSpy('failFunction');
    //             workingEnvironmentResource.stopVsa(j.nocHttp, workingEnvironment, false).catch(failFunction);
    //
    //             j.$httpBackend.flush();
    //
    //             expect(failFunction).toHaveBeenCalled();
    //         });
    //     });
    //
    //     describe('update on off schedules', () => {
    //         const workingEnvironment = {_pathPrefix: '/vsa/', publicId: 'we1'};
    //         const updateSchedulesURL = '/occm/api/vsa/working-environments/we1/schedules';
    //         const schedules = ['schedule1', 'schedule2'];
    //
    //         it('should make a get request and return the promise (test when promise succeeds)', () => {
    //             j.$httpBackend.expectPUT(updateSchedulesURL, {schedules}).respond({});
    //
    //             const successFunction = jasmine.createSpy('successFunction');
    //             workingEnvironmentResource.updateSchedules(j.nocHttp, workingEnvironment, schedules).then(successFunction);
    //
    //             j.$httpBackend.flush();
    //
    //             expect(successFunction).toHaveBeenCalled();
    //         });
    //
    //         it('should make a get request and return the promise (failure)', () => {
    //             j.$httpBackend.expectPUT(updateSchedulesURL, {schedules}).respond(400, {});
    //
    //             const failFunction = jasmine.createSpy('failFunction');
    //             workingEnvironmentResource.updateSchedules(j.nocHttp, workingEnvironment, schedules).catch(failFunction);
    //
    //             j.$httpBackend.flush();
    //
    //             expect(failFunction).toHaveBeenCalled();
    //         });
    //     });
    //
    //     describe('getCost', () => {
    //         beforeEach(() => {
    //             spyOn(j.nocHttp, 'get').mockReturnValue(j.$q.when('fakeCostResponse'));
    //         });
    //
    //         it('should work upon success', () => {
    //             const workingEnvironment = {_pathPrefix: '/vsa/', publicId: 1};
    //
    //             expect(workingEnvironmentResource.getCost(j.nocHttp, workingEnvironment))
    //                 .toBeResolvedWith('fakeCostResponse');
    //
    //             expect(j.nocHttp.get).toHaveBeenCalledWith('/vsa/working-environments/1/cost-and-usage', 'retrieving cost', j.nocHttp.LOADING);
    //         });
    //     });
    //
    //     describe('getSavings', () => {
    //         beforeEach(() => {
    //             spyOn(j.nocHttp, 'get').mockReturnValue(j.$q.when('fakeSavingsResponse'));
    //         });
    //
    //         it('should work upon success for VSA with status ON', () => {
    //             const workingEnvironment = {_pathPrefix: '/vsa/', publicId: 1, status: {status: 'on'}};
    //
    //             expect(workingEnvironmentResource.getSavings(j.nocHttp, workingEnvironment))
    //                 .toBeResolvedWith('fakeSavingsResponse');
    //
    //             expect(j.nocHttp.get).toHaveBeenCalledWith('/vsa/working-environments/1/ontap-saving', 'retrieving savings', j.nocHttp.LOADING);
    //         });
    //
    //         it('should work upon success for VSA with status Degraded', () => {
    //             const workingEnvironment = {_pathPrefix: '/vsa/', publicId: 1, status: {status: 'degraded'}};
    //
    //             expect(workingEnvironmentResource.getSavings(j.nocHttp, workingEnvironment))
    //                 .toBeResolvedWith('fakeSavingsResponse');
    //
    //             expect(j.nocHttp.get).toHaveBeenCalledWith('/vsa/working-environments/1/ontap-saving', 'retrieving savings', j.nocHttp.LOADING);
    //         });
    //
    //         it('should work upon success for VSA with status not ON', () => {
    //             const workingEnvironment = {_pathPrefix: '/vsa/', publicId: 1, status: {status: 'off'}};
    //
    //             expect(workingEnvironmentResource.getSavings(j.nocHttp, workingEnvironment))
    //                 .toBeResolvedWith({isVsaAvailable: true});
    //         });
    //     });
    //
    //     describe('ransomware protection: activateSnapshot', () => {
    //         it('should work upon success', () => {
    //             spyOn(j.nocHttp, 'put').mockReturnValue(j.$q.when());
    //             const workingEnvironment = {_pathPrefix: '/vsa/', publicId: 1};
    //
    //             workingEnvironmentResource.activateSnapshot(j.nocHttp, workingEnvironment);
    //
    //             expect(j.nocHttp.put).toHaveBeenCalledWith('/vsa/working-environments/1/activate-snapshot-policy', 'activating snapshot policy');
    //         });
    //     });
    //
    //     describe('Start vsa polling', () => {
    //         let stopFunction;
    //         beforeEach(() => {
    //             stopFunction = jasmine.createSpy('stopFunction');
    //             spyOn(j.nocPoller, 'start').mockReturnValue(stopFunction);
    //         });
    //
    //         it('should start the poller with the correct parameters', () => {
    //             const workingEnvironment = {_pathPrefix: '/vsa/', _cloudProviderName: 'AWS', publicId: 'the_vsa'};
    //             spyOn(j.workingEnvironmentResource, 'initWorkingEnvironment');
    //
    //             workingEnvironmentResource.startVsaPolling(workingEnvironment);
    //
    //             expect(j.nocPoller.start).toHaveBeenCalled();
    //
    //             const args = mostRecentCall(j.nocPoller.start).args;
    //
    //             expect(args[0]).toEqual({
    //                 method: 'GET',
    //                 url: '/vsa/working-environments/the_vsa?fields=status,encryptionProperties,ontapClusterProperties.fields(upgradeVersions),activeActions,supportedFeatures,k8sProperties,fpolicyProperties,awsProperties'
    //             });
    //             expect(args[1]).toEqual(120000);
    //
    //             spyOn(j.$rootScope, '$broadcast');
    //
    //             const workingEnvironmentResponse = {
    //                 status: {status: 'good'},
    //                 vsa: 'vsa',
    //                 ontapClusterProperties: {serialNumber: 2, writingSpeedState: 'NORMAL'},
    //                 encryptionProperties: {ontapEncryption: true}
    //             };
    //             args[2](workingEnvironmentResponse);
    //             expect(j.workingEnvironmentResource.initWorkingEnvironment).toHaveBeenCalled();
    //
    //             args[3]({message: 'error'});
    //             expect(j.$rootScope.$broadcast).toHaveBeenCalledWith(workingEnvironmentResource.VSA_GET_ERROR_EVENT, {
    //                 workingEnvironment,
    //                 error: {message: 'error'}
    //             });
    //         });
    //
    //         it('should return a function which stops the poller when called', () => {
    //             const stopper = workingEnvironmentResource.startVsaPolling({});
    //             stopper();
    //
    //             expect(stopFunction).toHaveBeenCalled();
    //         });
    //     });
    // });
    //
    // describe('shouldShowTurnOnAction', () => {
    //     let we1, scope;
    //     beforeEach(() => {
    //         we1 = {
    //             name: 'we1',
    //             publicId: '1',
    //             workingEnvironmentType: 'VSA',
    //             _normalizedReservedSizeTB: {size: 0, unit: 'TB'}
    //         };
    //         scope = {selectedWorkingEnvironment: we1, actionsRequired: []};
    //     });
    //
    //     it('should return true if actionsRequired is undefined', () => {
    //         scope.actionsRequired = undefined;
    //         expect(workingEnvironmentResource.shouldShowTurnOnAction(scope)).toEqual(true);
    //     });
    //
    //     it('should return true if no actionsRequired are defined for this WE', () => {
    //         scope.actionsRequired = {};
    //         expect(workingEnvironmentResource.shouldShowTurnOnAction({}, we1)).toEqual(true);
    //     });
    //
    //     it('should return false if this WE has "licenseWasSuspended" action', () => {
    //         scope.actionsRequired = {
    //             1: [
    //                 {actionType: 'otherAction1'},
    //                 {actionType: 'licenseWasSuspended'},
    //                 {actionType: 'otherAction2'}
    //             ]
    //         };
    //         expect(workingEnvironmentResource.shouldShowTurnOnAction(scope)).toEqual(false);
    //     });
    //
    //     it('should return true if this WE has no "licenseWasSuspended" action', () => {
    //         scope.actionsRequired = {
    //             1: [
    //                 {actionType: 'otherAction1'},
    //                 {actionType: 'otherAction2'},
    //                 {actionType: 'otherAction3'}
    //             ]
    //         };
    //         expect(workingEnvironmentResource.shouldShowTurnOnAction(scope)).toEqual(true);
    //     });
    // });
    //
    // describe('changeOntapImage', () => {
    //     beforeEach(() => {
    //         j.$httpBackend.expectPOST(j.nocBaseUrl + '/vsa/working-environments/1/update-image', {}).respond({});
    //     });
    //     it('should use the path prefix from the working environment when exists', () => {
    //         const workingEnvironment = {_pathPrefix: '/vsa/', publicId: 1};
    //         workingEnvironmentResource.changeOntapImage(j.nocHttp, workingEnvironment, {});
    //         j.$httpBackend.flush();
    //     });
    //     it('should use the state params if the path prefix is empty', () => {
    //         const workingEnvironment = {publicId: 1};
    //         j.$state.params = {type: 'VSA', provider: 'AWS'};
    //         workingEnvironmentResource.changeOntapImage(j.nocHttp, workingEnvironment, {});
    //         j.$httpBackend.flush();
    //     });
    // });
    //
    // describe('changeLicenseAndInstanceType', () => {
    //     const workingEnvironment = {
    //         _pathPrefix: '/vsa/',
    //         publicId: 1,
    //         status: {status: 'ON'},
    //         _nameForInstance: 'instance'
    //     };
    //     const url = '/vsa/working-environments/1/license-instance-type';
    //
    //     describe('creating the right message', () => {
    //         beforeEach(() => {
    //             spyOn(j.nocHttp, 'put').mockReturnValue(j.$q.when());
    //         });
    //         it('should build the right message for change license', () => {
    //             workingEnvironmentResource.changeLicenseAndInstanceType(j.nocHttp, workingEnvironment, 'license1', null);
    //             expect(j.nocHttp.put).toHaveBeenCalledWith(
    //                 {
    //                     url,
    //                     data: {licenseType: 'license1', instanceType: null}
    //                 }, 'changing license type', j.nocHttp.BLOCK
    //             );
    //         });
    //
    //         it('should build the right message for change instance', () => {
    //             workingEnvironmentResource.changeLicenseAndInstanceType(j.nocHttp, workingEnvironment, null, 'instance1');
    //             expect(j.nocHttp.put).toHaveBeenCalledWith(
    //                 {
    //                     url,
    //                     data: {licenseType: null, instanceType: 'instance1'}
    //                 }, 'changing instance type', j.nocHttp.BLOCK
    //             );
    //         });
    //
    //         it('should build the right message for change license and instance', () => {
    //             workingEnvironmentResource.changeLicenseAndInstanceType(j.nocHttp, workingEnvironment, 'license2', 'instance2');
    //             expect(j.nocHttp.put).toHaveBeenCalledWith(
    //                 {
    //                     url,
    //                     data: {licenseType: 'license2', instanceType: 'instance2'}
    //                 }, 'changing license and instance type', j.nocHttp.BLOCK);
    //         });
    //
    //         it('should change the working environment status to updating if its not OFF', () => {
    //             workingEnvironmentResource.changeLicenseAndInstanceType(j.nocHttp, workingEnvironment, 'license2', 'instance2');
    //             j.$timeout.flush();
    //             expect(workingEnvironment.status.status).toEqual('updating');
    //         });
    //
    //         it('should not change the working environment status if it is OFF', () => {
    //             workingEnvironment.status.status = 'off';
    //             workingEnvironmentResource.changeLicenseAndInstanceType(j.nocHttp, workingEnvironment, 'license2', 'instance2');
    //             j.$timeout.flush();
    //             expect(workingEnvironment.status.status).toEqual('off');
    //         });
    //     });
    // });
    //
    // describe('Enable capacity tiering', () => {
    //     let workingEnvironment;
    //     const vsaUrl = '/occm/api/vsa/working-environments/we1/enable-capacity-tiering',
    //         haUrl = '/occm/api/aws/ha/working-environments/we1/enable-capacity-tiering';
    //
    //     beforeEach(() => {
    //         workingEnvironment = {_pathPrefix: '/vsa/', publicId: 'we1'};
    //     });
    //
    //     it('should make a get request and return the promise (test when promise succeeds)', () => {
    //         j.$httpBackend.expectPOST('/occm/api/vsa/working-environments/we1/enable-capacity-tiering').respond({});
    //
    //         const successFunction = jasmine.createSpy('successFunction');
    //         workingEnvironmentResource.configureFabricPool(j.nocHttp, workingEnvironment).then(successFunction);
    //         j.$httpBackend.flush();
    //
    //         expect(successFunction).toHaveBeenCalled();
    //     });
    //
    //     it('should make a get request and return the promise (failure)', () => {
    //         j.$httpBackend.expectPOST(vsaUrl).respond(400, {});
    //
    //         const failFunction = jasmine.createSpy('failFunction');
    //         workingEnvironmentResource.configureFabricPool(j.nocHttp, workingEnvironment).catch(failFunction);
    //         j.$httpBackend.flush();
    //
    //         expect(failFunction).toHaveBeenCalled();
    //     });
    //
    //     it('should change the api according to the we type', () => {
    //         workingEnvironment._pathPrefix = '/aws/ha/';
    //         j.$httpBackend.expectPOST(haUrl).respond(400, {});
    //
    //         const failFunction = jasmine.createSpy('failFunction');
    //         workingEnvironmentResource.configureFabricPool(j.nocHttp, workingEnvironment).catch(failFunction);
    //         j.$httpBackend.flush();
    //
    //         expect(failFunction).toHaveBeenCalled();
    //     });
    // });
});
