describe('Manifest Mapper', () => {
    'use strict';
    var j;    beforeEach(() => {        j = magnifispec.setup();    });
    let manifestMapper, permutations;

    beforeEach(inject((_ManifestMapper_) => {
        manifestMapper = _ManifestMapper_;
        permutations = magnifispec.getMockedResponses().awsPermutations;
    }));

    describe('initialization and end of life', () => {
        it('should initialize with permutations', () => {
            magnifispec.initializeManifestWithPermutations();
            expect(j.nocHttp.get).toHaveBeenCalledWith('/vsa/metadata/permutations?latest_only=true', '', j.nocHttp.LOADING);
            expect(manifestMapper.getMode()).toEqual('VSA');
            expect(manifestMapper.isInitialized()).toEqual(true);
        });

        it('should initialize with no permutations', () => {
            magnifispec.initializeManifestWithNoPermutations();
            expect(manifestMapper.isInitialized()).toEqual(true);
        });

        it('can RIP', () => {
            magnifispec.initializeManifestWithPermutations();
            manifestMapper.suicide();
            expect(manifestMapper.isInitialized()).toEqual(false);
        });
    });

    describe('filterOutByLicenses', () => {
        it('should return distinct licenses', () => {
            expect(manifestMapper.filterOutByLicenses(permutations).length).toEqual(4);
            expect(manifestMapper.filterOutByLicenses([]).length).toEqual(0);
        });
    });

    describe('filterOutRegions', () => {
        it('should return distinct regions', () => {
            expect(manifestMapper.filterOutRegions(permutations).length).toEqual(5);
            expect(manifestMapper.filterOutRegions([]).length).toEqual(0);
        });
    });

    describe('getRegions', () => {
        it('should get the available regions', () => {
            magnifispec.initializeManifestWithPermutations();
            expect(manifestMapper.getRegions()).toEqual(
                [
                    {name: 'EU', code: 'eu-central-1', location: 'Frankfurt'},
                    {name: 'EU', code: 'eu-west-1', location: 'Ireland'},
                    {name: 'Asia Pacific', code: 'ap-southeast-1', location: 'Singapore'},
                    {name: 'US West', code: 'us-west-2', location: 'Oregon'},
                    {name: 'US West', code: 'us-west-1', location: 'N. California'}
                ]
            );
        });

        it('should return no region for an empty manifest', () => {
            magnifispec.initializeManifestWithNoPermutations();
            expect(manifestMapper.getRegions()).toEqual([]);
        });
    });

    describe('getLicenses', () => {
        it('should return all licenses', () => {
            magnifispec.initializeManifestWithPermutations();
            expect(manifestMapper.getLicenses()).toEqual([{
                type: 'cot-explore-paygo',
                name: 'Cloud Volumes ONTAP Explore',
                description: 'Suitable for smaller capacity applications (up to 2TB of underlying AWS storage)',
                'capacity_limit': '2TB',
                platformLicenseRequired: false,
                default: false,
                capacityLimit: {
                    size: 2,
                    unit: 'TB'
                }
            }, {
                type: 'cot-standard-paygo',
                name: 'Cloud Volumes ONTAP Standard',
                description: 'Flexible performance and larger capacity for a wider range of applications (up to 10TB of underlying AWS storage)',
                'capacity_limit': '10TB',
                platformLicenseRequired: false,
                default: true,
                capacityLimit: {
                    size: 10,
                    unit: 'TB'
                }
            }, {
                type: 'cot-premium-paygo',
                name: 'Cloud Volumes ONTAP Premium',
                description: 'Flexible performance and expanded capacity for a wider range of applications. Supports up to 368 TB',
                'capacity_limit': '368TB',
                platformLicenseRequired: false,
                default: false,
                capacityLimit: {
                    size: 368,
                    unit: 'TB'
                }
            }, {
                type: 'cot-premium-byol',
                name: 'Cloud Volumes ONTAP BYOL',
                description: 'Flexible performance and expanded capacity for a wider range of applications. Supports up to 368 TB',
                'capacity_limit': '368TB',
                platformLicenseRequired: true,
                default: false,
                capacityLimit: {
                    size: 368,
                    unit: 'TB'
                }
            }]);
        });

        it('should return no license for an empty manifest', () => {
            magnifispec.initializeManifestWithNoPermutations();
            expect(manifestMapper.getLicenses()).toEqual([]);
        });
    });

    describe('filterByRegionCode', () => {
        it('should filter by region code', () => {
            expect(manifestMapper.filterByRegionCode(permutations, 'us-west-1').length).toEqual(5);
            expect(manifestMapper.filterByRegionCode(permutations, 'us-west-2').length).toEqual(1);
            expect(manifestMapper.filterByRegionCode(permutations, 'us-west-3').length).toEqual(0);
        });
    });

    describe('filterByVersion', () => {
        it('should filter by version', () => {
            expect(manifestMapper.filterByVersion(permutations, 'ONTAP-9.0X21.T1').length).toEqual(3);
            expect(manifestMapper.filterByVersion(permutations, 'ONTAP-8.4X16.T1.ha').length).toEqual(2);
            expect(manifestMapper.filterByVersion(permutations, '').length).toEqual(0);
        });
    });

    describe('filterByLicenseType', () => {
        it('should filter by license type', () => {
            expect(manifestMapper.filterByLicenseType(permutations, 'cot-explore-paygo').length).toEqual(3);
            expect(manifestMapper.filterByLicenseType(permutations, 'cot-premium-paygo').length).toEqual(1);
            expect(manifestMapper.filterByLicenseType(permutations, '').length).toEqual(0);
        });
    });

    describe('filterByFeature', () => {
        it('should filter by feature', () => {
            expect(manifestMapper.filterByFeature(permutations, 'encryption').length).toEqual(2);
            expect(manifestMapper.filterByFeature(permutations, 'new-feature').length).toEqual(1);
            expect(manifestMapper.filterByFeature(permutations, '').length).toEqual(0);
        });
    });

    describe('filterByRegionCodeAndVersion', () => {
        it('should filter by region code and version', () => {
            expect(manifestMapper.filterByRegionCodeAndVersion(permutations, 'eu-west-1', 'ONTAP-9.0X21.T1').length).toEqual(1);
            expect(manifestMapper.filterByRegionCodeAndVersion(permutations, 'us-west-2', 'ONTAP-9.0X21.T1').length).toEqual(1);
            expect(manifestMapper.filterByRegionCodeAndVersion(permutations, '', 'ONTAP-9.0X21.T1').length).toEqual(0);
            expect(manifestMapper.filterByRegionCodeAndVersion(permutations, 'us-west-2', '').length).toEqual(0);
        });
    });

    describe('filterByVersionAndLicenseType', () => {
        it('should filter by version and license type', () => {
            expect(manifestMapper.filterByRegionCodeVersionAndLicenseType(permutations, 'eu-central-1', 'ONTAP-9.0X21.T1', 'cot-explore-paygo').length).toEqual(1);
            expect(manifestMapper.filterByRegionCodeVersionAndLicenseType(permutations, 'ap-southeast-1', 'ONTAP.ENG-8.3.2.T5', 'cot-standard-paygo').length).toEqual(1);
            expect(manifestMapper.filterByRegionCodeVersionAndLicenseType(permutations, 'eu-central-1', '', '').length).toEqual(0);
            expect(manifestMapper.filterByRegionCodeVersionAndLicenseType(permutations, '', 'cot-explore-paygo').length).toEqual(0);
            expect(manifestMapper.filterByRegionCodeVersionAndLicenseType(permutations, 'ONTAP-9.0X21.T1', '').length).toEqual(0);
        });
    });

    describe('filterByVersionAndFeature', () => {
        it('should filter by version and feature', () => {
            expect(manifestMapper.filterByVersionAndFeature(permutations, 'ONTAP-8.4X16.T1.ha', 'encryption').length).toEqual(1);
            expect(manifestMapper.filterByVersionAndFeature(permutations, 'ONTAP.ENG-8.3.2.T5', 'new-feature').length).toEqual(1);
            expect(manifestMapper.filterByVersionAndFeature(permutations, '', 'new-feature').length).toEqual(0);
            expect(manifestMapper.filterByVersionAndFeature(permutations, 'ONTAP-9.0X21.T1', '').length).toEqual(0);
        });
    });

    describe('getAvailableVersions', () => {
        it('should return the available versions', () => {
            magnifispec.initializeManifestWithPermutations();
            expect(manifestMapper.getAvailableVersions('us-west-1')).toEqual(['ONTAP-8.4X16.T1.ha', 'ONTAP.ENG-8.3.2.T5', 'ONTAP.ENG-8.3.2.T9']);
            expect(manifestMapper.getAvailableVersions('us-west-2')).toEqual(['ONTAP-9.0X21.T1']);
            expect(manifestMapper.getAvailableVersions('us-west-3')).toEqual([]);
        });
    });

    describe('getAvailableLicenses', () => {
        it('should return the available licenses', () => {
            magnifispec.initializeManifestWithPermutations();
            expect(manifestMapper.getAvailableLicenses('us-west-1', 'ONTAP.ENG-8.3.2.T5')).toEqual([
                {
                    type: 'cot-premium-byol',
                    name: 'Cloud Volumes ONTAP BYOL',
                    description: 'Flexible performance and expanded capacity for a wider range of applications. Supports up to 368 TB',
                    'capacity_limit': '368TB',
                    platformLicenseRequired: true,
                    default: false,
                    capacityLimit: {size: 368, unit: 'TB'},
                    availableInstances: ['m4.xlarge', 'm9.xlarge'],
                    _supportsOntapEncryption: false
                }
            ]);
            expect(manifestMapper.getAvailableLicenses('us-west-1', '')).toEqual([]);
        });
    });

    describe('getAvailableInstances', () => {
        it('should return the available instance types', () => {
            magnifispec.initializeManifestWithPermutations();
            expect(manifestMapper.getAvailableInstances('eu-central-1', 'ONTAP-9.0X21.T1', 'cot-explore-paygo')).toEqual(['m4.xlarge']);
            expect(manifestMapper.getAvailableInstances('ap-southeast-1', 'ONTAP.ENG-8.3.2.T5', 'cot-standard-paygo')).toEqual(['m4.2xlarge']);
            expect(manifestMapper.getAvailableInstances('eu-central-1', '', '')).toEqual([]);
            expect(manifestMapper.getAvailableInstances('', 'cot-explore-paygo')).toEqual([]);
            expect(manifestMapper.getAvailableInstances('ONTAP-9.0X21.T1', '')).toEqual([]);
        });
    });

    describe('getDefaultInstance', () => {
        it('should return the available instance types', () => {
            magnifispec.initializeManifestWithPermutations();
            expect(manifestMapper.getDefaultInstance('eu-central-1', 'ONTAP-9.0X21.T1', 'cot-explore-paygo')).toEqual('m4.xlarge');
            expect(manifestMapper.getDefaultInstance('ap-southeast-1', 'ONTAP.ENG-8.3.2.T5', 'cot-standard-paygo')).toEqual('m4.2xlarge');
            expect(manifestMapper.getDefaultInstance('eu-central-1', '', '')).toEqual(undefined);
            expect(manifestMapper.getDefaultInstance('', 'cot-explore-paygo')).toEqual(undefined);
            expect(manifestMapper.getDefaultInstance('ONTAP-9.0X21.T1', '')).toEqual(undefined);
        });
    });

    describe('getEncryptionEnabledInstanceTypes', () => {
        it('should return the available instances for encryption', () => {
            magnifispec.initializeManifestWithPermutations();
            expect(manifestMapper.getEncryptionEnabledInstanceTypes('ONTAP.ENG-8.3.2.T5')).toEqual(['m4.2xlarge']);
            expect(manifestMapper.getEncryptionEnabledInstanceTypes('ONTAP-8.4X16.T1.ha')).toEqual(['r3.2xlarge']);
            expect(manifestMapper.getEncryptionEnabledInstanceTypes('')).toEqual([]);
        });

        it('should return no instances for encryption for an empty manifest', () => {
            magnifispec.initializeManifestWithNoPermutations();
            expect(manifestMapper.getEncryptionEnabledInstanceTypes()).toEqual([]);
        });
    });

    describe('has valid mapping', () => {
        it('should return true if there is an option to create a vsa', () => {
            magnifispec.initializeManifestWithPermutations();
            expect(manifestMapper.hasValidMapping()).toEqual(true);
        });

        it('should return false it there is not any', () => {
            magnifispec.initializeManifestWithNoPermutations();
            expect(manifestMapper.hasValidMapping()).toEqual(false);
        });
    });
});
