describe('CreateWorkingEnvironmentWizardService', () => {
    'use strict';

    let j;
    beforeEach(() => {
        j = magnifispec.setup();
    });
    let CreateWorkingEnvironmentWizardService;

    beforeEach(inject((_CreateWorkingEnvironmentWizardService_) => {
        CreateWorkingEnvironmentWizardService = _CreateWorkingEnvironmentWizardService_;
        j.$rootScope.currentUser = {tenantId: 1, roleId: 'Role-1'};
        j.scope = {
            wizardController: {
                workingEnvironmentTypeAndProvider: {type: 'VSA', providerName: 'AWS'},
                workingEnvironmentType: 'VSA',
                createVpcSelectObjects: {subnet: {subnetId: 'subnet-1'}},
                selectedRegion: 'region-code-1',
                selectedLicense: {type: 'standard-paygo'},
                selectedOntapVersion: 'ONTAP-1-2-3',
                selectedInstanceType: 'm4-large',
                providedLicense: 'some-license'
            }
        };

        CreateWorkingEnvironmentWizardService.call(j.scope, j.nocHttp, j.nocUI);
    }));

    describe('isMarketPlaceRegistered', () => {
        let MetadataResource;

        beforeEach(inject((_MetadataResource_) => {
            MetadataResource = _MetadataResource_;
            jest.spyOn(MetadataResource, 'isRegisteredToMarketPlace').mockImplementation(() => {
            });
        }));

        it('should return true for Azure if check was disabled', () => {
            j.nocLocalStorage.DISABLE_AZURE_MP_CHECK.save(true);
            j.scope.wizardController.workingEnvironmentTypeAndProvider.providerName = 'Azure';

            expect(j.scope.isMarketPlaceRegistered()).toBeResolvedWith({subscribed: true});
        });

        it('should ignore localStorage for not Azure', () => {
            jest.spyOn(j.nocLocalStorage.DISABLE_AZURE_MP_CHECK, 'restore').mockImplementation(() => {
            });

            j.scope.isMarketPlaceRegistered();

            expect(j.nocLocalStorage.DISABLE_AZURE_MP_CHECK.restore).not.toHaveBeenCalled();
        });

        const request = {
            region: 'region-code-1',
            subnetId: 'subnet-1',
            vsaMetadata: {
                ontapVersion: 'ONTAP-1-2-3',
                licenseType: 'standard-paygo',
                instanceType: 'm4-large',
                providedLicense: 'some-license'
            }
        };

        it('should pass correct values to API', () => {

            j.scope.isMarketPlaceRegistered();

            expect(MetadataResource.isRegisteredToMarketPlace).toHaveBeenCalledWith(j.nocHttp, request, 'VSA', 'AWS');
        });

        it('should pass correct values to API for Azure', () => {
            j.nocLocalStorage.DISABLE_AZURE_MP_CHECK.save(false);
            j.scope.wizardController.workingEnvironmentTypeAndProvider = {type: 'VSA', providerName: 'Azure'};
            j.scope.wizardController.selectedAzureSubscription = {name: 'lili', subscriptionId: '123'};
            j.scope.wizardController.selectedSubnet = {id: 'azure-subnet-id'};
            request.subnetId = 'azure-subnet-id';
            request.subscriptionId = '123';

            j.scope.isMarketPlaceRegistered();

            expect(MetadataResource.isRegisteredToMarketPlace).toHaveBeenCalledWith(j.nocHttp, request, 'VSA', 'Azure');
        });
    });

    describe('continueAfterLicenseOrPackagesStep', () => {
        beforeEach(() => {
            j.$state.go = jest.fn();
            j.$rootScope.runningInIC = false;
        });

        it('should check invalidity', () => {
            j.scope.isValid = jest.fn().mockReturnValue(false);

            j.scope.continueAfterLicenseOrPackagesStep('.stepName');

            expect(j.nocUI.handleErrorsInWizard).toHaveBeenCalled();
        });

        it('should skip to final step in IC (NSS and marketplace are not available)', () => {
            j.scope.isValid = jest.fn().mockReturnValue(true);
            j.$rootScope.runningInIC = true;

            j.scope.continueAfterLicenseOrPackagesStep('.stepName');

            expect(j.$state.go).toHaveBeenCalledWith('.stepName');
        });

        it('should go to check-mp-registration step if needed', () => {
            j.scope.isValid = jest.fn().mockReturnValue(true);
            jest.spyOn(j.scope, 'isMarketPlaceRegistered').mockReturnValue(j.$q.when({subscribed: false}));

            j.scope.continueAfterLicenseOrPackagesStep('.stepName');
            magnifispec.processPromise();

            expect(j.$state.go).toHaveBeenCalledWith('.check-mp-registration');
        });

        it('should skip check-mp-registration step if needed', () => {
            j.scope.isValid = jest.fn().mockReturnValue(true);
            jest.spyOn(j.scope, 'isMarketPlaceRegistered').mockReturnValue(j.$q.when({subscribed: true}));

            j.scope.continueAfterLicenseOrPackagesStep('.stepName');
            magnifispec.processPromise();

            expect(j.$state.go).toHaveBeenCalledWith('.stepName');
        });
    });
});
