describe('ADSetupController', () => {
    'use strict';

    var j;
    beforeEach(() => {
        j = magnifispec.setup();
    });
    const adUrl = '/occm/api/active-directory';
    const adTestUrl = '/occm/api/active-directory/check-connection';

    let ad, ads, successfullyTestedAD;

    beforeEach(() => {
        magnifispec.mockStandaloneController('ADSetupController');

        //Skip first initialization
        j.$httpBackend.expectGET(adUrl).respond([]);
        j.$httpBackend.flush();

        ad = {
            id: 1,
            publicId: 'ActiveDirectoryEntry$-AZVqhI3z',
            name: 'default',
            host: 'ad.com',
            port: 3
        };

        ads = [ad];

        successfullyTestedAD = _.clone(ad);
        successfullyTestedAD.successfullyConnectedTo = {host: ad.host, port: ad.port};
    });

    describe('initialize', () => {
        beforeEach(() => {
            j.scope.adForm = {$setPristine: jest.fn().mockReset()};
            j.scope.getAD = jest.fn();
            j.scope.isInitialized = false;
        });

        it('should initialize scope when AD received', () => {
            j.scope.getAD.mockReturnValue(j.$q.when(ad));

            j.scope.initialize();
            magnifispec.processPromise();

            expect(j.scope.ad).toEqual(successfullyTestedAD);
            expect(j.scope.editMode).toEqual(false);
        });

        it('should initialize scope when no AD received', () => {
            j.scope.getAD.mockReturnValue(j.$q.when(undefined));

            j.scope.initialize();
            magnifispec.processPromise();

            expect(j.scope.ad).toEqual(undefined);
            expect(j.scope.editMode).toEqual(true);
        });

        it('should initialize scope when failed to get AD', () => {
            j.scope.getAD.mockReturnValue(j.$q.reject());

            j.scope.initialize();
            magnifispec.processPromise();

            expect(j.scope.ad).toEqual(undefined);
            expect(j.scope.editMode).toEqual(true);
        });

        afterEach(() => {
            expect(j.scope.adForm.$setPristine).toHaveBeenCalled();
            expect(j.scope.isInitialized).toEqual(true);
        });
    });

    describe('getAD', () => {
        it('should get the AD', () => {
            j.$httpBackend.expectGET(adUrl).respond(ads);
            const adGot = j.scope.getAD();
            j.$httpBackend.flush();

            expect(adGot).toBeResolvedWith(ad);
        });

        it('should get no AD', () => {
            j.$httpBackend.expectGET(adUrl).respond([]);
            const adGot = j.scope.getAD();
            j.$httpBackend.flush();

            expect(adGot).toBeResolvedWith(undefined);
        });

        it('should report an error', () => {
            j.$httpBackend.expectGET(adUrl).respond(400, j.errorResponse);
            const adGot = j.scope.getAD();
            j.$httpBackend.flush();

            expect(adGot).toBeRejected();
            expect(j.pidgeon.error).toHaveBeenCalled();
        });
    });

    describe('needsConnectivityTest', () => {
        it('should return false iff AD host and AD port were tested and connected successfully', () => {
            expect(j.scope.needsConnectivityTest(successfullyTestedAD)).toEqual(false);
        });

        it('should return true if AD does not exist', () => {
            expect(j.scope.needsConnectivityTest(undefined)).toEqual(true);
        });

        it('should return true if AD was not tested', () => {
            expect(j.scope.needsConnectivityTest(ad)).toEqual(true);
        });

        it('should return true if AD was tested and then user changed host or port', () => {
            const successfullyTestedADAndChanged = _.clone(ad);
            successfullyTestedADAndChanged.successfullyConnectedTo = {host: 'different host', port: 9999};
            expect(j.scope.needsConnectivityTest(successfullyTestedADAndChanged)).toEqual(true);
        });
    });

    describe('saveADError', () => {
        const pristineForm = {$pristine: true};
        const invalidForm = {$invalid: true};
        const validForm = {};

        it('should return an error if form is invalid', () => {
            expect(j.scope.saveADError(invalidForm, undefined)).toEqual('AD details are invalid');
        });

        it('should return an error if form is pristine and ad exists', () => {
            expect(j.scope.saveADError(pristineForm, ad)).toEqual('AD is already stored');
        });

        it('should return an error if AD does not exist', () => {
            expect(j.scope.saveADError(validForm, undefined)).toEqual('Connectivity to AD must be confirmed before saving');
        });

        it('should return an error if AD was not tested', () => {
            expect(j.scope.saveADError(undefined, ad)).toEqual('Connectivity to AD must be confirmed before saving');
        });

        it('should not return any error on a valid form and successfully tested AD', () => {
            expect(j.scope.saveADError(validForm, successfullyTestedAD)).toEqual('');
        });
    });

    describe('saveAD', () => {
        const newAD = {host: 'ad.com', port: 3, name: 'default'}; //No public id

        beforeEach(() => {
            j.scope.initialize = jest.fn().mockReset();
        });

        it('should update AD when there is an AD to update', () => {
            j.$httpBackend.expectPUT(adUrl + '/' + successfullyTestedAD.publicId,
                {host: successfullyTestedAD.host, port: successfullyTestedAD.port, name: 'default'}).respond(200);
            j.scope.saveAD(successfullyTestedAD);
            j.$httpBackend.flush();

            expect(j.pidgeon.success).toHaveBeenCalledWith('Active directory saved');
            expect(j.scope.initialize).toHaveBeenCalled();
        });

        it('should create AD when there is no AD to update', () => {
            j.$httpBackend.expectPOST(adUrl, newAD).respond(200);
            j.scope.saveAD(newAD);
            j.$httpBackend.flush();

            expect(j.pidgeon.success).toHaveBeenCalledWith('Active directory saved');
            expect(j.scope.initialize).toHaveBeenCalled();
        });

        it('should report an error when save is not successful', () => {
            j.$httpBackend.expectPOST(adUrl, newAD).respond(400, j.errorResponse);
            j.scope.saveAD(newAD);
            j.$httpBackend.flush();

            expect(j.pidgeon.error).toHaveBeenCalled();
        });
    });

    describe('testAD', () => {
        beforeEach(() => {
            ad.userName = 'user';
            ad.password = 'pass';
        });

        it('should test connectivity to the AD', () => {
            j.$httpBackend.expectPOST(adTestUrl, {
                host: ad.host,
                port: ad.port,
                userName: ad.userName,
                password: ad.password
            }).respond(200);

            j.scope.testAD(ad);
            j.$httpBackend.flush();

            expect(ad.successfullyConnectedTo).toEqual({host: ad.host, port: ad.port});
            expect(j.pidgeon.success).toHaveBeenCalledWith('Connection is successful');
        });

        it('should report an error when test connectivity to the AD failed', () => {
            j.$httpBackend.expectPOST(adTestUrl, {
                host: ad.host,
                port: ad.port,
                userName: ad.userName,
                password: ad.password
            }).respond(400, j.errorResponse);
            j.scope.testAD(ad);
            j.$httpBackend.flush();

            expect(j.pidgeon.error).toHaveBeenCalled();
        });
    });

    describe('deleteADTestCredentials', () => {
        it('should delete userName and password', () => {
            j.scope.ad = {userName: 'user', password: 'pass'};
            j.scope.deleteADTestCredentials();
            expect(j.scope.ad.userName).toBeUndefined();
            expect(j.scope.ad.password).toBeUndefined();
        });
    });
});
