describe('Data Fabric Controller', () => {
    'use strict';

    let j;
    beforeEach(() => {
        j = magnifispec.setup();
    });

    let stopFunction;
    let we1, we2, we3, ha, workingEnvironments;
    let workingEnvironmentResource;
    let storage;
    let _nocSelectedWorkspaceOrTenantService;

    beforeEach(inject((_workingEnvironmentResource_, nocSelectedWorkspaceOrTenantService) => {
        workingEnvironmentResource = _workingEnvironmentResource_;
        const fakeSetActionRequiredFor = (publicId, isActive, actionRequiredResponse) => {
            j.scope.actionsRequired[publicId] = actionRequiredResponse[publicId] && actionRequiredResponse[publicId].length && !isActive ? actionRequiredResponse[publicId] : undefined;
        };
        j.$rootScope.clearActionsRequired = jest.fn();
        j.$rootScope.setComputingCapacity = jest.fn();
        j.$rootScope.setActionRequiredFor = jest.fn();
        j.$rootScope.setActionRequiredFor.mockImplementation(fakeSetActionRequiredFor);
        j.$rootScope.selectedTenant = {publicId: 1};
        j.$rootScope.actionsRequired = {};
        stopFunction = jest.fn();
        jest.spyOn(j.workingEnvironmentResource, 'getWorkingEnvironments');

        jest.spyOn(j.workingEnvironmentResource, 'startWorkingEnvironmentsPolling').mockReturnValue(stopFunction);
        jest.spyOn(j.actionRequiredResource, 'startGlobalActionRequiredPolling').mockReturnValue(stopFunction);
        const setWorkingEnvironments = (wes) => {
            j.$rootScope.workingEnvironments = wes;
        };

        storage = jasmine.createSpyObj('storage', ['save', 'restore']);
        jest.spyOn(j.nocLocalStorage, 'GALLERY_VIEW_SHOWN').mockReturnValue(storage);

        magnifispec.mockTabsController('DataFabricController', null, {setWorkingEnvironments});
        we1 = {
            name: 'we1',
            publicId: '1',
            workingEnvironmentType: 'VSA',
            cloudProviderName: 'AWS',
            _normalizedReservedSizeTB: {size: 0, unit: 'TB'}
        };
        we2 = {
            name: 'we2',
            publicId: '2',
            workingEnvironmentType: 'VSA',
            cloudProviderName: 'AWS',
            _normalizedReservedSizeTB: {size: 0, unit: 'TB'}
        };
        we3 = {
            name: 'we3',
            publicId: '3',
            workingEnvironmentType: 'ON_PREM',
            location: 'AWS',
            _normalizedReservedSizeTB: {size: 0, unit: 'TB'}
        };
        ha = {
            name: 'ha',
            publicId: '4',
            workingEnvironmentType: 'VSA',
            cloudProviderName: 'AWS',
            isHA: true,
            _normalizedReservedSizeTB: {size: 0, unit: 'TB'}
        };

        workingEnvironments = [we1, we2, we3, ha];
        _.forEach(workingEnvironments, (workingEnvironment) => {
            workingEnvironmentResource.initWorkingEnvironment(workingEnvironment);
        });
        j.scope.resetCapacityByTypeMap(workingEnvironments);

        _nocSelectedWorkspaceOrTenantService = nocSelectedWorkspaceOrTenantService;
        jest.spyOn(_nocSelectedWorkspaceOrTenantService, 'getPublicId').mockReturnValue(1);
    }));

    describe('mergeNewWorkingEnvironmentResults', () => {
        beforeEach(() => {
            jest.spyOn(j.scope, 'updateCapacityByTypeMap').mockImplementation(() => {
            });
        });

        it('should do nothing if the newWorkingEnvironment is no longer in the workingEnvironments array', () => {
            const existingArray = [{publicId: 1}, {publicId: 2}, {publicId: 3}];
            j.scope.mergeNewWorkingEnvironmentResults(existingArray, {publicId: 4, newKey: 'newVal'});
            expect(existingArray).toEqual([{publicId: 1}, {publicId: 2}, {publicId: 3}]);
        });

        it('should update the correct item in the old working environments array if exists', () => {
            const existingArray = [{publicId: 1}, {publicId: 2}, {publicId: 3}];
            j.scope.mergeNewWorkingEnvironmentResults(existingArray, {publicId: 2, newKey: 'newVal'}, {publicId: 2});
            expect(existingArray).toEqual([{publicId: 1}, {publicId: 2, newKey: 'newVal'}, {publicId: 3}]);
            expect(j.scope.updateCapacityByTypeMap).toHaveBeenCalledWith({
                publicId: 2,
                newKey: 'newVal'
            }, {publicId: 2});
        });
    });

    describe('mergeWorkingEnvironmentsList', () => {
        it('should return empty in case currentWorkingEnvironments is empty', () => {
            const previousList = [{publicId: 1}, {publicId: 2}];
            const currentList = [];
            const result = j.scope.mergeWorkingEnvironmentsList(previousList, currentList);
            expect(result).toEqual([]);
        });

        it('should return currentWorkingEnvironments in case previousWorkingEnvironments is empty', () => {
            const previousList = [];
            const currentList = [{publicId: 1}, {publicId: 2}];
            const result = j.scope.mergeWorkingEnvironmentsList(previousList, currentList);
            expect(result).toEqual(currentList);
        });

        it('should remove the items from previousList that are no longer in currentList and add new additions from currentList to previousList', () => {
            const previousList = [{publicId: 1}, {publicId: 2}, {publicId: 'deleted'}];
            const currentList = [{publicId: 2}, {publicId: 1}, {publicId: 'new', workingEnvironmentType: 'onPrem'}];
            const result = j.scope.mergeWorkingEnvironmentsList(previousList, currentList);
            expect(result).toEqual([{publicId: 'new', workingEnvironmentType: 'onPrem'}, {publicId: 1}, {publicId: 2}]);
        });
    });

    describe('on get working environment', () => {
        beforeEach(() => {
            j.$rootScope.currentUser = {roleId: 'Role-2'};
        });

        describe('Merging the working environment list', () => {
            it('should merge the existing working environments with the payload working environments', () => {
                jest.spyOn(j.scope, 'mergeWorkingEnvironmentsList');
                j.scope.workingEnvironments = [{publicId: 2}];
                j.$rootScope.$broadcast(j.workingEnvironmentResource.WORKING_ENVIRONMENTS_GET_EVENT, {
                    workingEnvironments: [workingEnvironments[0]],
                    tenantId: 1
                });
                expect(j.scope.mergeWorkingEnvironmentsList).toHaveBeenCalledWith([{publicId: 2}], [workingEnvironments[0]]);
            });

            it('should merge an empty array with the payload working environments in case nothing is on the scope yet', () => {
                jest.spyOn(j.scope, 'mergeWorkingEnvironmentsList');
                j.scope.workingEnvironments = null;
                j.$rootScope.$broadcast(j.workingEnvironmentResource.WORKING_ENVIRONMENTS_GET_EVENT, {
                    workingEnvironments: [workingEnvironments[0]],
                    tenantId: 1
                });
                expect(j.scope.mergeWorkingEnvironmentsList).toHaveBeenCalledWith([], [workingEnvironments[0]]);
            });
        });

        describe('requesting more data', () => {
            beforeEach(() => {
                jest.spyOn(j.scope, 'updateCapacityByTypeMap').mockImplementation(() => {
                });
                jest.spyOn(j.workingEnvironmentResource, 'getWorkingEnvironmentByPublicId').mockReturnValue(j.$q.when({
                    publicId: 1,
                    newKey: 'newVal',
                    replicationProperties: {peers: [], replicationTargets: []}
                }));
                jest.spyOn(j.scope, 'mergeNewWorkingEnvironmentResults').mockImplementation(() => {
                });
            });

            it('should make an additional more data request for each WE', () => {
                const we = {publicId: 1};
                jest.spyOn(j.scope, 'mergeWorkingEnvironmentsList').mockReturnValue([we]);
                j.scope.createReplicationMapByType = jest.fn();
                j.$rootScope.$broadcast(j.workingEnvironmentResource.WORKING_ENVIRONMENTS_GET_EVENT, {
                    workingEnvironments: [workingEnvironments[0]],
                    tenantId: 1
                });
                j.$timeout.flush();
                expect(j.scope.mergeNewWorkingEnvironmentResults).toHaveBeenCalledWith([{
                    publicId: 1,
                    _moreDataRequest: null,
                    replicationProperties: {peers: [], replicationTargets: []},
                    peers: [],
                    replicationTargets: [],
                    _peers: undefined
                }], {
                    publicId: 1,
                    newKey: 'newVal',
                    replicationProperties: {peers: [], replicationTargets: []}
                }, {publicId: 1, _moreDataRequest: null});
                expect(we._moreDataRequest).toBeNull();
                expect(j.scope.createReplicationMapByType).toHaveBeenCalledWith([{
                    publicId: 1,
                    _moreDataRequest: null,
                    replicationProperties: {peers: [], replicationTargets: []},
                    peers: [],
                    replicationTargets: [],
                    _peers: undefined
                }]);
            });

            it('should not make an additional more data request if the previous more data request has not yet finished', () => {
                jest.spyOn(j.scope, 'mergeWorkingEnvironmentsList').mockReturnValue([{
                    publicId: 1,
                    _moreDataRequest: 'previous request'
                }]);
                j.$rootScope.$broadcast(j.workingEnvironmentResource.WORKING_ENVIRONMENTS_GET_EVENT, {
                    workingEnvironments: [workingEnvironments[0]],
                    tenantId: 1
                });
                j.$timeout.flush();
                expect(j.scope.mergeNewWorkingEnvironmentResults).not.toHaveBeenCalled();
            });
        });

        describe('controlling selected working environment', () => {
            it('should keep the selected working environment if exists', () => {
                j.scope.selectedWorkingEnvironment = {publicId: '1'};
                j.$rootScope.$broadcast(j.workingEnvironmentResource.WORKING_ENVIRONMENTS_GET_EVENT, {
                    workingEnvironments: [workingEnvironments[0]],
                    tenantId: 1
                });
                expect(j.scope.workingEnvironments.length).toEqual(1);
                const extendedWorkingEnvironment = _.assignIn(workingEnvironments[0], {peers: []});
                expect(j.scope.workingEnvironments[0]).toEqual(extendedWorkingEnvironment);
                expect(j.scope.selectedWorkingEnvironment).toEqual(extendedWorkingEnvironment);
            });

            it('should reset the selected working environment if does not exist', () => {
                j.scope.selectedWorkingEnvironment = {publicId: '2'};
                j.$rootScope.$broadcast(j.workingEnvironmentResource.WORKING_ENVIRONMENTS_GET_EVENT, {
                    workingEnvironments: [workingEnvironments[0]],
                    tenantId: 1
                });
                const extendedWorkingEnvironment = _.assignIn(workingEnvironments[0], {peers: []});
                expect(j.scope.workingEnvironments).toEqual([extendedWorkingEnvironment]);
                expect(j.scope.selectedWorkingEnvironment).toEqual(undefined);
            });
        });
    });

    it('should have started working environments polling', () => {
        expect(j.scope.clearActionsRequired).toHaveBeenCalled();
        expect(j.workingEnvironmentResource.startWorkingEnvironmentsPolling).toHaveBeenCalled();
    });

    it('should be listening on workingEnvironmentResource.WORKING_ENVIRONMENTS_GET_EVENT (when not oncloud admin)', () => {
        j.$rootScope.currentUser = {roleId: 'Role-2'};
        j.$rootScope.$broadcast(j.workingEnvironmentResource.WORKING_ENVIRONMENTS_GET_EVENT, {
            workingEnvironments: [we1, we2],
            tenantId: 1
        });
        expect(j.scope.workingEnvironments).toEqual([we1, we2]);
    });

    it('should be listening on workingEnvironmentResource.WORKING_ENVIRONMENTS_GET_ERROR_EVENT', () => {
        j.scope.selectedTenant = {publicId: 1};
        j.$rootScope.$broadcast(j.workingEnvironmentResource.WORKING_ENVIRONMENTS_GET_ERROR_EVENT, {
            error: j.errorResponse,
            tenantId: 1
        });
        expect(j.pidgeon.error).toHaveBeenCalledWith(j.pidgeon.serverErrorPrefix + 'retrieving working environments', j.errorResponse, 'list');
    });

    describe('capacity action required', () => {
        beforeEach(() => {
            j.$rootScope.currentUser = {roleId: 'Role-1'};
            j.scope.selectedTenant = {publicId: 1};
        });

        it('should not start poller if there are no working environments', () => {
            j.$rootScope.$broadcast(j.workingEnvironmentResource.WORKING_ENVIRONMENTS_GET_EVENT, {
                workingEnvironments: [],
                tenantId: '1'
            });
            expect(j.actionRequiredResource.startGlobalActionRequiredPolling).not.toHaveBeenCalled();
            expect(j.scope.setComputingCapacity).not.toHaveBeenCalled();
        });

        it('should start poller after get working environments', () => {
            j.$rootScope.$broadcast(j.workingEnvironmentResource.WORKING_ENVIRONMENTS_GET_EVENT, {
                workingEnvironments: [we1, we2],
                tenantId: 1
            });
            expect(j.actionRequiredResource.startGlobalActionRequiredPolling).toHaveBeenCalled();
            expect(j.scope.setComputingCapacity).toHaveBeenCalledWith(true);
        });

        it('should update action required', () => {
            const actions = {'1': [{'something': 'else'}], '2': [{'something': 'else2'}]};
            j.scope.workingEnvironments = [{
                _pathPrefix: '/vsa/',
                publicId: '1',
                activeActions: null,
                workingEnvironmentType: 'VSA'
            }, {publicId: '2', activeActions: null, workingEnvironmentType: 'ON_PREM'}];
            j.$rootScope.$broadcast(j.actionRequiredResource.GLOBAL_ACTION_REQUIRED_GET_EVENT, actions);
            expect(j.scope.setActionRequiredFor).toHaveBeenCalledWith('1', false, actions);
            expect(j.scope.totalNumOfActionsRequired).toEqual(2);
        });

        it('should update specific VSA when moves from active to inactive', () => {
            jest.spyOn(j.scope, 'setComputingCapacityForWE').mockImplementation(() => {
            });
            const oldWe = {
                _pathPrefix: '/vsa/',
                publicId: '1',
                activeActions: ['one two three'],
                workingEnvironmentType: 'VSA',
                _workingEnvironmentTypeByLocation: 'VSA',
                _normalizedReservedSizeTB: {size: 0, unit: 'TB'}
            };
            const workingEnvironments = [oldWe];
            const newWeResponse = {
                _pathPrefix: '/vsa/',
                name: 'we1',
                publicId: '1',
                activeActions: null,
                workingEnvironmentType: 'VSA',
                _workingEnvironmentTypeByLocation: 'VSA',
                _normalizedReservedSizeTB: {size: 0, unit: 'TB'}
            };
            j.scope.mergeNewWorkingEnvironmentResults(workingEnvironments, newWeResponse, oldWe);
            expect(j.scope.setComputingCapacityForWE).toHaveBeenCalledWith(oldWe, true);
            j.$httpBackend.expectGET(j.nocBaseUrl + '/vsa/working-environments/1?fields=status,actionsRequired').respond(200, {actionsRequired: ['blah']});
            j.$httpBackend.flush();
            expect(j.scope.setActionRequiredFor).toHaveBeenCalledWith('1', false, {1: ['blah']});
            expect(j.scope.setComputingCapacityForWE).toHaveBeenCalledWith(oldWe, false);
        });

        it('should fail gracefully', () => {
            j.$rootScope.$broadcast(j.actionRequiredResource.GLOBAL_ACTION_REQUIRED_ERROR_EVENT, j.errorResponse);
            expect(j.pidgeon.error).toHaveBeenCalledWith(j.pidgeon.serverErrorPrefix + 'retrieving required actions for capacity', j.errorResponse, 'action-required');
        });
    });

    describe('shouldShowReplicationAction', () => {
        beforeEach(() => {
            j.scope.createReplicationMapByType = jest.fn();
            j.scope.replicationMapByType = {};
        });

        it('should create the map before examining it', () => {
            j.scope.shouldShowReplicationAction(we1);
            expect(j.scope.createReplicationMapByType).toHaveBeenCalled();
        });

        it('should return false if the working environment is VSA and there are less than 2 VSAs', () => {
            we1.workingEnvironmentType = 'VSA';
            j.scope.replicationMapByType['VSA'] = 1;
            expect(j.scope.shouldShowReplicationAction(we1)).toBeFalsy();
        });

        it('should return false if the working environment type is ON_PREM but there is no VSA working environment', () => {
            we1.workingEnvironmentType = 'ON_PREM';
            j.scope.replicationMapByType['ON_PREM'] = 1;
            expect(j.scope.shouldShowReplicationAction(we1)).toBeFalsy();
        });

        it('should return true if the working environment is VSA and there are at least 2 VSAs', () => {
            we1.workingEnvironmentType = 'VSA';
            j.scope.replicationMapByType['VSA'] = 2;
            expect(j.scope.shouldShowReplicationAction(we1)).toBeTruthy();
        });

        it('should return true if the working environment is ON_PREM and there is at least one VSA', () => {
            we1.workingEnvironmentType = 'ON_PREM';
            j.scope.replicationMapByType['VSA'] = 1;
            j.scope.replicationMapByType['ON_PREM'] = 1;
            expect(j.scope.shouldShowReplicationAction(we1)).toBeTruthy();
        });

        it('should return true if the working environment is ON_PREM and there are at least 2 ON_PREM', () => {
            we1.workingEnvironmentType = 'ON_PREM';
            j.scope.replicationMapByType['ON_PREM'] = 2;
            expect(j.scope.shouldShowReplicationAction(we1)).toBeTruthy();
        });
    });

    describe('remove we', () => {
        beforeEach(() => {
            j.scope.removeWorkingEnvironment = jest.fn().mockReturnValue(j.$q.when());
            j.scope.workingEnvironments = [we1, we2];
        });

        it('should remove that we from the scope', () => {
            j.scope.removeWE(we1);
            expect(j.scope.removeWorkingEnvironment).toHaveBeenCalledWith(we1, false);
            j.scope.$digest();
            expect(j.scope.workingEnvironments).toEqual([we2]);
        });

        it('should not remove anything if not found', () => {
            const we3 = {publicId: '3', status: {}};
            j.scope.removeWE(we3);
            expect(j.scope.removeWorkingEnvironment).toHaveBeenCalledWith(we3, false);
            j.scope.$digest();
            expect(j.scope.workingEnvironments).toEqual([we1, we2]);
        });
    });

    describe('resetCapacityByTypeMap', () => {
        it('should keep only one type for each type of WE, with the right type and description', () => {
            const workingEnvironments = [we1, we2, ha];
            j.scope.resetCapacityByTypeMap(workingEnvironments);
            expect(j.scope.capacityByTypeMap).toEqual({
                'VSA': {
                    '_normalizedReservedSizeTB': {size: 0, unit: 'TB'},
                    type: 'VSA',
                    description: 'Cloud Volumes ONTAP',
                    count: 0
                },
                'HA': {
                    '_normalizedReservedSizeTB': {size: 0, unit: 'TB'},
                    type: 'HA',
                    description: 'Cloud Volumes ONTAP HA',
                    count: 0
                }
            });
        });

        it('should keep the right types for different working environment types along with the right type and description', () => {
            we1.workingEnvironmentType = 'ON_PREM';
            we1._workingEnvironmentTypeByLocation = 'ON_PREM';
            we2.workingEnvironmentType = 'ON_PREM';
            we2._workingEnvironmentTypeByLocation = 'NPS';
            const workingEnvironments = [we1, we2, ha];
            j.scope.resetCapacityByTypeMap(workingEnvironments);
            expect(j.scope.capacityByTypeMap).toEqual({
                'ON_PREM': {
                    '_normalizedReservedSizeTB': {size: 0, unit: 'TB'},
                    count: 0,
                    type: 'ON_PREM',
                    description: 'On-Premises'
                },
                'HA': {
                    '_normalizedReservedSizeTB': {size: 0, unit: 'TB'},
                    type: 'HA',
                    description: 'Cloud Volumes ONTAP HA',
                    count: 0
                },
                'NPS': {'_normalizedReservedSizeTB': {size: 0, unit: 'TB'}, count: 0, type: 'NPS', description: 'NPS'}
            });
        });

        it('should empty the map when the list of working environments is empty', () => {
            j.scope.resetCapacityByTypeMap([]);
            expect(j.scope.capacityByTypeMap).toEqual({});
        });

        it('should empty the map when the list of working environments is undefined', () => {
            j.scope.resetCapacityByTypeMap();
            expect(j.scope.capacityByTypeMap).toEqual({});
        });
    });

    describe('onDrop', () => {
        it('should allow drop for allowed WEs', () => {
            j.scope.checkAndSetState = jest.fn();
            const source = {status: {status: 'on'}, workingEnvironmentType: 'VSA', _typeAndProvider: 'VSA_AWS'};
            const target = {
                status: {status: 'on'},
                workingEnvironmentType: 'VSA',
                _typeAndProvider: 'VSA_AWS',
                ontapClusterProperties: {ontapVersion: '8.3'}
            };
            j.scope.onDrop(source, target);
            expect(j.scope.checkAndSetState).toHaveBeenCalled();
        });

        it('should allow drop for onPrem -> onPrem', () => {
            j.scope.checkAndSetState = jest.fn();
            const source = {status: {status: 'on'}, workingEnvironmentType: 'ON_PREM', _typeAndProvider: 'ON_PREM'};
            const target = {
                status: {status: 'on'},
                workingEnvironmentType: 'ON_PREM',
                _typeAndProvider: 'ON_PREM',
                ontapClusterProperties: {ontapVersion: '8.3'}
            };
            j.scope.onDrop(source, target);
            expect(j.scope.checkAndSetState).toHaveBeenCalled();
        });
    });

    describe('updateCapacityByTypeMap', () => {
        beforeEach(() => {
            j.scope.resetCapacityByTypeMap([we1, we3, ha]);
        });

        it('should update the NewFAS size when it is a dataONTAP which is not ON_PREM', () => {
            ha._normalizedReservedSizeTB.size = 100;
            j.scope.updateCapacityByTypeMap(ha);
            expect(j.scope.capacityByTypeMap['HA']._normalizedReservedSizeTB.size).toEqual(100);
            expect(j.scope.capacityByTypeMap['VSA']._normalizedReservedSizeTB.size).toEqual(0);
        });

        it('should add the new we value and subtract the old we value in case new and old are provided', () => {
            const newWe3 = _.cloneDeep(we3);
            newWe3._normalizedReservedSizeTB.size = 100;
            j.scope.updateCapacityByTypeMap(newWe3, we3);
            expect(j.scope.capacityByTypeMap['NPS']._normalizedReservedSizeTB.size).toEqual(100);
        });
    });

    describe('gotoCreateWE', () => {
        beforeEach(() => {
            j.scope.checkAndSetState = jest.fn();
            j.$state.current.name = 'name1';
        });

        it('should call $state.go with correct parameters for brand new WE (no params)', () => {
            j.scope.gotoCreateWE();

            expect(j.scope.checkAndSetState).toHaveBeenCalledWith('wizard', {
                from: 'name1',
                id: undefined,
                type: undefined,
                pathPrefix: undefined
            });
        });

        it('should call $state.go with correct parameters for recreate WE (with params)', () => {
            j.scope.gotoCreateWE(null, 'pid1', '/aws/ha/');

            expect(j.scope.checkAndSetState).toHaveBeenCalledWith('wizard', {
                from: 'name1',
                id: 'pid1',
                type: 'HA_AWS',
                pathPrefix: '/aws/ha/',
                startDiscoverWizard: null
            });
        });
    });
});
