describe('Key Manager Controller', () => {
    'use strict';
    var j;    beforeEach(() => {        j = magnifispec.setup();    });

    let constants, nocEncryptionResourcesService, nocEncryptionResourcesServiceSpy;

    beforeEach(inject((Constants, _nocEncryptionResourcesService_) => {
        constants = Constants;
        nocEncryptionResourcesService = _nocEncryptionResourcesService_;
        j.$httpBackend.expectGET(j.nocBaseUrl + '/occm/config').respond(200, {keyManagerPort: 1234});
        j.$httpBackend.expectGET(j.nocBaseUrl + '/occm/encryption/key-managers').respond(200, [{uninitializedValue: 'uninitializedValue'}]);
        nocEncryptionResourcesServiceSpy = jest.spyOn(nocEncryptionResourcesService, 'getKeyManagersCaCertificates').mockImplementation(
            magnifispec.mockPromise([{
                publicId: 'initialized_value2',
                endDate: 'today',
                _expirationState: 'mocked_state'
            }])
        );
        magnifispec.mockTabsController('KeyManagerController');
        j.$httpBackend.flush();
        magnifispec.processPromise();
    }));

    describe('initialize', () => {
        it('should call init methods', () => {
            jest.spyOn(j.scope, 'getKeyManagers').mockImplementation(() => {});
            jest.spyOn(j.scope, 'getKeyManagersCaCertificates').mockImplementation(() => {});
            jest.spyOn(j.scope, 'getKeyManagerPort').mockImplementation(() => {});

            j.scope.initialize();

            expect(j.scope.getKeyManagers).toHaveBeenCalled();
            expect(j.scope.getKeyManagersCaCertificates).toHaveBeenCalled();
            expect(j.scope.getKeyManagerPort).toHaveBeenCalled();
            expect(j.scope.usernameFields).toEqual(['', 'CommonName', 'EmailAddress', 'OrganizationalUnitName']);
        });
    });

    describe('getKeyManagerPort', () => {
        it('should work upon success', () => {
            j.$httpBackend.expectGET(j.nocBaseUrl + '/occm/config').respond(200, {
                keyManagerPort: 4321
            });

            j.scope.getKeyManagerPort();
            j.$httpBackend.flush();

            expect(j.scope.keyManagerPort).toEqual(4321);
            expect(j.scope.originalKeyManagerPort).toEqual(4321);
        });

        describe('should overcome wrong values', () => {
            it('should error for too big values', () => {
                j.$httpBackend.expectGET(j.nocBaseUrl + '/occm/config').respond(200, {
                    keyManagerPort: 65536
                });
            });

            it('should error for too small values', () => {
                j.$httpBackend.expectGET(j.nocBaseUrl + '/occm/config').respond(200, {
                    keyManagerPort: 0
                });
            });

            afterEach(() => {
                j.scope.getKeyManagerPort();
                j.$httpBackend.flush();

                expect(j.pidgeon.error).toHaveBeenCalledWith('There was a problem with the key manager port. Please reconfigure it.', null, 'keyManager');
            });
        });

        it('should work upon error', () => {
            j.$httpBackend.expectGET(j.nocBaseUrl + '/occm/config').respond(400, j.errorResponse);

            j.scope.getKeyManagerPort();
            j.$httpBackend.flush();

            expect(j.pidgeon.error).toHaveBeenCalled();
        });

        afterEach(() => {
            expect(j.pidgeon.clear).toHaveBeenCalled();
            expect(j.nocUI.loading).toHaveBeenCalled();
            expect(j.nocUI.unloading).toHaveBeenCalled();
        });
    });

    describe('saveKeyManagerPort', () => {
        it('should validate the form', () => {
            j.scope.keyManagerPortForm = {$valid: false};
            j.scope.saveKeyManagerPort();
            expect(j.nocUI.handleFormValidity).toHaveBeenCalled();
        });

        describe('when form is valid', () => {
            beforeEach(() => {
                j.scope.keyManagerPortForm = {$valid: true};
            });

            it('should work upon success', () => {
                j.scope.portChangeMode = true;
                j.$httpBackend.expectPUT(j.nocBaseUrl + '/occm/config', {
                    'keyManagerPort': 4321
                }).respond(200);

                j.scope.keyManagerPort = 4321;
                j.scope.saveKeyManagerPort();
                j.$httpBackend.flush();

                expect(j.scope.originalKeyManagerPort).toEqual(4321);
                expect(j.pidgeon.success).toHaveBeenCalledWith('Key manager port updated to 4321.');
                expect(j.scope.portChangeMode).toEqual(false);
            });

            it('should work upon error', () => {
                j.$httpBackend.expectPUT(j.nocBaseUrl + '/occm/config', {
                    'keyManagerPort': 4321
                }).respond(400, j.errorResponse);

                j.scope.keyManagerPort = 4321;
                j.scope.saveKeyManagerPort();
                j.$httpBackend.flush();

                expect(j.pidgeon.error).toHaveBeenCalled();
            });

            afterEach(() => {
                expect(j.pidgeon.clear).toHaveBeenCalled();
                expect(j.nocUI.loading).toHaveBeenCalled();
                expect(j.nocUI.unloading).toHaveBeenCalled();
            });
        });
    });

    describe('getKeyManagers', () => {
        let alteredKeyManagers;
        beforeEach(() => {
            j.scope.gridOptionsKeyManagers = {};
            alteredKeyManagers = [
                {name: 'a', address: '7.8.9.10', usernameField: 'Street', usernameValue: 'Sesame', _usernameString: 'Sesame (in field Street)'},
                {name: 'b', address: '11.12.13.14', _usernameString: undefined}
            ];
        });
        it('should work upon success', () => {
            const keyManagers = [
                {name: 'a', address: '7.8.9.10', usernameField: 'Street', usernameValue: 'Sesame'},
                {name: 'b', address: '11.12.13.14'}
            ];
            j.$httpBackend.expectGET(j.nocBaseUrl + '/occm/encryption/key-managers').respond(200, keyManagers);

            j.scope.getKeyManagers();
            j.$httpBackend.flush();

            expect(j.scope.keyManagers).toEqual(alteredKeyManagers);
            expect(j.scope.gridOptionsKeyManagers.data).toEqual(alteredKeyManagers);
        });

        it('should work upon error', () => {
            j.$httpBackend.expectGET(j.nocBaseUrl + '/occm/encryption/key-managers').respond(400, j.errorResponse);

            j.scope.getKeyManagers();
            j.$httpBackend.flush();

            expect(j.scope.keyManagers).toEqual([
                {uninitializedValue: 'uninitializedValue', _usernameString: undefined}
            ]);
            expect(j.scope.gridOptionsKeyManagers.data).not.toEqual(alteredKeyManagers);
            expect(j.pidgeon.error).toHaveBeenCalled();
        });

        afterEach(() => {
            expect(j.pidgeon.clear).toHaveBeenCalled();
            expect(j.nocUI.loading).toHaveBeenCalled();
            expect(j.nocUI.unloading).toHaveBeenCalled();
        });
    });

    describe('addOrEditKeyManager', () => {
        let finish;

        beforeEach(() => {
            jest.spyOn(j.scope, 'getKeyManagers').mockImplementation(() => {});
            finish = magnifispec.fakeDialog();
            j.scope.usernameFields = ['val1', 'val2', 'val3'];
        });

        it('Add - should call dialog with correct locals and refresh the list upon ok', () => {
            j.scope.addOrEditKeyManager();
            finish();

            expect(j.nocDialog.dialog).toHaveBeenCalledWith('Add Key Manager', 'assets/templates/tabs/encryption/add-or-edit-key-manager.html', {
                usernameFields: ['val1', 'val2', 'val3']
            });
        });

        it('Edit - should call dialog with correct locals and refresh the list upon ok', () => {
            j.scope.addOrEditKeyManager({
                publicId: 'pid1',
                name: 'name1',
                address: '3.4.5.6',
                usernameField: 'country',
                usernameValue: 'Bhutan'
            });
            finish();

            expect(j.nocDialog.dialog).toHaveBeenCalledWith('Edit Key Manager', 'assets/templates/tabs/encryption/add-or-edit-key-manager.html', {
                usernameFields: ['val1', 'val2', 'val3'],
                publicId: 'pid1',
                name: 'name1',
                address: '3.4.5.6',
                usernameField: 'country',
                usernameValue: 'Bhutan'
            });
        });
        afterEach(() => {
            expect(j.scope.getKeyManagers).toHaveBeenCalled();
        });
    });

    describe('deleteKeyManager', () => {
        let keyManager;
        beforeEach(() => {
            keyManager = {_pathPrefix: '/vsa/', publicId: 'a', name: 'b'};
            jest.spyOn(j.scope, 'getKeyManagers').mockImplementation(() => {});
        });

        it('should do nothing when the user clicks cancel', () => {
            magnifispec.mockConfirm(false);

            j.scope.deleteKeyManager(keyManager);
            magnifispec.processPromise();

            expect(j.scope.getKeyManagers).not.toHaveBeenCalled();
            expect(j.pidgeon.error).not.toHaveBeenCalled();
        });

        describe('should call API for deleting if the user clicks OK', () => {
            beforeEach(() => {
                magnifispec.mockConfirm(true);
            });

            it('should work upon success', () => {
                j.$httpBackend.expectDELETE(j.nocBaseUrl + '/occm/encryption/key-managers/' + 'a').respond(200);

                j.scope.deleteKeyManager(keyManager);
                j.$httpBackend.flush();

                expect(j.scope.getKeyManagers).toHaveBeenCalled();
            });

            it('should work upon error', () => {
                j.$httpBackend.expectDELETE(j.nocBaseUrl + '/occm/encryption/key-managers/' + 'a').respond(400, j.errorResponse);

                j.scope.deleteKeyManager(keyManager);
                j.$httpBackend.flush();

                expect(j.pidgeon.error).toHaveBeenCalled();
            });

            afterEach(() => {
                expect(j.pidgeon.clear).toHaveBeenCalled();
                expect(j.nocUI.loading).toHaveBeenCalled();
                expect(j.nocUI.unloading).toHaveBeenCalled();
            });
        });

        afterEach(() => {
            expect(j.nocDialog.confirm).toHaveBeenCalledWith('Delete Key Manager?', 'assets/templates/tabs/encryption/delete-key-manager-dialog.html', {keyManagerName: 'b'});
        });
    });

    describe('getKeyManagersCaCertificates', () => {
        beforeEach(() => {
            j.scope.gridOptionsKeyManagersCaCertificate = {};
        });

        it('should work upon success', () => {
            nocEncryptionResourcesServiceSpy.mockImplementation(magnifispec.mockPromise([
                {publicId: 'id1', endDate: 'tomorrow', _expirationState: 'mocked_state'},
                {publicId: 'id2', endDate: 'yesterday', _expirationState: 'mocked_state'}
            ]));

            j.scope.getKeyManagersCaCertificates();
            magnifispec.processPromise();

            expect(j.scope.keyManagersCaCertificates).toEqual([
                {publicId: 'id1', endDate: 'tomorrow', _expirationState: 'mocked_state'},
                {publicId: 'id2', endDate: 'yesterday', _expirationState: 'mocked_state'}
            ]);
            expect(j.scope.gridOptionsKeyManagersCaCertificate.data).toEqual([
                {publicId: 'id1', endDate: 'tomorrow', _expirationState: 'mocked_state'},
                {publicId: 'id2', endDate: 'yesterday', _expirationState: 'mocked_state'}
            ]);
        });

        it('should work upon error', () => {
            nocEncryptionResourcesServiceSpy.mockImplementation(magnifispec.mockPromise());

            j.scope.getKeyManagersCaCertificates();
            magnifispec.processPromise();

            expect(j.scope.keyManagersCaCertificates).toEqual([{
                publicId: 'initialized_value2',
                endDate: 'today',
                _expirationState: 'mocked_state'
            }]);
            expect(j.scope.gridOptionsKeyManagersCaCertificate.data).not.toEqual('key_managers_ca_certificates_list');
        });

        afterEach(() => {
            expect(j.pidgeon.clear).toHaveBeenCalled();
            expect(j.nocUI.loading).toHaveBeenCalled();
            expect(j.nocUI.unloading).toHaveBeenCalled();
        });
    });

    describe('addKeyManagerCaCertificate', () => {
        it('should refresh the list upon ok', () => {
            jest.spyOn(j.scope, 'getKeyManagersCaCertificates').mockImplementation(() => {});
            const finish = magnifispec.fakeDialog();
            j.scope.addKeyManagerCaCertificate();
            finish();

            expect(j.nocDialog.dialog).toHaveBeenCalledWith('Add Key Manager CA certificate', 'assets/templates/tabs/encryption/add-key-manager-ca-certificate.html', {
                width: constants.dialogWidthForDisplayingCertificate
            });
            expect(j.scope.getKeyManagersCaCertificates).toHaveBeenCalled();
        });
    });

    describe('show key manager CA certificate', () => {
        it('should open the proper dialog', () => {
            jest.spyOn(document, 'getElementById').mockReturnValue({select: jest.fn()});
            const certificate = {a: 'b', certificate: 'value333'};
            magnifispec.mockConfirm(true);

            j.scope.showKeyManagerCaCertificates(certificate);

            expect(j.nocDialog.confirm).toHaveBeenCalledWith('Key Manager CA Certificate', 'assets/templates/tabs/encryption/show-certificate.html', {
                certificate: 'value333',
                width: constants.dialogWidthForDisplayingCertificate,
                callback: j.scope.selectCertificateText
            });
        });
    });

    describe('deleteKeyManagerCaCertificates', () => {
        let keyManagerCaCertificate;
        beforeEach(() => {
            keyManagerCaCertificate = {publicId: 'c', name: 'd'};
            jest.spyOn(j.scope, 'getKeyManagersCaCertificates').mockImplementation(() => {});
        });

        it('should do nothing when the user clicks cancel', () => {
            magnifispec.mockConfirm(false);

            j.scope.deleteKeyManagerCaCertificates(keyManagerCaCertificate);
            magnifispec.processPromise();

            expect(j.scope.getKeyManagersCaCertificates).not.toHaveBeenCalled();
            expect(j.pidgeon.error).not.toHaveBeenCalled();
        });

        describe('should call API for deleting if the user clicks OK', () => {
            beforeEach(() => {
                magnifispec.mockConfirm(true);
            });

            it('should work upon success', () => {
                j.$httpBackend.expectDELETE(j.nocBaseUrl + '/occm/encryption/key-managers-ca-certificates/' + 'c').respond(200);

                j.scope.deleteKeyManagerCaCertificates(keyManagerCaCertificate);
                j.$httpBackend.flush();

                expect(j.scope.getKeyManagersCaCertificates).toHaveBeenCalled();
            });

            it('should work upon error', () => {
                j.$httpBackend.expectDELETE(j.nocBaseUrl + '/occm/encryption/key-managers-ca-certificates/' + 'c').respond(400, j.errorResponse);

                j.scope.deleteKeyManagerCaCertificates(keyManagerCaCertificate);
                j.$httpBackend.flush();

                expect(j.pidgeon.error).toHaveBeenCalled();
            });

            afterEach(() => {
                expect(j.pidgeon.clear).toHaveBeenCalled();
                expect(j.nocUI.loading).toHaveBeenCalled();
                expect(j.nocUI.unloading).toHaveBeenCalled();
            });
        });
    });
});
