describe('action required controller', () => {
    var j;
    beforeEach(() => {
        j = magnifispec.setup();
    });

    const workingEnvironment = {_pathPrefix: '/vsa/', publicId: '1', instanceId: 'id'};
    const parentScope = {
        selectedWorkingEnvironment: workingEnvironment,
        actionsRequired: {
            '1': [
                {
                    'actionType': 'ShouldBeRemoved',
                    'desc': 'something',
                    'severity': 'warning'
                },
                {
                    'actionType': 'NeedAddDisks',
                    'desc': 'something',
                    'severity': 'error'
                }
            ]
        }
    };

    beforeEach(() => {
        magnifispec.mockConfirm(true);
        magnifispec.mockTabsController('ActionRequiredController', {selectedWorkingEnvironment: workingEnvironment}, parentScope);
    });

    it('should initialize correctly', () => {
        expect(j.scope.actionsRequired).toEqual(j.scope.actionsRequiredStatic);
        expect(j.scope.selectedWEActionsRequired).toEqual([ //Should be sorted by severity
            {
                'actionType': 'NeedAddDisks',
                'desc': 'something',
                'severity': 'error'
            },
            {
                'actionType': 'ShouldBeRemoved',
                'desc': 'something',
                'severity': 'warning'
            }
        ]);
    });

    it('should go back to monitoring if no action required', () => {
        const parentScope = {
            selectedWorkingEnvironment: workingEnvironment,
            showMonitoring: jest.fn(),
            actionsRequired: {}
        };

        magnifispec.mockTabsController('ActionRequiredController', {selectedWorkingEnvironment: workingEnvironment}, parentScope);
        expect(j.scope.showMonitoring).toHaveBeenCalledWith();
    });

    describe('isFixable', () => {
        it('should tell if something is fixable', () => {
            expect(j.scope.isFixable({actionType: 'ShouldBeRemoved'})).toEqual(true);
            expect(j.scope.isFixable({actionType: 'NeedsMoreDisks'})).toEqual(true);
            expect(j.scope.isFixable({actionType: 'MoreDisksNeededButDiskLimitExceeded', parameters: {}})).toBeFalsy();
            expect(j.scope.isFixable({actionType: 'MoreDisksNeededButVsaLimitExceeded', parameters: {}})).toBeFalsy();
            expect(j.scope.isFixable({
                actionType: 'MoreDisksNeededButVsaLimitExceeded',
                parameters: {licenseParameters: 'some parameters'}
            })).toBeDefined();
            expect(j.scope.isFixable({actionType: 'LicenseUsedCapacityWarning', parameters: {}})).toBeFalsy();
            expect(j.scope.isFixable({
                actionType: 'LicenseUsedCapacityWarning',
                parameters: {licenseParameters: 'some parameters'}
            })).toBeDefined();
            expect(j.scope.isFixable({actionType: 'licenseExpirationWarning'})).toEqual(false);
            expect(j.scope.isFixable({actionType: 'licenseGracePeriod'})).toEqual(false);
            expect(j.scope.isFixable({actionType: 'licenseWasSuspended'})).toEqual(false);
            expect(j.scope.isFixable({actionType: 'SomeCrazyShit'})).toEqual(false);
        });
    });

    describe('Open review dialog', () => {
        beforeEach(() => {
            jest.spyOn(j.nocDialog, 'dialog').mockImplementation(() => {
            });
        });

        it('should open a dialog', () => {
            j.scope.openReviewDialog({
                actionType: 'ShouldBeRemoved',
                parameters: {
                    aggregateName: 'name'
                }
            });
            expect(j.nocDialog.confirm).toHaveBeenCalledWith('', 'assets/templates/dialogs/review-action-required.html', {
                action: {
                    actionType: 'ShouldBeRemoved',
                    parameters: {
                        aggregateName: 'name'
                    }
                },
                we: undefined,
                isFixable: j.scope.isFixable,
                hasAdvancedInformation: true,
                secondMessage: ''
            });
        });

        it('should add second message when about to delete encrypted aggregate', () => {
            j.scope.selectedWorkingEnvironment.ontapEncryption = true;
            j.scope.openReviewDialog({
                actionType: 'ShouldBeRemoved',
                parameters: {
                    aggregateName: 'name'
                }
            });
            expect(j.nocDialog.confirm).toHaveBeenCalledWith('', 'assets/templates/dialogs/review-action-required.html', {
                action: {
                    actionType: 'ShouldBeRemoved',
                    parameters: {
                        aggregateName: 'name'
                    }
                },
                isFixable: j.scope.isFixable,
                we: undefined,
                hasAdvancedInformation: true,
                secondMessage: 'The encryption key used for this aggregate is listed in the timeline. You will need to manually delete the key from key managers.'
            });
        });
    });

    describe('fix', () => {
        beforeEach(() => {
            jest.spyOn(j.scope, 'addDisks').mockImplementation(() => {});
            jest.spyOn(j.scope, 'deleteAggregate').mockImplementation(() => {});
            jest.spyOn(j.scope, 'moveVolume').mockImplementation(() => {});
            jest.spyOn(j.scope, 'changeLicense').mockImplementation(() => {});
        });

        it('should call the right action', () => {
            const addDisksAction = {
                actionType: 'NeedsMoreDisks',
                parameters: {
                    numberOfDisks: 1,
                    aggregateName: 'aggr1'
                }
            };
            j.scope.fix(addDisksAction);
            expect(j.scope.addDisks).toHaveBeenCalledWith(addDisksAction);
        });

        it('should call the remove aggregates sometimes', () => {
            const removeAggAction = {
                actionType: 'ShouldBeRemoved',
                parameters: {
                    aggregateName: 'aggr1'
                }
            };
            j.scope.fix(removeAggAction);
            expect(j.scope.deleteAggregate).toHaveBeenCalledWith(removeAggAction);
        });

        it('should call the move volume when needed', () => {
            const removeAggAction = {
                actionType: 'MoreDisksNeededButDiskLimitExceeded',
                parameters: {
                    volumeMoveParameters: 'aggr1'
                }
            };
            j.scope.fix(removeAggAction);
            expect(j.scope.moveVolume).toHaveBeenCalledWith(removeAggAction);
        });

        it('should change license for MoreDisksNeededButVsaLimitExceeded', () => {
            const action = {
                actionType: 'MoreDisksNeededButVsaLimitExceeded',
                parameters: {
                    licenseParameters: 'params1'
                }
            };

            j.scope.fix(action);

            expect(j.scope.changeLicense).toHaveBeenCalledWith(action);
        });

        it('should change license for LicenseUsedCapacityWarning', () => {
            const action = {
                actionType: 'LicenseUsedCapacityWarning',
                parameters: {
                    licenseParameters: 'params1'
                }
            };

            j.scope.fix(action);

            expect(j.scope.changeLicense).toHaveBeenCalledWith(action);
        });

        it('should change license for NonCompliantConfig', () => {
            const action = {
                actionType: 'NonCompliantConfig',
                parameters: {
                    licenseParameters: 'params1'
                }
            };

            j.scope.fix(action);

            expect(j.scope.changeLicense).toHaveBeenCalledWith(action);
        });
    });

    describe('add disks', () => {
        beforeEach(() => {
            jest.spyOn(j.scope, 'completeActionRequiredHandling').mockImplementation(() => {
            });
        });
        it('should send a request if told to', () => {
            j.scope.addDisks({
                parameters: {
                    numOfDisks: 1,
                    aggregateName: 'aggr'
                }
            });

            j.$httpBackend.expectPOST(j.nocBaseUrl + '/vsa/aggregates/1/aggr/disks',
                {
                    numberOfDisks: 1,
                    name: 'aggr'
                }).respond(200);
            j.$httpBackend.flush();
            expect(j.pidgeon.clear).toHaveBeenCalled();
            expect(j.scope.completeActionRequiredHandling).toHaveBeenCalledWith({
                parameters: {
                    numOfDisks: 1,
                    aggregateName: 'aggr'
                }
            });
        });

        it('should fail gracefully', () => {
            j.scope.addDisks({
                parameters: {
                    numOfDisks: 1,
                    aggregateName: 'aggr'
                }
            });

            j.$httpBackend.expectPOST(j.nocBaseUrl + '/vsa/aggregates/1/aggr/disks',
                {
                    numberOfDisks: 1,
                    name: 'aggr'
                }).respond(400, j.errorResponse);
            j.$httpBackend.flush();
            expect(j.pidgeon.error).toHaveBeenCalled();
        });
    });

    describe('delete aggregate', () => {
        beforeEach(() => {
            jest.spyOn(j.scope, 'completeActionRequiredHandling').mockImplementation(() => {
            });
        });
        it('should send a request if told to', () => {
            j.scope.deleteAggregate({
                parameters: {
                    aggregateName: 'aggr'
                }
            });

            j.$httpBackend.expectDELETE(j.nocBaseUrl + '/vsa/aggregates/1/aggr').respond(200);
            j.$httpBackend.flush();
            expect(j.pidgeon.clear).toHaveBeenCalled();
            expect(j.scope.completeActionRequiredHandling).toHaveBeenCalledWith({parameters: {aggregateName: 'aggr'}});
        });

        it('should fail gracefully', () => {
            j.scope.deleteAggregate({
                parameters: {
                    numOfDisks: 1,
                    aggregateName: 'aggr'
                }
            });

            j.$httpBackend.expectDELETE(j.nocBaseUrl + '/vsa/aggregates/1/aggr').respond(400, j.errorResponse);
            j.$httpBackend.flush();
            expect(j.pidgeon.error).toHaveBeenCalled();
        });
    });

    describe('moveVolume', () => {
        beforeEach(() => {
            jest.spyOn(j.scope, 'completeActionRequiredHandling').mockImplementation(() => {
            });
        });
        it('should send a request if told to', () => {
            j.scope.moveVolume({
                parameters: {
                    volumeMoveParameters: {
                        svmName: 'svm1',
                        volumeName: 'volume2',
                        isNewTargetAggregate: true,
                        targetAggregateName: 'targetName1'
                    },
                    numOfDisks: 2
                }
            });

            j.$httpBackend.expectPOST(j.nocBaseUrl + '/vsa/volumes/1/svm1/volume2/move').respond(200);
            j.$httpBackend.flush();
            expect(j.scope.completeActionRequiredHandling).toHaveBeenCalledWith({
                parameters: {
                    volumeMoveParameters: {
                        svmName: 'svm1',
                        volumeName: 'volume2',
                        isNewTargetAggregate: true,
                        targetAggregateName: 'targetName1'
                    },
                    numOfDisks: 2
                }
            });
        });

        it('should fail gracefully', () => {
            j.scope.moveVolume({
                parameters: {
                    volumeMoveParameters: {
                        svmName: 'svm1',
                        volumeName: 'volume2',
                        isNewTargetAggregate: true,
                        targetAggregateName: 'targetName1'
                    },
                    numOfDisks: 2
                }
            });

            j.$httpBackend.expectPOST(j.nocBaseUrl + '/vsa/volumes/1/svm1/volume2/move').respond(400, j.errorResponse);
            j.$httpBackend.flush();
            expect(j.scope.completeActionRequiredHandling).not.toHaveBeenCalled();
            expect(j.pidgeon.error).toHaveBeenCalled();
        });
    });

    describe('reviewable', () => {
        it('should return the right answer for each action type', () => {
            expect(j.scope.reviewable({actionType: 'ShouldBeRemoved'})).toEqual(true);
            expect(j.scope.reviewable({actionType: 'NeedsMoreDisks'})).toEqual(true);
            expect(j.scope.reviewable({actionType: 'MoreDisksNeededButDiskLimitExceeded'})).toEqual(true);
            expect(j.scope.reviewable({actionType: 'MoreDisksNeededButVsaLimitExceeded'})).toEqual(true);
            expect(j.scope.reviewable({actionType: 'LicenseUsedCapacityWarning'})).toEqual(true);
            expect(j.scope.reviewable({actionType: 'licenseExpirationWarning'})).toEqual(true);
            expect(j.scope.reviewable({actionType: 'licenseGracePeriod'})).toEqual(true);
            expect(j.scope.reviewable({actionType: 'licenseWasSuspended'})).toEqual(true);
            expect(j.scope.reviewable({actionType: 'RestrictedAggregatesCanBeFixed'})).toEqual(false);
            expect(j.scope.reviewable({actionType: 'RestrictedAggregatesCannotBeFixed'})).toEqual(false);
            expect(j.scope.reviewable({actionType: 'NonCompliantConfig'})).toEqual(true);
        });
    });

    describe('completeActionRequiredHandling', () => {
        const action1 = {
            parameters: {
                numOfDisks: 1,
                aggregateName: 'aggr1'
            }
        };
        const action2 = {
            parameters: {
                numOfDisks: 1,
                aggregateName: 'aggr1'
            }
        };

        beforeEach(() => {
            j.scope.selectedWorkingEnvironment = {_pathPrefix: '/vsa/', publicId: 1};
            j.scope.showMonitoring = jest.fn();
        });

        it('should remove action2 and clone deep the remaining action and go back to monitoring although there are more actions, to allow recalculating the actions', () => {
            j.scope.actionsRequired[1] = [action1, action2];
            j.scope.completeActionRequiredHandling(action1);
            expect(j.scope.actionsRequired).toEqual({1: [action2]});
            expect(j.scope.actionsRequiredStatic[1].length).toEqual(1);
            expect(j.scope.showMonitoring).toHaveBeenCalled();
        });

        it('should remove the one action and go back to monitoring', () => {
            j.scope.actionsRequired[1] = [action1];
            j.scope.completeActionRequiredHandling(action1);
            expect(j.scope.actionsRequired).toEqual({});
            expect(j.scope.actionsRequiredStatic).toEqual({});
            expect(j.scope.showMonitoring).toHaveBeenCalled();
        });
    });
});
