describe('Monitoring Controller', () => {
    var j;    beforeEach(() => {        j = magnifispec.setup();    });
    let volumesResource;
    let workingEnvironment;
    let getWorkingEnvironmentByPublicIdSpy;

    beforeEach(inject((_volumesResource_) => {
        workingEnvironment = {
            _pathPrefix: '/vsa/',
            publicId: '1',
            name: 'we1',
            _cloudProviderName: 'AWS',
            _allowedOperations: {resources: true}
        };
        volumesResource = _volumesResource_;
        jest.spyOn(j.workingEnvironmentResource, 'startStatusPolling').mockReturnValue(jest.fn());
        jest.spyOn(j.workingEnvironmentResource, 'initWorkingEnvironment').mockImplementation(() => {});
        jest.spyOn(volumesResource, 'startVolumesPolling').mockReturnValue(jest.fn());
        getWorkingEnvironmentByPublicIdSpy = jest.spyOn(j.workingEnvironmentResource, 'getWorkingEnvironmentByPublicId').mockReturnValue(j.$q.when(workingEnvironment));
        j.$rootScope.instancePlacement = {installLocation: 'AWS'};
        magnifispec.mockTabsController('MonitoringController', {}, {
            goToGadget: jest.fn(),
            showMonitoring: jest.fn()
        });
        magnifispec.processPromise();
    }));

    it('should go back to gallery if the selected working environment is not allowed to be in resources', () => {
        j.$state.params.workingEnvironmentId = '1';
        getWorkingEnvironmentByPublicIdSpy.mockReturnValue(j.$q.when({
            publicId: '1',
            name: 'we1',
            status: {status: 'on'},
            _allowedOperations: {}
        }));

        magnifispec.mockTabsController('MonitoringController', {}, {
            backToGallery: jest.fn()
        });
        magnifispec.processPromise();

        expect(j.parentScope.backToGallery).toHaveBeenCalled();
    });

    it('should startStatusPolling in case of not ON_PREM', () => {
        j.scope.selectedWorkingEnvironment = {
            _allowedOperations: {resources: true},
            workingEnvironmentType: 'VSA',
            _cloudProviderName: 'AWS',
            ontapClusterProperties: {}
        };
        j.scope.vsaResponse = {};
        j.scope.initializeController();
        expect(j.workingEnvironmentResource.startStatusPolling).toHaveBeenCalledWith(j.scope.selectedWorkingEnvironment);
    });


    describe('initializeShutdownSchedules', () => {
        let schedulesURL;
        beforeEach(() => {
            j.scope.selectedWorkingEnvironment = {
                _allowedOperations: {resources: true},
                workingEnvironmentType: 'VSA',
                _pathPrefix: '/vsa/',
                publicId: 'we1'
            };
            schedulesURL = '/occm/api/vsa/working-environments/we1/schedules';
        });

        it('should update schedules upon success', () => {
            j.$httpBackend.expectGET(schedulesURL).respond(200, [{name: 'schedule1'}, {name: 'schedule2'}]);

            j.scope.initializeShutdownSchedules();
            j.$httpBackend.flush();
            expect(j.scope.selectedWorkingEnvironment._originalSchedules).toEqual([{name: 'schedule1'}, {name: 'schedule2'}]);
        });

        it('should show an error upon error', () => {
            j.$httpBackend.expectGET(schedulesURL).respond(400, j.errorResponse);

            j.scope.initializeShutdownSchedules();
            j.$httpBackend.flush();

            expect(j.scope.selectedWorkingEnvironment._originalSchedules).toBeUndefined();
            expect(j.pidgeon.error).toHaveBeenCalled();
        });
    });

    describe('getEnabledSchedules', () => {
        it('should return the list of names of enabled schedules when there are enabled schedules', () => {
            j.scope.selectedWorkingEnvironment._originalSchedules = [{name: 'schedule1', enabled: true}, {
                name: 'schedule2',
                enabled: true
            }, {name: 'schedule3', enabled: false}];
            expect(j.scope.getEnabledSchedules()).toEqual(['schedule1', 'schedule2']);
        });

        it('should return an empty array when none is enabled', () => {
            j.scope.selectedWorkingEnvironment._originalSchedules = [{
                name: 'schedule1',
                enabled: false
            }, {name: 'schedule2', enabled: false}, {name: 'schedule3', enabled: false}];
            expect(j.scope.getEnabledSchedules()).toEqual([]);
        });
    });

    describe('getScheduleDowntimeTitle', () => {
        it('should return schedule downtime when no schedule is defined', () => {
            jest.spyOn(j.scope, 'getEnabledSchedules').mockReturnValue([]);
            expect(j.scope.getScheduleDowntimeTitle()).toEqual('Schedule Downtime');
        });

        it('should return the right title when at least one schedule is defined', () => {
            jest.spyOn(j.scope, 'getEnabledSchedules').mockReturnValue(['Weekdays']);
            expect(j.scope.getScheduleDowntimeTitle()).toEqual('Scheduled Downtime: Weekdays');
        });

        it('should return a comma separated list of schedules when more than one schedule is defined', () => {
            jest.spyOn(j.scope, 'getEnabledSchedules').mockReturnValue(['Weekdays', 'Weekend']);
            expect(j.scope.getScheduleDowntimeTitle()).toEqual('Scheduled Downtime: Weekdays, Weekend');
        });
    });

    describe('isSuspended', () => {
        beforeEach(() => {
            j.scope.actionsRequired = {};
        });

        it('should return true if license was suspended', () => {
            j.scope.actionsRequired[j.scope.selectedWorkingEnvironment.publicId] = [{actionType: 'licenseWasSuspended'}, {actionType: 'licenseGracePeriod'}];
            expect(j.scope.isSuspended()).toEqual(true);
        });

        it('should return false if license is not suspended', () => {
            j.scope.actionsRequired[j.scope.selectedWorkingEnvironment.publicId] = [{actionType: 'licenseExpirationWarning'}, {actionType: 'licenseGracePeriod'}];
            expect(j.scope.isSuspended()).toEqual(false);
        });
    });

    describe('showEnabledSchedulesIcon', () => {
        it('should return false there are no enabled schedules even if the status is on', () => {
            j.scope.selectedWorkingEnvironment._originalSchedules = [{
                name: 'schedule1',
                enabled: false
            }, {name: 'schedule2', enabled: false}, {name: 'schedule3', enabled: false}];
            j.scope.selectedWorkingEnvironment.status = {status: 'on'};
            expect(j.scope.showEnabledSchedulesIcon()).toEqual(false);
        });

        it('should return false when there are enabled schedules but the status is failed', () => {
            j.scope.selectedWorkingEnvironment._originalSchedules = [{
                name: 'schedule1',
                enabled: true
            }, {name: 'schedule2', enabled: true}, {name: 'schedule3', enabled: false}];
            j.scope.selectedWorkingEnvironment.status = {status: 'failed'};
            expect(j.scope.showEnabledSchedulesIcon()).toEqual(false);
        });

        it('should return false when there are enabled schedules but the status is not-found', () => {
            j.scope.selectedWorkingEnvironment._originalSchedules = [{
                name: 'schedule1',
                enabled: true
            }, {name: 'schedule2', enabled: true}, {name: 'schedule3', enabled: false}];
            j.scope.selectedWorkingEnvironment.status = {status: 'not-found'};
            expect(j.scope.showEnabledSchedulesIcon()).toEqual(false);
        });

        it('should return true when there are enabled schedules and the status is on', () => {
            j.scope.selectedWorkingEnvironment._originalSchedules = [{
                name: 'schedule1',
                enabled: true
            }, {name: 'schedule2', enabled: true}, {name: 'schedule3', enabled: false}];
            j.scope.selectedWorkingEnvironment.status = {status: 'on'};
            expect(j.scope.showEnabledSchedulesIcon()).toEqual(true);
        });
    });

    describe('$on WE_STATUS event', () => {
        it('WE_STATUS_GET_EVENT should update the status on the working environment and re-init', () => {
            j.scope.$broadcast('1' + j.workingEnvironmentResource.WE_STATUS_GET_EVENT, {status: 'new', message: null});

            expect(j.scope.selectedWorkingEnvironment.status).toEqual({status: 'new', message: null});
            expect(j.workingEnvironmentResource.initWorkingEnvironment).toHaveBeenCalledWith(j.scope.selectedWorkingEnvironment);
        });
        it('WE_STATUS_ERROR_EVENT should update the status on the working environment to deleted in case was deleting and status code is 404', () => {
            jest.spyOn(j.workingEnvironmentResource, 'updateWorkingEnvironmentStatus').mockImplementation(() => {});
            j.scope.selectedWorkingEnvironment.status = 'deleting';
            j.scope.$broadcast('1' + j.workingEnvironmentResource.WE_STATUS_ERROR_EVENT, {status: 404});
            expect(j.workingEnvironmentResource.updateWorkingEnvironmentStatus).toHaveBeenCalledWith(j.scope.selectedWorkingEnvironment, ('deleted'));
        });
    });

    describe('$on VOLUMES_GET_EVENT', () => {
        let initVolumesForMonitoringDisplay;
        beforeEach(() => {
            initVolumesForMonitoringDisplay = jest.spyOn(volumesResource, 'initVolumesForMonitoringDisplay').mockReturnValue([
                {name: 'vol1'},
                {name: 'vol2'}
            ]);
        });

        it('should do nothing if the payload belongs to different WE', () => {
            j.scope.$broadcast(volumesResource.VOLUMES_GET_EVENT, {
                workingEnvironment: {publicId: '2'},
                volumes: ['vol1weId2']
            });
            magnifispec.processPromise();

            expect(j.scope.volumes).not.toEqual(['vol1weId2']);
        });

        it('should update cloudSyncProperties and init volumes with it', () => {
            j.scope.selectedWorkingEnvironment._allowedOperations['sync-to-s3'] = true;
            jest.spyOn(j.scope, 'updateExtraProperties').mockImplementation((we) => {
                we.cloudSyncProperties = 'cloudSyncProperties';

                return j.$q.when();
            });

            j.scope.$broadcast(volumesResource.VOLUMES_GET_EVENT, {
                workingEnvironment: {publicId: '1'},
                volumes: 'volumes'
            });
            magnifispec.processPromise();

            expect(j.scope.updateExtraProperties).toHaveBeenCalledWith(j.scope.selectedWorkingEnvironment);
            expect(j.scope.volumes).toEqual([
                {name: 'vol1'},
                {name: 'vol2'}
            ]);
            expect(initVolumesForMonitoringDisplay).toHaveBeenCalledWith('cloudSyncProperties', 'volumes', 'AWS');
        });

        afterEach(() => {
            expect(j.pidgeon.clear).toHaveBeenCalledWith('volumes');
        });
    });

    describe('$on VOLUMES_GET_ERROR_EVENT', () => {
        it('should do nothing if the payload belongs to different WE', () => {
            j.scope.$broadcast(volumesResource.VOLUMES_GET_ERROR_EVENT, {
                workingEnvironment: {
                    publicId: '2'
                }
            });

            expect(j.pidgeon.error).not.toHaveBeenCalled();
            expect(j.$log.error).not.toHaveBeenCalled();
        });

        it('should pidgeon the error', () => {
            j.scope.$broadcast(volumesResource.VOLUMES_GET_ERROR_EVENT, {
                workingEnvironment: {
                    publicId: '1'
                },
                error: j.errorResponse
            });

            expect(j.pidgeon.error).toHaveBeenCalledWith('An error occurred while retrieving volumes for working environment we1', j.errorResponse, 'volumes');
            expect(j.$log.error).not.toHaveBeenCalled();
        });

        it('should log instead not pidgeon if the error is due to synchronization problem in client code', () => {
            j.scope.$broadcast(volumesResource.VOLUMES_GET_ERROR_EVENT, {
                workingEnvironment: {
                    publicId: '1'
                },
                error: {message: 'Operation may only be performed on working environments whose status is ON. Current status is STOPPING'}
            });

            expect(j.pidgeon.error).not.toHaveBeenCalled();
            expect(j.$log.error).toHaveBeenCalledWith('An error occurred while retrieving volumes for working environment we1: Operation may only be performed on working environments whose status is ON. Current status is STOPPING. This error was not reported to user.');
        });
    });

    describe('box go back', () => {
        beforeEach(() => {
            j.scope.lastGadget = 'gadget1';
        });

        it('should go to last gadget', () => {
            j.scope.boxGoBack();
            expect(j.scope.goToGadget).toHaveBeenCalledWith('gadget1');
        });

        it('should just go to volumes if there is no last svm box state', () => {
            delete j.scope.lastGadget;
            j.scope.boxGoBack();
            expect(j.scope.showMonitoring).toHaveBeenCalled();
        });
    });

    describe('launchSystemManager', () => {
        it('should show dialog if has system manager url', () => {
            _.set(j.scope.selectedWorkingEnvironment, 'workingEnvironmentType', 'VSA');
            _.set(j.scope.selectedWorkingEnvironment, 'ontapClusterProperties.systemManagerUrl', '1.2.3.4');
            jest.spyOn(j.nocDialog, 'dialog').mockImplementation(() => {});

            j.scope.launchSystemManager();

            expect(j.nocDialog.dialog).toHaveBeenCalledWith('Launch System Manager', 'assets/templates/dialogs/launch-system-manager-content.html', {
                launchUrl: '1.2.3.4',
                showWarning: 'VSA'
            });
        });

        it('should show a message when there is no available url', () => {
            _.set(j.scope.selectedWorkingEnvironment, 'ontapClusterProperties', {});
            jest.spyOn(j.nocDialog, 'confirm').mockImplementation(() => {});

            j.scope.launchSystemManager();

            expect(j.nocDialog.confirm).toHaveBeenCalledWith('Unable to launch System Manager, ONTAP version must be above 8.3');
        });
    });

    describe('deleteVolume ', () => {
        beforeEach(() => {
            j.scope.selectedWorkingEnvironment = {
                publicId: 1,
                workingEnvironmentType: 'VSA',
                _pathPrefix: j.nocUtil.workingEnvironmentTypeToRequestPathPrefix('VSA', 'AWS')
            };
        });
        it('should prompt to the uer to confirm deletion', () => {
            magnifispec.mockConfirm(true);
            j.scope.deleteVolume({name: 'vol1'});
            expect(j.nocDialog.confirm).toHaveBeenCalledWith(j.Messages.deleteConfirm({
                type: 'volume',
                name: 'vol1'
            }));
        });

        it('should block the ui and make a post request to delete and on success unblock', () => {
            magnifispec.mockConfirm(true);
            j.$httpBackend.expectDELETE('/occm/api' + j.nocUtil.workingEnvironmentTypeToRequestPathPrefix(j.scope.selectedWorkingEnvironment.workingEnvironmentType, 'AWS') + 'volumes/' + j.scope.selectedWorkingEnvironment.publicId + '/svm1/vol1').respond(200);

            j.scope.deleteVolume({name: 'vol1', svmName: 'svm1'});
            magnifispec.processPromise();

            expect(j.nocUI.block.mock.calls.length).toEqual(1);
            j.$httpBackend.flush();
            expect(j.nocUI.unblock.mock.calls.length).toEqual(1);
        });

        it('should block the ui and make a post request to delete and on success unblock for ha', () => {
            j.scope.selectedWorkingEnvironment._pathPrefix = '/aws/ha/';
            magnifispec.mockConfirm(true);
            j.$httpBackend.expectDELETE('/occm/api' + '/aws/ha/' + 'volumes/' + j.scope.selectedWorkingEnvironment.publicId + '/svm1/vol1').respond(200);

            j.scope.deleteVolume({name: 'vol1', svmName: 'svm1'});
            magnifispec.processPromise();

            expect(j.nocUI.block.mock.calls.length).toEqual(1);
            j.$httpBackend.flush();
            expect(j.nocUI.unblock.mock.calls.length).toEqual(1);
        });

        it('should block the ui and make a post request to delete and on failure unblock and push an alert', () => {
            magnifispec.mockConfirm(true);
            j.$httpBackend.expectDELETE('/occm/api' + j.nocUtil.workingEnvironmentTypeToRequestPathPrefix(j.scope.selectedWorkingEnvironment.workingEnvironmentType, 'AWS') + 'volumes/' + j.scope.selectedWorkingEnvironment.publicId + '/svm1/vol1').respond(400, j.errorResponse);

            j.scope.deleteVolume({name: 'vol1', svmName: 'svm1'});
            magnifispec.processPromise();

            expect(j.nocUI.block.mock.calls.length).toEqual(1);
            j.$httpBackend.flush();
            expect(j.nocUI.unblock.mock.calls.length).toEqual(1);

            expect(j.pidgeon.error).toHaveBeenCalled();
        });
    });

    it('go to action required should go to action required and set last svm box state', () => {
        j.scope.gadget = 'mode';
        j.scope.goToActionRequired();

        expect(j.scope.lastGadget).toEqual('mode');
        expect(j.scope.goToGadget).toHaveBeenCalledWith('actionRequired');
    });

    describe('buildDataForInstanceInformationDialog', () => {
        it('should work', inject((MetadataResource) => {
            jest.spyOn(MetadataResource, 'getInstanceProfiles').mockReturnValue(j.$q.when([{
                instanceProfileId: 'id1',
                name: 'name1'
            }]));
            const instance = {name: 'lulu', instanceType: 'type1', state: 'state1', instanceProfileId: 'id1'};

            const returnedPromise = j.scope.buildDataForInstanceInformationDialog(instance);

            expect(returnedPromise).toBeResolvedWith([
                {key: 'Name:', text: 'lulu', trimmedText: true, title: 'lulu'},
                {key: 'Instance Type:', text: 'type1'},
                {key: 'State:', text: 'state1'},
                {key: 'Public Ip Address:', text: 'None'},
                {key: 'Public Dns Name:', text: 'None', trimmedText: true},
                {key: 'Private Dns Name:', trimmedText: true},
                {key: 'Image Id:'},
                {key: 'IAM Role:', text: 'name1', trimmedText: true}
            ]);
        }));
    });

    it('showInstanceInfoBox should open a dialog with the right parameters', (() => {
        jest.spyOn(j.scope, 'buildDataForInstanceInformationDialog').mockReturnValue(j.$q.when(['0', '1', '2', '3', '4', '5', '6']));
        const finish = magnifispec.fakeDialog();
        const instance = {id: 'id1', privateIpAddress: '1111'};
        const event = {};
        event.stopPropagation = jest.fn();
        j.scope.showInstanceInfoBox(event, instance);
        magnifispec.processPromise();
        finish();
        expect(j.nocDialog.dialog).toHaveBeenCalledWith('1111 (id1)', 'assets/templates/dialogs/general-info-dialog.html', {
            width: 904,
            lists: {leftList: ['0', '1', '2', '3'], rightList: ['4', '5', '6']}
        });
        expect(event.stopPropagation).toHaveBeenCalled();
    }));

    describe('Filter functions', () => {
        beforeEach(() => {
            jest.spyOn(j.nocUtil, ('genericItemFilter')).mockImplementation(() => {});
            j.scope.search = 'searchString';
        });

        it('instancesFilter', () => {
            j.scope.instancesFilter('Instance1');
            expect(j.nocUtil.genericItemFilter).toHaveBeenCalledWith('searchString', 'Instance1', ['name', 'privateIpAddress']);
        });
        it('aggregatesFilter', () => {
            j.scope.aggregatesFilter('Aggr1');
            expect(j.nocUtil.genericItemFilter).toHaveBeenCalledWith('searchString', 'Aggr1', ['name', '_displayCapacity']);
        });
        it('keyManagersFilter', () => {
            j.scope.keyManagersFilter('KeyManager1');
            expect(j.nocUtil.genericItemFilter).toHaveBeenCalledWith('searchString', 'KeyManager1', ['address', '_displayStatus']);
        });
    });

    describe('gotoUpgradeWorkingEnvironment', () => {
        it('should call parent function with appropriate params', () => {
            j.scope.$parent.gotoUpgradeWorkingEnvironment = jest.fn();
            j.scope.gotoUpgradeWorkingEnvironment();
            expect(j.scope.$parent.gotoUpgradeWorkingEnvironment.mock.calls[0][0]).toEqual(j.scope.selectedWorkingEnvironment);
        });
    });

    describe('goToRestoreVolume', () => {
        let getSnapshotsSpy;
        const volume = {name: 'vol1'};

        beforeEach(() => {
            jest.spyOn(j.nocUtil, 'workingEnvironmentTypeToRequestPathPrefix').mockImplementation(() => {});
            getSnapshotsSpy = jest.spyOn(volumesResource, 'getSnapshots');
        });

        it('should go to restoreVolume gadget if snapshots exist', () => {
            getSnapshotsSpy.mockReturnValue(j.$q.when([{name: 'snap1'}, {name: 'snap2'}]));
            j.scope.goToRestoreVolume(volume);
            magnifispec.processPromise();

            expect(j.scope.goToGadget).toHaveBeenCalledWith('restoreVolume', volume);
        });

        it('should show message if no snapshots exist', () => {
            jest.spyOn(j.nocDialog, 'alert').mockImplementation(() => {});
            getSnapshotsSpy.mockReturnValue(j.$q.when([]));
            j.scope.goToRestoreVolume(volume);
            magnifispec.processPromise();

            expect(j.nocDialog.alert).toHaveBeenCalledWith('Volume \'vol1\' has no Snapshots.');
            expect(j.scope.goToGadget).not.toHaveBeenCalledWith('restoreVolume', volume);
        });
    });

    describe('svm name edit', () => {
        it('should open a dialog', () => {
            j.scope.selectedWorkingEnvironment.svmName = 1;
            jest.spyOn(j.nocDialog, 'confirm').mockReturnValue(j.$q.when());
            j.scope.popupSvmNameEditDialog();
            expect(j.nocDialog.confirm).toHaveBeenCalledWith('', 'assets/templates/dialogs/change-svm-name.html', {
                svmName: 1
            });
        });

        describe('upon approving the dialog', () => {
            const requestUrl = '/occm/api/vsa/working-environments/1/svm';
            beforeEach(() => {
                j.scope.selectedWorkingEnvironment = {_pathPrefix: '/vsa/', publicId: 1, svmName: 'oldName'};
            });

            it('should not call API if the new name is identical to the old one', () => {
                magnifispec.mockConfirm({svmName: 'oldName'});

                j.scope.popupSvmNameEditDialog();
                j.$httpBackend.verifyNoOutstandingRequest();

                expect(j.scope.selectedWorkingEnvironment.svmName).toEqual('oldName');
                expect(j.pidgeon.clear).not.toHaveBeenCalledWith('editing SVM name');
            });

            it('should send a request upon approving the dialog and optimistically change the name if successful', () => {
                j.$httpBackend.expectPUT(requestUrl).respond(200);
                magnifispec.mockConfirm({svmName: 'newName'});

                j.scope.popupSvmNameEditDialog();
                j.$httpBackend.flush();

                j.expectUItoBlockAndUnblock();
                expect(j.scope.selectedWorkingEnvironment.svmName).toEqual('newName');
            });

            it('should send a request upon approving the dialog fire a pidgeon if unsuccessful', () => {
                j.$httpBackend.expectPUT(requestUrl).respond(400, j.errorResponse);
                magnifispec.mockConfirm({svmName: 'newName'});

                j.scope.popupSvmNameEditDialog();
                j.$httpBackend.flush();

                j.expectUItoBlockAndUnblock();
                expect(j.scope.selectedWorkingEnvironment.svmName).not.toEqual('newName');
                expect(j.pidgeon.error).toHaveBeenCalled();
            });
        });
    });

    describe('goto edit volume', () => {
        it('should go to the right place', () => {
            const volume = {name: 'vol1'};

            j.scope.goToEditVolume(volume);

            expect(j.scope.goToGadget).toHaveBeenCalledWith('editVolume', volume);
        });
    });

    describe('syncVolumeToS3', () => {
        let syncVolumeSpy;
        beforeEach(inject((nocCloudSyncService) => {
            j.$state.params = {type: 'VSA'};
            syncVolumeSpy = jest.spyOn(nocCloudSyncService, 'syncVolume').mockReturnValue(j.$q.when('response'));
        }));

        it('should call cloudSyncService with appropriate params', () => {
            j.scope.syncVolumeToS3({name: 'vol1'});

            expect(syncVolumeSpy).toHaveBeenCalledWith(jasmine.any(Object), workingEnvironment.publicId, 'vol1', 'VSA');
        });
    });

    describe('deleteSyncVolumeToS3', () => {
        let deleteSyncVolumeSpy;
        beforeEach(inject((nocCloudSyncService) => {
            j.$state.params = {type: 'VSA'};
            deleteSyncVolumeSpy = jest.spyOn(nocCloudSyncService, 'deleteSyncVolume').mockReturnValue(j.$q.when('response'));
        }));

        it('should call cloudSyncService with appropriate params', () => {
            j.scope.deleteSyncVolumeToS3({name: 'vol1'});

            expect(deleteSyncVolumeSpy).toHaveBeenCalledWith(jasmine.any(Object), workingEnvironment.publicId, 'vol1', 'VSA');
        });
    });

    describe('updateExtraProperties', () => {
        beforeEach(() => {
            getWorkingEnvironmentByPublicIdSpy.mockClear();
            getWorkingEnvironmentByPublicIdSpy.mockReturnValue(j.$q.when({cloudSyncProperties: {isSynced: true}}));
        });

        it('should call the resource with correct parameters and update the selectedWorkingEnvironment', () => {
            j.scope.selectedWorkingEnvironment.cloudSyncProperties = {isSynced: false};
            j.scope.selectedWorkingEnvironment._allowedOperations['sync-to-s3'] = true;

            expect(j.scope.updateExtraProperties(j.scope.selectedWorkingEnvironment)).toBeResolved();

            expect(j.scope.selectedWorkingEnvironment.cloudSyncProperties).toEqual({isSynced: true});
            expect(j.workingEnvironmentResource.getWorkingEnvironmentByPublicId).toHaveBeenCalledWith(jasmine.any(Object), '1', '/vsa/', 'cloudSyncProperties,k8sProperties');
        });

        it('should resolve immediately if sync-to-s3 is not allowed', () => {
            j.scope.selectedWorkingEnvironment.cloudSyncProperties = null;
            j.scope.selectedWorkingEnvironment._allowedOperations['sync-to-s3'] = false;

            expect(j.scope.updateExtraProperties(j.scope.selectedWorkingEnvironment)).toBeResolved();

            expect(j.scope.selectedWorkingEnvironment.cloudSyncProperties).toBeNull();
            expect(j.workingEnvironmentResource.getWorkingEnvironmentByPublicId).not.toHaveBeenCalled();
        });
    });

    describe('updateSelectedWorkingEnvironment', () => {
        beforeEach(() => {
            j.scope.workingEnvironments = [
                {publicId: 1, svmName: 'svm1', cloudProviderName: 'AWS', status: {status: 'on'}}
            ];

            j.$state.params = {type: 'VSA', provider: 'AWS'};
            getWorkingEnvironmentByPublicIdSpy.mockClear();
            j.scope.selectedWorkingEnvironment = undefined;
        });

        it('should update the working environment from the list of working environments if found there and there are no required fields', () => {
            j.scope.updateSelectedWorkingEnvironment(1)
                .then(() => {
                    expect(j.scope.selectedWorkingEnvironment).toEqual(j.scope.workingEnvironments[0]);
                });
            magnifispec.processPromise();

            expect(j.workingEnvironmentResource.getWorkingEnvironmentByPublicId).not.toHaveBeenCalled();
        });

        it('should update the working environment from the list if found and the required fields are also found ', () => {
            j.scope.updateSelectedWorkingEnvironment(1, ['svmName', 'cloudProviderName'])
                .then(() => {
                    expect(j.scope.selectedWorkingEnvironment).toEqual(j.scope.workingEnvironments[0]);
                });
            magnifispec.processPromise();

            expect(j.workingEnvironmentResource.getWorkingEnvironmentByPublicId).not.toHaveBeenCalled();
        });

        it('should get the working environment by id when not found in the list', () => {
            getWorkingEnvironmentByPublicIdSpy.mockReturnValue(j.$q.when({publicId: 2}));
            j.scope.updateSelectedWorkingEnvironment(2)
                .then(() => {
                    expect(j.scope.selectedWorkingEnvironment).toEqual({publicId: 2});
                });
            magnifispec.processPromise();

            expect(j.workingEnvironmentResource.getWorkingEnvironmentByPublicId).toHaveBeenCalledWith(jasmine.any(Object), 2, '/vsa/', 'status,activeActions,awsProperties,encryptionProperties,ontapClusterProperties.fields(upgradeVersions),actionsRequired,capacityFeatures');
        });

        it('should get the working environment by id when working environments list is unavailable', () => {
            j.scope.workingEnvironments = null;
            getWorkingEnvironmentByPublicIdSpy.mockReturnValue(j.$q.when({publicId: 2}));

            j.scope.updateSelectedWorkingEnvironment(2)
                .then(() => {
                    expect(j.scope.selectedWorkingEnvironment).toEqual({publicId: 2});
                });
            magnifispec.processPromise();

            expect(j.workingEnvironmentResource.getWorkingEnvironmentByPublicId).toHaveBeenCalledWith(jasmine.any(Object), 2, '/vsa/', 'status,activeActions,awsProperties,encryptionProperties,ontapClusterProperties.fields(upgradeVersions),actionsRequired,capacityFeatures');
        });

        it('should get the working environment by id when required field is null, and preserve current fields', () => {
            getWorkingEnvironmentByPublicIdSpy.mockReturnValue(j.$q.when({
                publicId: 1,
                svmName: 'svm33',
                nonExistingKey: 'key1',
                status: null
            }));
            j.scope.workingEnvironments[0].svmName = null;
            j.scope.updateSelectedWorkingEnvironment(1, ['svmName'])
                .then(() => {
                    expect(j.scope.selectedWorkingEnvironment).toEqual({
                        publicId: 1,
                        svmName: 'svm33',
                        cloudProviderName: 'AWS',
                        status: {status: 'on'},
                        nonExistingKey: 'key1'
                    });
                });
            magnifispec.processPromise();

            expect(j.workingEnvironmentResource.getWorkingEnvironmentByPublicId).toHaveBeenCalledWith(jasmine.any(Object), 1, '/vsa/', 'status,activeActions,awsProperties,encryptionProperties,ontapClusterProperties.fields(upgradeVersions),actionsRequired,capacityFeatures,svmName');
        });

        it('should keep existing working environment', () => {
            j.scope.selectedWorkingEnvironment = {
                publicId: 1,
                theCatcher: 'inTheRye',
                status: {status: 'on'}
            };
            getWorkingEnvironmentByPublicIdSpy.mockReturnValue(j.$q.when({
                publicId: 1,
                svmName: 'svm33',
                status: null
            }));

            j.scope.updateSelectedWorkingEnvironment(1, ['svmName'])
                .then(() => {
                    expect(j.scope.selectedWorkingEnvironment).toEqual({
                        publicId: 1,
                        svmName: 'svm33',
                        status: {status: 'on'},
                        theCatcher: 'inTheRye'
                    });
                });
            magnifispec.processPromise();
        });
    });
});
