describe("Tenants New Edit", function () {
    "use strict";

    var j;    beforeEach(() => {        j = magnifispec.setup();    });

    var mockedTenants = [
        {   publicId: "1",
            name: "Paris",
            descrition: "This is paris",
            costCenter: "1234"},
        {   publicId: "2",
            name: "TLV",
            descrition: "This is Tel-Aviv",
            costCenter: "2345"},
        {   publicId: "3",
            name: "Lapid",
            descrition: "This is my home",
            costCenter: "3456"}
    ];

    beforeEach(function () {
        j.$state.params = {tenantId: 1};
        magnifispec.mockTabsController("TenantsNewEditController", {tenantsMode: "new"}, jasmine.createSpyObj("TenantsController", ['setTenants', 'deleteTenant']));

        j.extendedTenantsUrl = j.nocBaseUrl + "/tenants?fields=workingEnvironmentsInformation";
        j.tenantsUrl = j.nocBaseUrl + "/tenants";
        j.parentScope.backToGallery = jest.fn();
        j.parentScope.updateOriginalTenant = jest.fn();
    });

    describe("resolveTenantByPublicId", function () {
        var tenants, successFn, errorFn;

        beforeEach(function () {
            tenants = [{publicId: 1, name: 'a'}, {publicId: 2, name: 'b'}];

            successFn = jest.fn();
            errorFn = jest.fn();
        });

        describe("case where tenants are available", function () {
            it("work when tenant exists", function () {
                j.scope.resolveTenantByPublicId(2, tenants)
                    .then(successFn, errorFn);
                j.$httpBackend.verifyNoOutstandingRequest();
                j.$rootScope.$digest();
                expect(successFn).toHaveBeenCalledWith({publicId: 2, name: 'b'});
                expect(errorFn).not.toHaveBeenCalled();
            });
            it("work when tenant does not exist", function () {
                j.scope.resolveTenantByPublicId(3, tenants)
                    .then(successFn, errorFn);
                j.$rootScope.$digest();
                expect(successFn).not.toHaveBeenCalled();
                expect(errorFn).toHaveBeenCalled();
            });
        });

        describe("case when tenants are unavailable", function () {
            it("call API and work upon success", function () {
                j.$httpBackend.expectGET(j.nocBaseUrl + "/tenants/" + 2 + "?fields=workingEnvironmentsInformation")
                    .respond(200, {publicId: 2, name: 'b'});
                j.scope.resolveTenantByPublicId(2, null)
                    .then(successFn, errorFn);
                j.$httpBackend.flush();
                expect(successFn).toHaveBeenCalledWith({publicId: 2, name: 'b'});
                expect(errorFn).not.toHaveBeenCalled();
            });
            it("work when tenant does not exist", function () {
                j.$httpBackend.expectGET(j.nocBaseUrl + "/tenants/" + 2 + "?fields=workingEnvironmentsInformation")
                    .respond(400, {});
                j.scope.resolveTenantByPublicId(2, null)
                    .then(successFn, errorFn);
                j.$httpBackend.flush();
                expect(successFn).not.toHaveBeenCalled();
                expect(errorFn).toHaveBeenCalled();
            });
        });
    });

    describe("updateOriginalTenant", function(){
        it("should set the originalTenant to be the selectedTenant on updateOriginalTenant", function () {
            j.scope.selectedTenant = "not nothing";
            j.scope.originalSelectedTenant = "another not nothing";
            j.scope.updateOriginalTenant();

            expect(j.scope.selectedTenant === j.scope.originalSelectedTenant).toEqual(true);
        });

        it("should swap out the tenants in the tenant array if present", inject(function(nocTenantsService){
            jest.spyOn(nocTenantsService, 'getTenants').mockReturnValue(["tenant1", "tenant2"]);
            j.scope.tenants = ["tenant1", "tenant2"];
            j.scope.selectedTenant = "tenant3";
            j.scope.originalSelectedTenant = "tenant2";
            j.scope.updateOriginalTenant();
            expect(j.scope.tenants).toEqual(["tenant1", "tenant3"]);
        }));
    });

    describe("Is valid", function () {
        it("should return true if the tenantsForm is valid", function () {
            j.scope.tenantsForm = {$valid:true};
            expect(j.scope.isValid()).toEqual(true);
        });

        it("should return false if the tenantsForm is invalid", function () {
            j.scope.tenantsForm = {$valid:false};
            expect(j.scope.isValid()).toEqual(false);
        });
    });

    describe("Is dirty", function () {
        it("should return true if the original is not the same as the modified", function () {
            j.scope.originalSelectedTenant = {};
            j.scope.selectedTenant = {name:"different"};
            expect(j.scope.isDirty()).toEqual(true);
        });

        it("should return false if the original has not been modified", function () {
            j.scope.originalSelectedTenant = {name:"the same"};
            j.scope.selectedTenant = {name:"the same"};
            expect(j.scope.isDirty()).toEqual(false);

        });
    });

    describe("Deleting the selected tenant", function () {
        var nocTenantsService;

        beforeEach(inject(function (_nocTenantsService_) {
            nocTenantsService = _nocTenantsService_;
            jest.spyOn(nocTenantsService, 'reloadTenants').mockImplementation(() => {});
            j.scope.tenants = [{name: "Paris", publicId: 1}, {name: "Tel Aviv", publicId: 2}];
            j.scope.originalSelectedTenant = j.scope.tenants[0];
            j.scope.selectedTenant = {name: "Paris", publicId: 1};
            jest.spyOn(j.scope, "updateOriginalTenant").mockImplementation(() => {});
        }));

        it("Should show pidgeon if trying to delete the last tenant, and not delete it", function () {
            jest.spyOn(j.nocDialog, "confirm").mockImplementation(() => {});
            j.scope.tenants = [{name: "Paris", publicId: 1}];

            j.scope.deleteSelectedTenant();
            expect(j.nocDialog.confirm).not.toHaveBeenCalled();
            expect(j.pidgeon.error).toHaveBeenCalledWith("The last tenant cannot be deleted.");
        });

        it("should do nothing if the tenant doesn't confirm (and check the message)", function () {
            magnifispec.fakeConfirmNew(false);

            j.scope.deleteSelectedTenant();
            expect(j.nocDialog.confirm).toHaveBeenCalledWith(j.Messages.deleteConfirm({type: "tenant", name: "Paris"}));
            expect(j.scope.updateOriginalTenant).not.toHaveBeenCalled();
            expect(j.scope.backToGallery).not.toHaveBeenCalled();
        });

        it("should call delete for the tenant, and on success remove the selected tenant from the list and call close new edit", function () {
            magnifispec.fakeConfirmNew(true);
            j.$httpBackend.expectDELETE(j.tenantsUrl + "/1").respond(200, {});

            j.scope.deleteSelectedTenant();
            j.$httpBackend.flush();
            expect(j.nocUI.block.mock.calls.length).toEqual(1);
            expect(j.nocUI.unblock.mock.calls.length).toEqual(1);

            expect(j.scope.updateOriginalTenant).toHaveBeenCalled();
            expect(j.scope.backToGallery).toHaveBeenCalled();
            expect(j.pidgeon.clear).toHaveBeenCalledWith("deleting tenant");
        });

        it("should call delete for the tenant and on failure push an alert and not call close", function () {
            magnifispec.fakeConfirmNew(true);
            j.$httpBackend.expectDELETE(j.tenantsUrl + "/1").respond(400,j.errorResponse);

            j.scope.deleteSelectedTenant();
            j.$httpBackend.flush();
            expect(j.nocUI.block.mock.calls.length).toEqual(1);
            expect(j.nocUI.unblock.mock.calls.length).toEqual(1);

            expect(j.scope.updateOriginalTenant).not.toHaveBeenCalled();
            expect(j.scope.backToGallery).not.toHaveBeenCalled();
            expect(j.pidgeon.error).toHaveBeenCalled();
        });
    });

    describe("Initialize the controller", function(){
        describe("When in new mode", function(){
            it("should create an empty tenant", function () {
                j.scope.initializeController("new");
                expect(j.scope.selectedTenant).toEqual({});
                expect(j.scope.selectedTenant === j.scope.originalSelectedTenant).toEqual(false);
                expect(j.scope.selectedTenant).toEqual(j.scope.originalSelectedTenant);
            });
        });

        describe("When in edit mode", function(){
            it("should call resolveTenantByPublicId by passing in the value from $state.params", function(){
                var thenFunction = jest.fn();
                j.scope.tenants = ["tenant1"];
                jest.spyOn(j.scope, "resolveTenantByPublicId").mockReturnValue({then: thenFunction});
                j.scope.initializeController("edit");

                expect(j.scope.resolveTenantByPublicId).toHaveBeenCalledWith(1, ["tenant1"]);
                expect(thenFunction).toHaveBeenCalledWith(j.scope.initializeEditMode, j.scope.backToGallery);
            });
        });
    });

    describe("Save selected tenant", function () {
        it("should handle form validity if it is not valid", function () {
            jest.spyOn(j.scope,"isValid").mockReturnValue(false);
            j.scope.saveSelectedTenant();
            expect(j.nocUI.handleFormValidity).toHaveBeenCalled();
        });

        it("should call back to gallery if valid but not dirty", function () {
            jest.spyOn(j.scope,"isValid").mockReturnValue(true);
            jest.spyOn(j.scope,"isDirty").mockReturnValue(false);
            j.scope.saveSelectedTenant();
            expect(j.scope.backToGallery).toHaveBeenCalled();
        });


        describe("Success case", function () {
            var tenant;
            var nocTenantsService;

            beforeEach(inject(function (_nocTenantsService_) {
                nocTenantsService = _nocTenantsService_;
                jest.spyOn(nocTenantsService, 'reloadTenants').mockImplementation(() => {});
                jest.spyOn(j.scope,"isValid").mockReturnValue(true);
                jest.spyOn(j.scope,"isDirty").mockReturnValue(true);
                j.scope.selectedTenant = mockedTenants[2];
                tenant = _.cloneDeep(j.scope.selectedTenant);
                delete tenant.publicId;
                jest.spyOn(j.scope, "updateOriginalTenant").mockImplementation(() => {});
            }));

            it("should make a post request with the data if in new mode", function () {
                j.scope.tenantsMode = "new";
                j.scope.tenants = [];
                j.$httpBackend.expectPOST(j.tenantsUrl,tenant).respond(200,{});

                j.scope.saveSelectedTenant();

                j.$httpBackend.flush();
                expect(j.scope.updateOriginalTenant).toHaveBeenCalled();
                expect(j.scope.backToGallery).toHaveBeenCalled();
                expect(j.nocUI.block.mock.calls.length).toEqual(1);
                expect(j.nocUI.unblock.mock.calls.length).toEqual(1);
                expect(nocTenantsService.reloadTenants).toHaveBeenCalled();
            });

            it("should make a put request with the data if in edit mode", function () {
                j.scope.tenantsMode = "edit";
                j.$httpBackend.expectPUT(j.tenantsUrl+"/3",tenant).respond(200,{});

                j.scope.saveSelectedTenant();

                j.$httpBackend.flush();
                expect(j.scope.updateOriginalTenant).toHaveBeenCalled();
                expect(j.scope.backToGallery).toHaveBeenCalled();
                expect(j.nocUI.block.mock.calls.length).toEqual(1);
                expect(j.nocUI.unblock.mock.calls.length).toEqual(1);
                expect(j.pidgeon.clear).toHaveBeenCalledWith('saving changes');
                expect(nocTenantsService.reloadTenants).toHaveBeenCalled();
            });

            it("should push an alert in case the request fails", function () {

                j.scope.tenantsMode = "edit";
                j.$httpBackend.expectPUT(j.tenantsUrl+"/3",tenant).respond(400,j.errorResponse);

                j.scope.saveSelectedTenant();

                j.$httpBackend.flush();
                expect(j.scope.updateOriginalTenant).not.toHaveBeenCalled();
                expect(j.scope.backToGallery).not.toHaveBeenCalled();
                expect(j.pidgeon.error).toHaveBeenCalled();
                expect(j.nocUI.block.mock.calls.length).toEqual(1);
                expect(j.nocUI.unblock.mock.calls.length).toEqual(1);
            });

        });
    });

});
