describe('UsersNewEditController', () => {
    'use strict';

    let j, _nocCurrentUserService;
    beforeEach(() => {
        j = magnifispec.setup();
    });

    const mockedUsers = [
        {
            publicId: '1',
            firstName: 'Roy',
            lastName: 'Alon',
            email: 'kuki@puki.com',
            roleId: 'Role-1',
            role: {name: 'Oncloud Admin', publicId: 'Role-1'},
            tenantId: 'Tenant-1234',
            tenant: {publicId: 'Tenant-1234'},
            workingEnvironments: ['we-1', 'we-2', 'we-3'],
            isLdap: false
        },
        {
            publicId: '2',
            firstName: 'Daniel',
            lastName: 'QA',
            email: 'daniel@puki.com',
            roleId: 'Role-2',
            tenantId: 'Tenant-8999',
            tenant: {publicId: 'Tenant-8999'},
            workingEnvironments: [],
            isLdap: true,
            ldapUserName: 'daniel'
        },
        {
            publicId: '3',
            firstName: 'Anatoly',
            lastName: 'Shaoulov',
            email: 'anatoly@puki.com',
            roleId: 'Role-3',
            tenantId: 'Tenant-2345',
            tenant: {publicId: 'Tenant-2345'},
            workingEnvironments: ['we-1', 'we-2'],
            isLdap: false
        }
    ];

    beforeEach(() => {
        magnifispec.mockTabsController('UsersNewEditController', undefined, {
            users: mockedUsers,
            currentUser: _.cloneDeep(mockedUsers[0]),
            allowedRolesToCurrentRole: {
                'Role-1': {
                    'Role-1': true,
                    'Role-2': true,
                    'Role-3': true
                },
                'Role-2': {
                    'Role-2': true,
                    'Role-3': true
                },
                'Role-3': {
                    'Role-3': true
                }
            },
            roles: [
                {name: 'OnCloud Admin', publicId: 'Role-1'},
                {name: 'Tenant Admin', publicId: 'Role-2'},
                {name: 'WE Admin', publicId: 'Role-3'}
            ]
        });
        j.usersUrl = j.nocBaseUrl + '/users';

        j.$rootScope.checkAndSetState = jest.fn().mockReturnValue(j.$q.when());
        j.$rootScope.supportServices = {};

        j.$httpBackend.expectGET('/occm/api/active-directory').respond([{
            id: 1,
            publicId: 'AD1',
            name: 'ad',
            host: 'ad.com',
            port: 3
        }]);
    });


    describe('Initialization without users previously being set', () => {
        let init;
        beforeEach(() => {
            init = jest.fn().mockReturnValue(j.$q.when());
            magnifispec.mockTabsController('UsersNewEditController', undefined, {
                initializeUsers: init, selectedUser: {}
            });
        });

        it('should first call initialize users, then call initialize editor', () => {
            jest.spyOn(j.scope, 'initializeEditor').mockImplementation(() => {
            });
            expect(init).toHaveBeenCalled();
            j.$timeout.flush();
            expect(j.scope.initializeEditor).toHaveBeenCalled();
        });
    });

    describe('Initialization with and without active directory', () => {
        beforeEach(() => {
            j.$httpBackend.flush(); //Flash first AD request
        });

        it('should build authentication types list correctly when AD is configured', () => {
            j.$httpBackend.expectGET('/occm/api/active-directory').respond([
                {id: 1, publicId: 'AD1', name: 'ad', host: 'ad.com', port: 3}
            ]);
            j.scope.initializeEditor();
            j.$httpBackend.flush();
            expect(j.scope.authenticationTypes).toEqual(['Cloud Manager', 'Active Directory']);
        });

        it('should build authentication types list correctly when AD is not configured', () => {
            j.$httpBackend.expectGET('/occm/api/active-directory').respond([]);
            j.scope.initializeEditor();
            j.$httpBackend.flush();
            expect(j.scope.authenticationTypes).toEqual(['Cloud Manager']);
        });
    });

    it('should remove the selectedUser and originalSelected user on call to updateOriginalUser', () => {
        j.scope.selectedUser = 'not nothing';
        j.scope.originalSelectedUser = 'different not nothing';
        j.scope.updateOriginalUser();

        expect(j.scope.selectedUser === j.scope.originalSelectedUser).toEqual(true);
    });

    describe('navigating functions', () => {
        it('should call checkAndSetState when backToTable is called', () => {
            j.scope.backToTable();
            expect(j.$rootScope.checkAndSetState).toHaveBeenCalledWith('^.index');
        });

        it('should open relevant dialog when calling resetPasswordOfSelectedUser', () => {
            magnifispec.fakeDialog();
            j.scope.selectedUser = {publicId: '1'};
            j.scope.resetPasswordOfSelectedUser();
            expect(j.nocDialog.dialog).toHaveBeenCalledWith('Reset Password', 'assets/templates/tabs/users/reset-password.html', {selectedUser: {publicId: '1'}});
        });
    });

    describe('Delete tenant from given user', () => {
        beforeEach(inject(function (_nocCurrentUserService_) {
            _nocCurrentUserService = _nocCurrentUserService_;
            j.scope.selectedUser = {role: {publicId: 'Role-1'}};
        }));

        it('should delete the tenant from the given user if the current user is an on cloud admin', () => {
            _nocCurrentUserService.initializeCurrentUser({role: {publicId: 'Role-1'}});
            const user = {tenant: 'some value'};
            j.scope.deleteTenantFromGivenUser(user);

            expect(user.tenant).toBeUndefined();
        });

        it('should delete the tenant from the given user if the current user is not an on cloud admin', () => {
            _nocCurrentUserService.initializeCurrentUser({role: {publicId: 'Role-2'}});
            const user = {tenant: 'some value'};
            j.scope.deleteTenantFromGivenUser(user);

            expect(user.tenant).toBeUndefined();
        });
    });

    describe('Empty granted map from given user', () => {
        it('should empty the granted map of the given user', () => {
            const user = {grantedMap: {someWE: true}};
            j.scope.emptyGrantedMapFromGivenUser(user);

            expect(user.grantedMap).toEqual({});
        });
    });

    describe('Is valid', () => {
        it('should return true if the usersForm is valid', () => {
            j.scope.usersForm = {$valid: true};
            expect(j.scope.isValid()).toEqual(true);
        });

        it('should return false if the usersForm is invalid', () => {
            j.scope.usersForm = {$valid: false};
            expect(j.scope.isValid()).toEqual(false);
        });
    });

    describe('Is dirty', () => {
        it('should return true if the original is not the same as the modified', () => {
            j.scope.usersForm = {$dirty: true};
            j.scope.originalSelectedUser = {};
            j.scope.selectedUser = {name: 'different'};
            expect(j.scope.isDirty()).toEqual(true);
        });

        it('should return false if the original has not been modified', () => {
            j.scope.usersForm = {$dirty: true};
            j.scope.originalSelectedUser = {name: 'the same'};
            j.scope.selectedUser = {name: 'the same'};
            expect(j.scope.isDirty()).toEqual(false);
        });
    });

    describe('Save selected user', () => {
        it('should handle form validity if it is not valid', () => {
            jest.spyOn(j.scope, 'isValid').mockReturnValue(false);
            j.scope.saveSelectedUser();
            expect(j.nocUI.handleFormValidity).toHaveBeenCalled();
        });

        it('should call back to gallery if valid but not dirty', () => {
            jest.spyOn(j.scope, 'isValid').mockReturnValue(true);
            jest.spyOn(j.scope, 'isDirty').mockReturnValue(false);
            jest.spyOn(j.scope, 'backToTable').mockReturnValue();
            j.scope.saveSelectedUser();
        });

        describe('prepare Cloned User', () => {
            let clonedUser;
            beforeEach(() => {
                clonedUser = {
                    publicId: '3',
                    firstName: 'Anatoly',
                    lastName: 'Shaoulov',
                    email: 'anatoly@puki.com',
                    roleId: 'Role-2',
                    s3Bucket: 'bucket',
                    tenantId: 'Tenant-2345',
                    role: {name: 'WE Admin', publicId: 'Role-3'},
                    tenant: {name: 'Tenant1', publicId: 'Tenant-1234'},
                    workingEnvironments: [{name: 'WE1', publicId: 'we-1'}, {name: 'WE2', publicId: 'we-2'}],
                    retypePassword: 'blabla',
                    grantedMap: {},
                    isLdap: false,
                    ldapUserName: 'moshe'
                };
            });
            it('should properly prepare the cloned user for transport', () => {
                clonedUser.authenticationMode = 'cm';
                j.scope.prepareClonedUser(clonedUser);

                //Set the role id and delete the role
                expect(clonedUser.roleId).toEqual('Role-3');
                expect(clonedUser.role).toBeUndefined();
                //Set the tenant id and delete the tenant
                expect(clonedUser.tenantId).toEqual('Tenant-1234');
                expect(clonedUser.tenant).toBeUndefined();

                expect(clonedUser.workingEnvironments).toBeUndefined();
                expect(clonedUser.retypePassword).toBeUndefined();
                expect(clonedUser.publicId).toBeUndefined();
                expect(clonedUser.grantedMap).toBeUndefined();
                expect(clonedUser.name).toBeUndefined();
                expect(clonedUser.ldapUserName).toBeUndefined();
            });

            it('should set the tenant id to undefined if there is no selected Tenant', () => {
                clonedUser.tenantId = undefined;
                clonedUser.tenant = undefined;

                j.scope.prepareClonedUser(clonedUser);

                expect(clonedUser.tenantId).toBeUndefined();
            });

            it('should set a fake password and the AD user name is account type is AD', () => {
                clonedUser.authenticationMode = 'Active Directory';

                j.scope.prepareClonedUser(clonedUser);

                expect(clonedUser.password).toEqual('LDAP123456');
                expect(clonedUser.ldapUserName).toBeDefined();
            });
        });

        describe('Generating the grant permissions object', () => {
            it('should have two empty arrays if nothing has changed', () => {
                const clonedUser = {grantedMap: {'we-1': true, 'we-2': false}};
                const originalUser = {workingEnvironments: [{publicId: 'we-1'}]};
                const grantPermission = j.scope.buildGrantPermission(clonedUser, originalUser);
                expect(_.isEmpty(grantPermission.addWorkingEnvironments)).toEqual(true);
                expect(_.isEmpty(grantPermission.removeWorkingEnvironments)).toEqual(true);
            });

            it('should have a revoked if only removed', () => {
                const clonedUser = {grantedMap: {'we-1': true, 'we-2': false}};
                const originalUser = {workingEnvironments: [{publicId: 'we-1'}, {publicId: 'we-2'}]};
                const grantPermission = j.scope.buildGrantPermission(clonedUser, originalUser);
                expect(_.isEmpty(grantPermission.addWorkingEnvironments)).toEqual(true);
                expect(grantPermission.removeWorkingEnvironments).toEqual(['we-2']);
            });

            it('should have a granted if only added', () => {
                const clonedUser = {grantedMap: {'we-1': true, 'we-2': true}};
                const originalUser = {workingEnvironments: []};
                const grantPermission = j.scope.buildGrantPermission(clonedUser, originalUser);
                expect(grantPermission.addWorkingEnvironments).toEqual(['we-1', 'we-2']);
                expect(_.isEmpty(grantPermission.removeWorkingEnvironments)).toEqual(true);
            });

            it('should have both if removed and added', () => {
                const clonedUser = {grantedMap: {'we-1': true, 'we-2': false}};
                const originalUser = {workingEnvironments: [{publicId: 'we-2'}]};
                const grantPermission = j.scope.buildGrantPermission(clonedUser, originalUser);
                expect(grantPermission.addWorkingEnvironments).toEqual(['we-1']);
                expect(grantPermission.removeWorkingEnvironments).toEqual(['we-2']);
            });
        });

        describe('Success case', () => {
            let user;

            beforeEach(inject(function (nocCurrentUserService) {
                _nocCurrentUserService = nocCurrentUserService;

                jest.spyOn(j.scope, 'isValid').mockReturnValue(true);
                jest.spyOn(j.scope, 'isDirty').mockReturnValue(true);
                user = j.scope.selectedUser = {
                    publicId: '3',
                    firstName: 'Anatoly',
                    lastName: 'Shaoulov',
                    role: {
                        publicId: 'Role-3'
                    }
                };

                jest.spyOn(j.scope, 'prepareClonedUser').mockImplementation(() => {});

                jest.spyOn(nocCurrentUserService, 'getCurrentUserRolePublicId').mockReturnValue({publicId: 'Role-1'});
            }));

            it('should make a post request with the data if in new mode', () => {
                jest.spyOn(j.scope, 'buildGrantPermission').mockReturnValue({});
                jest.spyOn(j.scope, 'updateOriginalUser').mockReturnValue();
                jest.spyOn(j.scope, 'backToTable').mockReturnValue();
                j.scope.usersMode = 'new';
                j.$httpBackend.expectPOST(j.usersUrl, user).respond(200, {});

                j.scope.saveSelectedUser();

                j.$httpBackend.flush();
            });

            it('should make a put request with the data if in edit mode', () => {
                jest.spyOn(j.scope, 'buildGrantPermission').mockReturnValue({});
                jest.spyOn(j.scope, 'updateOriginalUser').mockReturnValue();
                jest.spyOn(j.scope, 'backToTable').mockReturnValue();

                j.scope.usersMode = 'edit';
                j.$httpBackend.expectPUT(j.usersUrl + '/3', user).respond(200, {});

                j.scope.saveSelectedUser();

                j.$httpBackend.flush();
            });

            it('should push an alert in case the request fails', () => {
                jest.spyOn(j.scope, 'buildGrantPermission').mockReturnValue({});

                j.scope.usersMode = 'edit';
                j.$httpBackend.expectPUT(j.usersUrl + '/3', user).respond(400, j.errorResponse);

                j.scope.saveSelectedUser();

                j.$httpBackend.flush();
                expect(j.pidgeon.error).toHaveBeenCalled();
            });

            it('should only make a grant request if role is 3', () => {
                jest.spyOn(j.scope, 'updateOriginalUser').mockReturnValue();
                jest.spyOn(j.scope, 'backToTable').mockReturnValue();
                _nocCurrentUserService.initializeCurrentUser({role: {publicId: 'Role-3'}});
                const grantPermission = {addWorkingEnvironments: [1], removeWorkingEnvironments: []};
                jest.spyOn(j.scope, 'buildGrantPermission').mockReturnValue(grantPermission);
                j.scope.usersMode = 'edit';
                j.$httpBackend.expectPOST(j.usersUrl + '/3/working-environments-grant-revoke', grantPermission).respond(200);

                j.scope.saveSelectedUser();

                //We know it works because there was no UNEXPECTED POST message
            });

            it('should make a put request with the data if in edit mode even if the role is 3, if it is the current user', () => {
                jest.spyOn(j.scope, 'buildGrantPermission').mockReturnValue({});
                jest.spyOn(j.scope, 'updateOriginalUser').mockReturnValue();
                jest.spyOn(j.scope, 'backToTable').mockReturnValue();
                _nocCurrentUserService.initializeCurrentUser({role: {publicId: 'Role-3'}});
                j.scope.selectedUser.currentUser = true;
                j.scope.usersMode = 'edit';
                j.$httpBackend.expectPUT(j.usersUrl + '/3', user).respond(200, {});

                j.scope.saveSelectedUser();

                j.$httpBackend.flush();
            });

            describe('Posting grant permissions after saving user', () => {
                beforeEach(() => {
                    j.scope.usersMode = 'new';
                    j.$httpBackend.expectPOST(j.usersUrl, user).respond(200, user);
                });
                it('should post to grant permission if only granted', () => {
                    const grantPermission = {addWorkingEnvironments: [1], removeWorkingEnvironments: []};
                    jest.spyOn(j.scope, 'buildGrantPermission').mockReturnValue(grantPermission);
                    jest.spyOn(j.scope, 'updateOriginalUser').mockReturnValue();
                    jest.spyOn(j.scope, 'backToTable').mockReturnValue();
                    j.$httpBackend.expectPOST(j.usersUrl + '/3/working-environments-grant-revoke', grantPermission).respond(200);
                    j.scope.saveSelectedUser();
                    j.$httpBackend.flush();
                });

                it('should post to grant if only revoked', () => {
                    const grantPermission = {addWorkingEnvironments: [], removeWorkingEnvironments: [1]};
                    jest.spyOn(j.scope, 'buildGrantPermission').mockReturnValue(grantPermission);
                    jest.spyOn(j.scope, 'updateOriginalUser').mockReturnValue();
                    jest.spyOn(j.scope, 'backToTable').mockReturnValue();
                    j.$httpBackend.expectPOST(j.usersUrl + '/3/working-environments-grant-revoke', grantPermission).respond(200);

                    j.scope.saveSelectedUser();
                    j.$httpBackend.flush();
                });

                it('should post to grant if both', () => {
                    const grantPermission = {addWorkingEnvironments: [2], removeWorkingEnvironments: [1]};
                    jest.spyOn(j.scope, 'buildGrantPermission').mockReturnValue(grantPermission);
                    jest.spyOn(j.scope, 'updateOriginalUser').mockReturnValue();
                    jest.spyOn(j.scope, 'backToTable').mockReturnValue();
                    j.$httpBackend.expectPOST(j.usersUrl + '/3/working-environments-grant-revoke', grantPermission).respond(200);

                    j.scope.saveSelectedUser();
                    j.$httpBackend.flush();
                });

                it('should push an error on fail', () => {
                    const grantPermission = {addWorkingEnvironments: [2], removeWorkingEnvironments: [1]};
                    jest.spyOn(j.scope, 'buildGrantPermission').mockReturnValue(grantPermission);
                    j.$httpBackend.expectPOST(j.usersUrl + '/3/working-environments-grant-revoke', grantPermission).respond(400, j.errorResponse);

                    j.scope.saveSelectedUser();
                    j.$httpBackend.flush();

                    expect(j.pidgeon.error).toHaveBeenCalled();
                });
            });

            describe('Updating CurrentUser names', () => {
                beforeEach(() => {
                    j.scope.selectedUser.currentUser = true;
                });

                it('Should update the currentUser name if editing the current user', () => {
                    j.scope.usersMode = 'edit';
                    j.scope.saveSelectedUser();
                    expect(j.scope.currentUser.firstName).toEqual('Anatoly');
                    expect(j.scope.currentUser.lastName).toEqual('Shaoulov');
                });

                it('Should not update the currentUser name if not editing the current user', () => {
                    j.scope.selectedUser.currentUser = false;
                    j.scope.saveSelectedUser();
                    expect(j.scope.currentUser.firstName).not.toEqual('Anatoly');
                    expect(j.scope.currentUser.lastName).not.toEqual('Shaoulov');
                });
            });
        });
    });

    describe('filterCreateRolesForCurrentUser', () => {
        beforeEach(inject(function (nocCurrentUserService) {
            _nocCurrentUserService = nocCurrentUserService;

            jest.spyOn(_nocCurrentUserService, 'isCurrentUserExist').mockReturnValue(true);

        }));

        it('should always return true while in edit mode', () => {
            j.scope.usersMode = 'edit';
            expect(j.scope.filterCreateRolesForCurrentUser()).toEqual(true);
        });

        it('should return false if the current user cant create the specified role', () => {
            j.scope.userMode = 'new';
            _nocCurrentUserService.initializeCurrentUser({roleId: 'Role-2'});
            jest.spyOn(_nocCurrentUserService, 'getCurrentUserRolePublicId').mockReturnValue('Role-2');
            expect(j.scope.filterCreateRolesForCurrentUser({publicId: 'Role-1'})).toEqual(false);
        });

        it('should return true if the current user can create the specified role', () => {
            j.scope.userMode = 'new';
            jest.spyOn(_nocCurrentUserService, 'getCurrentUserRolePublicId').mockReturnValue('Role-2');
            expect(j.scope.filterCreateRolesForCurrentUser({publicId: 'Role-2'})).toEqual(true);
        });

        it('should validate the map properly', () => {
            expect(j.scope.allowedRolesToCurrentRole).toEqual({
                'Role-1': {
                    'Role-1': true,
                    'Role-2': true,
                    'Role-3': true
                },
                'Role-2': {
                    'Role-2': true,
                    'Role-3': true
                },
                'Role-3': {
                    'Role-3': true
                }
            });
        });
    });

    describe('Edit branch of initialize', () => {
        const user = {
            name: 'user',
            publicId: '1',
            tenant: null,
            role: {name: 'Tenant Admin', publicId: 'Role-2'},
            authenticationMode: 'Cloud Manager'
        };
        const tenant = {name: 'Tenant2', publicId: 'Tenant-2345'};
        const userWithTenantAndRole = {
            publicId: '3',
            firstName: 'Anatoly',
            lastName: 'Shaoulov',
            email: 'anatoly@puki.com',
            roleId: 'Role-2',
            tenantId: 'Tenant-2345',
            role: {name: 'Tenant Admin', publicId: 'Role-2'},
            tenant,
            workingEnvironments: [{name: 'WE1', publicId: 'we-1'}, {name: 'WE2', publicId: 'we-2'}]
        };

        beforeEach(() => {
            j.parentScope.generateGrantedWorkingEnvironmentsMap = jest.fn().mockReturnValue({});
            j.scope.users = [user, userWithTenantAndRole];
            j.scope.selectedTenant = tenant;
        });

        it('should set the originalSelectedUser to be the passed in user', () => {
            j.scope.initializeEditor('1');
            expect(j.scope.originalSelectedUser).toEqual(user);
            expect(j.scope.generateGrantedWorkingEnvironmentsMap).toHaveBeenCalled();
        });

        it('should set the selected user to be a deep clone of the originalSelected user', () => {
            j.scope.initializeEditor('1');
            expect(j.scope.selectedUser === j.scope.originalSelectedUser).toEqual(false);
            expect(j.scope.selectedUser).toEqual(j.scope.originalSelectedUser);
        });

        it('should display the role and tenant of selected user', () => {
            j.scope.initializeEditor('3');
            expect(j.scope.selectedUserTenantId).toEqual('Tenant-2345');
            expect(j.scope.selectedRolePublicId).toEqual('Role-2');
        });

        it('should fill in the accountId of the user from the server', () => {
            j.$httpBackend.expectGET(j.usersUrl + '/1').respond({accountId: 12});
            j.scope.initializeEditor('1');
            expect(j.scope.selectedUser.accountId).not.toBeDefined();
            j.$httpBackend.flush();
            expect(j.scope.selectedUser.accountId).toEqual(12);
        });
    });

    describe('Creating a new user', () => {
        beforeEach(inject((_nocCurrentUserService_) => {
            _nocCurrentUserService = _nocCurrentUserService_;
            j.scope.selectedUser = {role: {publicId: 'Role-1'}};
        }));

        it('should create an empty user with an empty granted map, and the first allowed role should be selected, and clone it when the current user is of role oncloud admin', () => {
            _nocCurrentUserService.initializeCurrentUser(mockedUsers[0]);
            j.scope.initializeEditor();
            expect(j.scope.selectedUser).toEqual({
                grantedMap: {},
                role: j.scope.roles[0],
                authenticationMode: 'Cloud Manager',
                notificationsOptIn: true
            });
            expect(j.scope.selectedUser === j.scope.originalSelectedUser).toEqual(false);
            expect(j.scope.selectedUser).toEqual(j.scope.originalSelectedUser);
        });

        describe('(logged in as tenant admin)', () => {
            beforeEach(inject((_nocCurrentUserService_) => {
                _nocCurrentUserService = _nocCurrentUserService_;
                _nocCurrentUserService.initializeCurrentUser(mockedUsers[1]);

                j.scope.roles = [
                    {name: 'OnCloud Admin', publicId: 'Role-1'},
                    {name: 'Tenant Admin', publicId: 'Role-2'},
                    {name: 'WE Admin', publicId: 'Role-3'}
                ];
            }));

            it('should select the first allowed role', () => {
                j.parentScope.selectedTenant = {publicId: 'Paris'};
                jest.spyOn(_nocCurrentUserService, 'getCurrentUserRolePublicId').mockReturnValue('Role-2');
                j.scope.initializeEditor();
                expect(j.scope.selectedUser.role).toEqual(j.scope.roles[1]);
            });

            it('should set the current user\'s tenant to the new user', () => {
                j.parentScope.selectedTenant = {publicId: 'Paris'};
                j.scope.initializeEditor();
                expect(j.scope.selectedUser.tenant).toEqual({publicId: 'Paris'});
            });
        });
    });

    describe('selectedTenantWE', () => {
        beforeEach(() => {
            j.scope.workingEnvironments = [
                {id: 1, tenantId: 1},
                {id: 2, tenantId: 1},
                {id: 3, tenantId: 2}
            ];

            j.scope.tenants = [{publicId: 1}, {publicId: 2}, {publicId: 3}];
        });

        function selectTenant(id) {
            j.scope.$apply(() => {
                j.scope.selectedUserTenantId = id;
            });
        }

        it('should be empty when no user is selected', () => {
            expect(j.scope.selectedTenantWE).toEqual([]);
        });

        it('should be empty when no tenant is selected', () => {
            j.scope.$apply(() => {
                j.scope.selectedUser = {name: 'tenant'};
            });
            expect(j.scope.selectedTenantWE).toEqual([]);
        });

        it('should be empty when there are no WE in the selected tenant', () => {
            selectTenant(3);
            expect(j.scope.selectedTenantWE).toEqual([]);
        });

        //todo: fix
        // it('should not be empty when there are some WE in the selected tenant', () => {
        //     selectTenant(1);
        //     expect(j.scope.selectedTenantWE).toEqual([{id: 1, tenantId: 1}, {id: 2, tenantId: 1}]);
        // });

        it('should be populated when role changes', () => {
            selectTenant(2);
            j.scope.selectedUser.tenant = null;
            j.scope.$apply(() => {
                j.scope.selectedUser.role = 'bla';
                j.scope.selectedUserTenantId = 2;
            });

            expect(j.scope.selectedUser.tenant).toEqual({publicId: 2});
        });
    });
});
