describe('BaseCreateVolumeOnExistingWorkingEnvironment', function () {
    var j;
    beforeEach(() => {
        j = magnifispec.setup();
    });
    let BaseCreateVolumeOnExistingWorkingEnvironment;
    let CifsResource;
    beforeEach(inject(function (_BaseCreateVolumeOnExistingWorkingEnvironment_, _CifsResource_) {
        BaseCreateVolumeOnExistingWorkingEnvironment = _BaseCreateVolumeOnExistingWorkingEnvironment_;
        CifsResource = _CifsResource_;
        j.scope = {
            selectedWorkingEnvironment: {
                _pathPrefix: '/vsa/',
                _cloudProviderName: 'AWS',
                publicId: 'we_123',
                workingEnvironmentType: 'VSA',
                providerVolumeType: 'gp2',
                awsProperties: {vpc: {cidrBlock: '1.1.1.1/24'}}
            }
        };
        BaseCreateVolumeOnExistingWorkingEnvironment.call(j.scope, j.pidgeon);
    }));

    describe('checkCifsAvailability', () => {
        it('should resolve if there are cifs configuration', () => {
            const cifsConfiguraionArr = [1, 2, 3];
            jest.spyOn(CifsResource.prototype, 'getCifsConfigurations').mockReturnValue(j.$q.when(cifsConfiguraionArr));

            expect(j.scope.checkCifsAvailability('1', {workingEnvironmentType: 'type1'}, 'svm1')).toBeResolvedWith(cifsConfiguraionArr.length);
        });

        it('should reject if getCifsConfigurations rejects', () => {
            jest.spyOn(CifsResource.prototype, 'getCifsConfigurations').mockReturnValue(j.$q.when(j.$q.reject()));

            expect(j.scope.checkCifsAvailability('1', {workingEnvironmentType: 'type1'}, 'svm1')).toBeRejectedWith();
        });
    });

    describe('getDefaultExportPolicyIps', function () {
        it('should return the selected vpc cidr for AWS', function () {
            j.scope.selectedWorkingEnvironment._typeAndProvider = 'VSA_AWS';
            expect(j.scope.getDefaultExportPolicyIps()).toEqual(['1.1.1.1/24']);
        });

        it('should return the selected selected subnet cidr for Azure', function () {
            j.scope.selectedWorkingEnvironment._typeAndProvider = 'VSA_Azure';
            j.scope.selectedWorkingEnvironment.providerProperties = {vnetCidr: '1.1.1.1/2'};
            expect(j.scope.getDefaultExportPolicyIps()).toEqual(['1.1.1.1/2']);
        });

        it('should return an empty array for any other case (not aws not azure)', function () {
            j.scope.selectedWorkingEnvironment._typeAndProvider = 'ON_PREM';
            expect(j.scope.getDefaultExportPolicyIps()).toEqual([]);
        });
    });

    describe('volumeProtocolChanged', () => {
        beforeEach(() => {
            j.scope.wizardController = {
                path: {details: 'path-to-details'},
                setWizardStep: jest.fn()
            };
            j.scope.volumeProtocol = {};
            j.scope.initSyncToS3Status = jest.fn();
        });

        it('should should set the wizard step back to the first step', () => {
            j.scope.volumeProtocolChanged();

            expect(j.scope.wizardController.setWizardStep).toHaveBeenCalledWith('path-to-details');
        });

        it('case changed to NFS', () => {
            j.scope.volumeProtocol.type = 'NFS';

            j.scope.volumeProtocolChanged();

            expect(j.scope.needCifsConfiguration).toEqual(false);
            expect(j.scope.initSyncToS3Status).toHaveBeenCalled();
            expect(j.scope.syncToS3Status).not.toBeDefined();
        });

        describe('case changed to CIFS', () => {
            beforeEach(() => {
                j.scope.volumeRequest = {
                    name: 'volName',
                    svmName: 'svm1',
                    shareInfo: {}
                };
                j.scope.volumeProtocol.type = 'CIFS';
                jest.spyOn(j.scope, 'checkCifsAvailability').mockReturnValue(j.$q.when(false));
            });

            it('case VSA', () => {
                j.scope.volumeProtocolChanged();
                magnifispec.processPromise();

                expect(j.scope.checkCifsAvailability).toHaveBeenCalledWith('we_123', j.scope.selectedWorkingEnvironment, undefined);
                expect(j.scope.needCifsConfiguration).toEqual(true);
            });

            it('case ON_PREM', () => {
                j.scope.selectedWorkingEnvironment.workingEnvironmentType = 'ON_PREM';

                j.scope.volumeProtocolChanged();
                magnifispec.processPromise();

                expect(j.scope.checkCifsAvailability).toHaveBeenCalledWith('we_123', j.scope.selectedWorkingEnvironment, 'svm1');
                expect(j.scope.needCifsConfiguration).toEqual(false);
                expect(j.scope.volumeProtocol.type).toEqual('NFS');
                expect(j.pidgeon.error).toHaveBeenCalledWith('CIFS is not configured. You can configure it in System Manager.', '', 'cifs');
            });

            afterEach(() => {
                expect(j.scope.syncToS3Status).toEqual('CIFS');
                expect(j.scope.volumeRequest).toEqual({
                    name: 'volName',
                    svmName: 'svm1',
                    shareInfo: {shareName: 'volName_share'},
                    syncToS3: false
                });
            });
        });

        it('should truncate sharename if it is too long', () => {
            j.scope.volumeRequest = {
                name: 'volName',
                shareInfo: {}
            };
            j.scope.volumeProtocol.type = 'CIFS';
            jest.spyOn(j.scope, 'checkCifsAvailability').mockReturnValue(j.$q.when());

            j.scope.volumeRequest.name = '1bcdefghij2bcdefghij3bcdefghij4bcdefghij5bcdefghij6bcdefghij7bcdefghij8bcdX'; //75 chars, with '_share' it's 81.

            j.scope.volumeProtocolChanged();

            expect(j.scope.volumeRequest.shareInfo.shareName).toEqual('');
        });

    });
});