describe('ReplicationStatusController', () => {
    'use strict';

    var j;    beforeEach(() => {        j = magnifispec.setup();    });
    let nocSchedulesService, nocPoliciesService;

    const workingEnvironment = {name: 'we1'};
    const policies = [
        {isDefault: false, isSupported: true, reasons: [], _displayName: 'DPDefault', name: 'DPDefault', type: 'async_mirror', description: 'Default policy for DP relationship', rules: [{snapMirrorLabel: 'sm_created', retention: 1}]},
        {isDefault: true, isSupported: true, reasons: [], _displayName: 'Mirror all snapshots', name: 'MirrorAllSnapshots', type: 'async_mirror', description: 'Asynchronous SnapMirror policy for mirroring all snapshots and latest active file system', rules: [{snapMirrorLabel: 'sm_created', retention: 1}, {snapMirrorLabel: 'all_source_snapshots', retention: 1}]},
        {isDefault: false, isSupported: true, reasons: [], name: 'MirrorAndVault', type: 'mirror_vault', description: 'A unified Asynchronous SnapMirror and SnapVault policy for mirroring the latest active file system and daily and weekly snapshots', rules: [{snapMirrorLabel: 'sm_created', retention: 1}, {snapMirrorLabel: 'daily', retention: 7}]},
        {isDefault: false, isSupported: false, reason: 'unsupported version', name: 'XDPDefault', type: 'vault', description: 'Default policy for XDP relationship with daily and weekly rules', rules: [{snapMirrorLabel: 'daily', retention: 7}]}
    ];

    const mockedRelationship = {
        'source': {
            'workingEnvironmentId': 1,
            'clusterName': 'hili2912900',
            'svmName': 'svm_hili2912900',
            'nodeName': null,
            'volumeName': 'hilivol2912900'
        },
        'destination': {
            'workingEnvironmentId': 2,
            'clusterName': 'Itay6th',
            'svmName': 'svm_Itay6th',
            'nodeName': 'Itay6th-01',
            'volumeName': 'hilivol2912900_copy'
        },
        'mirrorState': 'SnapMirrored',
        'volumeUsedSize': {size: 500, unit: 'TB'}
    };

    const mockedRelationship2 = {
        'source': {
            'workingEnvironmentId': 2,
            'clusterName': 'hili2912901',
            'svmName': 'svm_hili2912901',
            'nodeName': null,
            'volumeName': 'hilivol2912901'
        },
        'destination': {
            'workingEnvironmentId': 2,
            'clusterName': 'Itay6th1',
            'svmName': 'svm_Itay6th1',
            'nodeName': 'Itay6th-02',
            'volumeName': 'hilivol2912900_copy2'
        },
        'mirrorState': 'SnapMirrored',
        'volumeUsedSize': {size: 1000, unit: 'TB'}
    };

    beforeEach(inject((_nocSchedulesService_, _nocPoliciesService_) => {
        nocSchedulesService = _nocSchedulesService_;
        nocPoliciesService = _nocPoliciesService_;

        magnifispec.mockTabsController('ReplicationStatusController', undefined, {});
        j.replicationUrl = j.nocBaseUrl + '/replication';
        j.scope.selectedWorkingEnvironment = workingEnvironment;
    }));

    describe('action methods', () => {
        beforeEach(() => {
            jest.spyOn(j.scope, 'sendPostRequest').mockImplementation(() => {});
        });

        it('should call the send put request with break when break action activated', () => {
            j.scope.breakRelationship(mockedRelationship);
            expect(j.scope.sendPostRequest).toHaveBeenCalledWith(mockedRelationship, 'break', 'breaking', true);
        });

        it('should call the send post request with reverse when reverse action activated', () => {
            magnifispec.mockConfirm(true);

            j.scope.reverseResyncRelationship(mockedRelationship);
            magnifispec.processPromise();

            expect(j.scope.sendPostRequest).toHaveBeenCalledWith(mockedRelationship, 'reverse', 'reversing', false, {
                sourceWorkingEnvironmentId: mockedRelationship.source.workingEnvironmentId,
                sourceSvmName: mockedRelationship.source.svmName,
                sourceVolumeName: mockedRelationship.source.volumeName
            });
        });

        it('should not call the sendPost request with reverse when the user does not confirm', () => {
            magnifispec.mockConfirm(false);
            j.scope.reverseResyncRelationship(mockedRelationship);
            expect(j.scope.sendPostRequest).not.toHaveBeenCalled();
        });

        it('should call the send post request with resync when resync action activated', () => {
            magnifispec.mockConfirm(true);

            j.scope.resyncRelationship(mockedRelationship);
            magnifispec.processPromise();

            expect(j.scope.sendPostRequest).toHaveBeenCalledWith(mockedRelationship, 'resync', 'resynchronizing', false);
        });

        it('should not call the sendPost request with resync when the user does not confirm', () => {
            magnifispec.mockConfirm(false);

            j.scope.resyncRelationship(mockedRelationship);
            magnifispec.processPromise();

            expect(j.scope.sendPostRequest).not.toHaveBeenCalled();
        });
    });

    describe('editSchedule', () => {
        let schedules;

        beforeEach(() => {
            jest.spyOn(j.replicationResource, 'updateRelationship').mockImplementation(() => {});
            mockedRelationship.schedule = '5min';
            schedules = [{name: '5min'}, {name: 'daily'}];
        });

        it('it should open a dialog and update relationship if changed', () => {
            jest.spyOn(nocSchedulesService, 'getSchedules').mockReturnValue(j.$q.resolve(schedules));
            const confirmDeferred = j.$q.defer();
            jest.spyOn(j.nocDialog, 'confirm').mockReturnValue(confirmDeferred.promise);
            j.scope.editSchedule(mockedRelationship);
            expect(nocSchedulesService.getSchedules).toHaveBeenCalled();
            magnifispec.processPromise();
            expect(j.nocDialog.confirm).toHaveBeenCalledWith('Select a replication schedule', 'assets/templates/dialogs/select-schedule-dialog.html', {
                schedules,
                width: 965,
                fromTop: '10%',
                setSelected: jasmine.any(Function),
                getSelected: jasmine.any(Function)
            });
            const getSelected = mostRecentCall(j.nocDialog.confirm)[2].getSelected;
            expect(getSelected()).toEqual({name: '5min'});
            const setSelected = mostRecentCall(j.nocDialog.confirm)[2].setSelected;
            setSelected({name: 'NEW_SCHEDULE'});
            confirmDeferred.resolve();
            magnifispec.processPromise();
            expect(j.replicationResource.updateRelationship).toHaveBeenCalledWith(jasmine.any(Object), mockedRelationship.destination,
                {schedule: 'NEW_SCHEDULE'}, 'updating schedule information');
        });

        it('should open a dialog and do nothing if no new schedule picked', () => {
            jest.spyOn(nocSchedulesService, 'getSchedules').mockReturnValue(j.$q.resolve(schedules));
            magnifispec.mockConfirm(true);
            j.scope.editSchedule(mockedRelationship);
            expect(nocSchedulesService.getSchedules).toHaveBeenCalled();
            magnifispec.processPromise();
            expect(j.nocDialog.confirm).toHaveBeenCalled();
            expect(j.replicationResource.updateRelationship).not.toHaveBeenCalled();
        });

        it('should open a dialog and do nothing if cancelled', () => {
            jest.spyOn(nocSchedulesService, 'getSchedules').mockReturnValue(j.$q.resolve(schedules));
            magnifispec.mockConfirm(false);
            j.scope.editSchedule(mockedRelationship);
            expect(nocSchedulesService.getSchedules).toHaveBeenCalled();
            magnifispec.processPromise();
            expect(j.nocDialog.confirm).toHaveBeenCalled();
            expect(j.replicationResource.updateRelationship).not.toHaveBeenCalled();
        });
    });

    describe('sendPostRequest', () => {
        it('should show a dialog with the right message and do nothing if not confirmed', () => {
            magnifispec.mockConfirm(false);
            j.scope.sendPostRequest(mockedRelationship, 'break', 'breaking', true);
            expect(j.nocDialog.confirm).toHaveBeenCalledWith('Are you sure you want to break the relationship between "hilivol2912900" and "hilivol2912900_copy"?');
        });

        it('should call a put request with the right parameters when the dialog is confirmed', () => {
            magnifispec.mockConfirm(true);
            j.$httpBackend.expectPOST(j.replicationUrl + '/break/2/svm_Itay6th/hilivol2912900_copy').respond(204);

            j.scope.sendPostRequest(mockedRelationship, 'break', 'breaking');
            j.$httpBackend.flush();

            expect(j.nocUI.unblock).toHaveBeenCalled();
        });

        it('should call a put request with the right parameters when the dialog is confirmed, and show an error when the put request fails', () => {
            magnifispec.mockConfirm(true);
            j.$httpBackend.expectPOST(j.replicationUrl + '/break/2/svm_Itay6th/hilivol2912900_copy').respond(400, j.errorResponse);

            j.scope.sendPostRequest(mockedRelationship, 'break', 'breaking');
            j.$httpBackend.flush();

            expect(j.nocUI.unblock).toHaveBeenCalled();
            expect(j.pidgeon.error).toHaveBeenCalled();
        });

        describe('on get statuses', () => {
            const statuses = ['statuses'];
            it('should fill statuses on right working environment send', () => {
                j.scope.$broadcast(j.replicationResource.REPLICATION_STATUS_GET_EVENT, {workingEnvironment, replicationStatusResponse: statuses});
                expect(j.scope.replicationStatusResponse).toEqual(statuses);
                expect(j.nocUI.unloading).toHaveBeenCalled();
            });

            it('should listen to get all too', () => {
                j.scope.$broadcast(j.replicationResource.REPLICATION_STATUS_GET_ALL_EVENT, {workingEnvironment, replicationStatusResponse: statuses});
                expect(j.scope.replicationStatusResponse).toEqual(statuses);
                expect(j.nocUI.unloading).toHaveBeenCalled();
            });
        });

        describe('on get statuses error', () => {
            it('pidgeon it', () => {
                j.scope.$broadcast(j.replicationResource.REPLICATION_STATUS_GET_ERROR_EVENT, {workingEnvironment, error: j.errorResponse});
                expect(j.pidgeon.error).toHaveBeenCalledWith(j.pidgeon.serverErrorPrefix + 'retrieving replication status for working environment we1', j.errorResponse, 'replicationStatus');
                expect(j.nocUI.unloading).toHaveBeenCalled();
            });

            it('should listen to get all error event too', () => {
                j.scope.$broadcast(j.replicationResource.REPLICATION_STATUS_GET_ALL_ERROR_EVENT, {workingEnvironment, error: j.errorResponse});
                expect(j.pidgeon.error).toHaveBeenCalledWith(j.pidgeon.serverErrorPrefix + 'retrieving replication status', j.errorResponse, 'replicationStatus');
                expect(j.nocUI.unloading).toHaveBeenCalled();
            });
        });

        describe('deleteRelationship', () => {
            it('should show a dialog with the right message and do nothing if not confirmed', () => {
                magnifispec.mockConfirm(false);
                j.scope.deleteRelationship(mockedRelationship);
                expect(j.nocDialog.confirm).toHaveBeenCalledWith('Are you sure you want to delete the relationship between "hilivol2912900" and "hilivol2912900_copy"?');
            });

            it('should call a delete request with the right parameters when the dialog is confirmed', () => {
                magnifispec.mockConfirm(true);
                j.$httpBackend.expectDELETE(j.replicationUrl + '/2/svm_Itay6th/hilivol2912900_copy').respond(204);

                j.scope.deleteRelationship(mockedRelationship);
                j.$httpBackend.flush();

                expect(j.nocUI.unblock).toHaveBeenCalled();
            });

            it('should call a delete request with the right parameters when the dialog is confirmed, and show an error when the delete request fails', () => {
                magnifispec.mockConfirm(true);

                j.$httpBackend.expectDELETE(j.replicationUrl + '/2/svm_Itay6th/hilivol2912900_copy').respond(400, j.errorResponse);

                j.scope.deleteRelationship(mockedRelationship);
                j.$httpBackend.flush();

                expect(j.nocUI.unblock).toHaveBeenCalled();
                expect(j.pidgeon.error).toHaveBeenCalled();
            });
        });
    });

    describe('calculateTotalCapacity', () => {
        beforeEach(() => {
            j.scope.replicationStatusResponse = [mockedRelationship, mockedRelationship2];
        });

        it('should calculate the total capacity correctly when both units are TB', () => {
            j.scope.calculateTotalVolumesUsedSize();
            expect(j.scope.totalVolumesUsedSize).toEqual({size: 1500, unit: 'TB'});
        });

        it('should convert to TB when required', () => {
            mockedRelationship.volumeUsedSize.unit = 'GB';
            mockedRelationship2.volumeUsedSize.unit = 'GB';
            j.scope.calculateTotalVolumesUsedSize();
            expect(j.scope.totalVolumesUsedSize).toEqual({size: 1.46484375, unit: 'TB'});
        });

        it('should use the best unit', () => {
            mockedRelationship.volumeUsedSize.unit = 'MB';
            mockedRelationship2.volumeUsedSize.unit = 'GB';
            j.scope.calculateTotalVolumesUsedSize();
            expect(j.scope.totalVolumesUsedSize).toEqual({size: 1000.48828125, unit: 'GB'});
        });
    });

    describe('refresh', () => {
        const replicationStatusResponse = {
            workingEnvironmentId: workingEnvironment.publicId,
            workingEnvironmentName: workingEnvironment.name
        };

        beforeEach(() => {
            jest.spyOn(j.scope, 'calculateTotalVolumesUsedSize').mockImplementation(() => {});
            j.$rootScope.selectedTenant = {publicId: '1'};
        });

        it('should call refresh on working environment resource and handle success case correctly', () => {
            jest.spyOn(j.replicationResource, 'getReplicationStatus').mockImplementation(magnifispec.mockPromise(replicationStatusResponse));

            j.scope.refresh();

            magnifispec.processPromise();

            expect(j.nocUI.loading).toHaveBeenCalled();
            expect(j.scope.isRefreshOngoing).toEqual(false);
            expect(j.scope.replicationStatusResponse).toEqual(replicationStatusResponse);
        });

        it('should set the ongoing status', () => {
            j.scope.selectedWorkingEnvironment = null;
            j.$httpBackend.expectGET(j.nocBaseUrl + '/replication/status?tenantId=1').respond(200, []);

            expect(j.scope.isRefreshOngoing).toEqual(false);
            j.scope.refresh();
            expect(j.scope.isRefreshOngoing).toEqual(true);
        });

        it('no GET while refresh is pending', () => {
            j.scope.selectedWorkingEnvironment = null;
            j.scope.isRefreshOngoing = true;
            j.scope.refresh();
            j.scope.refresh();
            j.$httpBackend.verifyNoOutstandingRequest();
        });
    });

    describe('should disable', () => {
        beforeEach(() => {
            mockedRelationship._allowedRelationshipOperations = {
                'break-relationship': false,
                'resync-relationship': false,
                'edit-relationship': false,
                'delete-relationship': false,
                'policy-info': false,
                'update-snapmirror': false,
                'reverse-snapmirror': false
            };
        });

        it('should return true if no menu item is allowed', () => {
            expect(j.scope.shouldDisable(mockedRelationship)).toEqual(true);
        });

        it('should return false if at least one menu item is allowed', () => {
            mockedRelationship._allowedRelationshipOperations['resync-relationship'] = true;
            expect(j.scope.shouldDisable(mockedRelationship)).toEqual(false);
        });
    });

    describe('policyInfo', () => {
        let relationship;

        beforeEach(() => {
            relationship = {source: {workingEnvironmentId: 'sourceWEId', svmName: 'svmName', volumeName: 'volName'}, destination: {workingEnvironmentId: 'targetWEId'}, policy: 'DPDefault'};
            jest.spyOn(nocPoliciesService, 'getPolicies').mockReturnValue(j.$q.when(policies));
            jest.spyOn(nocPoliciesService, 'getSnapshotLabels').mockReturnValue(j.$q.when(policies));
        });

        it('should call nocPoliciesService getPolicies and getSnapshotLabels for the right policy, then display dialog', () => {
            jest.spyOn(j.nocDialog, 'dialog').mockImplementation(() => {});
            j.scope.policyInfo(relationship);
            magnifispec.processPromise();

            expect(nocPoliciesService.getPolicies).toHaveBeenCalled();
            expect(nocPoliciesService.getSnapshotLabels).toHaveBeenCalled();

            expect(j.scope.policy).toEqual(policies[0]);

            const callsForGetPoliciesExceptNocHttp = nocPoliciesService.getPolicies.mock.calls[0].splice(-2);
            expect(callsForGetPoliciesExceptNocHttp).toEqual(['sourceWEId', 'targetWEId']);
            const callsForGetSnapshotLabelsExceptNocHttp = nocPoliciesService.getSnapshotLabels.mock.calls[0].splice(-4);
            expect(callsForGetSnapshotLabelsExceptNocHttp).toEqual([[policies[0]], 'sourceWEId', 'svmName', 'volName']);

            expect(j.nocDialog.dialog).toHaveBeenCalledWith('Policy Information', 'assets/templates/dialogs/policy-info.html', {
                policy: policies[0],
                width: 536
            });
        });
    });

    describe('editMaxTransferRateConfirm', () => {
        const relationship = {
            maxTransferRate: {size: 1024 * 1.5, unit: 'KB'},
            _totalTransferSize: {sortValue: 123 * 1024 * 1024},
            _totalTransferTime: {sortValue: 10}
        };

        beforeEach(() => {
            jest.spyOn(j.scope, 'editMaxTransferRate').mockImplementation(() => {});
        });

        it('should pass the correct args for the unlimited rate', () => {
            magnifispec.mockConfirm(false);
            j.scope.editMaxTransferRateConfirm({
                maxTransferRate: {size: 0, unit: 'KB'},
                _totalTransferSize: {sortValue: 20 * 1024 * 1024},
                _totalTransferTime: {sortValue: 10}
            });
            expect(j.nocDialog.confirm).toHaveBeenCalledWith('Edit Max Transfer Rate', 'assets/templates/dialogs/edit-max-transfer-rate-dialog.html', {
                width: 620,
                wizardController: {
                    limitMaxTransferRate: false,
                    maxTransferRateMBps: undefined,
                    avgTransferRateMBps: 2
                },
                maxInt32: 2147483647
            });
        });

        it('should open a confirm and do nothing if canceled', () => {
            magnifispec.mockConfirm(false);
            j.scope.editMaxTransferRateConfirm(relationship);
            expect(j.nocDialog.confirm).toHaveBeenCalledWith('Edit Max Transfer Rate', 'assets/templates/dialogs/edit-max-transfer-rate-dialog.html', {
                width: 620,
                wizardController: {
                    limitMaxTransferRate: true,
                    maxTransferRateMBps: 1.5,
                    avgTransferRateMBps: 12.3
                },
                maxInt32: 2147483647
            });
            expect(j.scope.editMaxTransferRate).not.toHaveBeenCalled();
        });

        it('should open a confirm and do it\'s work upon success', () => {
            magnifispec.mockConfirm({wizardController: {maxTransferRateMBps: 2, limitMaxTransferRate: true}});

            j.scope.editMaxTransferRateConfirm(relationship);
            magnifispec.processPromise();

            expect(j.nocDialog.confirm).toHaveBeenCalledWith('Edit Max Transfer Rate', 'assets/templates/dialogs/edit-max-transfer-rate-dialog.html', {
                width: 620,
                wizardController: {
                    limitMaxTransferRate: true,
                    maxTransferRateMBps: 1.5,
                    avgTransferRateMBps: 12.3
                },
                maxInt32: 2147483647
            });
            expect(j.scope.editMaxTransferRate).toHaveBeenCalledWith(relationship, true, 2 * 1024);
        });
    });

    describe('editMaxTransferRate', () => {
        let relationship;
        beforeEach(() => {
            relationship = {
                maxTransferRate: {size: 1234, unit: 'KB'},
                destination: {workingEnvironmentId: 'id1', svmName: 'svm1', volumeName: 'vol1'}
            };
        });

        it('should do nothing if the rate has not changed', () => {
            j.scope.editMaxTransferRate(relationship, true, 1234);
            j.$httpBackend.verifyNoOutstandingRequest(); //This actually doesn't test what it should because of angular bug: https://github.com/angular/angular.js/issues/5453
        });

        describe('should call http.put', () => {
            it('should work upon success with updated maxTransferRate', () => {
                j.$httpBackend.expectPUT(j.nocBaseUrl + '/replication/id1/svm1/vol1', {maxTransferRate: 3456}).respond(200);
                j.scope.editMaxTransferRate(relationship, true, 3456);
                j.$httpBackend.flush();
            });

            it('should work upon success with unlimited maxTransferRate', () => {
                j.$httpBackend.expectPUT(j.nocBaseUrl + '/replication/id1/svm1/vol1', {maxTransferRate: 0}).respond(200);
                j.scope.editMaxTransferRate(relationship, false, 1234);
                j.$httpBackend.flush();
            });

            it('should work upon error', () => {
                j.$httpBackend.expectPUT(j.nocBaseUrl + '/replication/id1/svm1/vol1', {maxTransferRate: 3456}).respond(400, j.errorResponse);
                j.scope.editMaxTransferRate(relationship, true, 3456);
                j.$httpBackend.flush();
                expect(j.pidgeon.error).toHaveBeenCalled();
            });

            afterEach(() => {
                expect(j.pidgeon.clear).toHaveBeenCalledWith('updating max transfer rate');
                expect(j.nocUI.block).toHaveBeenCalled();
                expect(j.nocUI.unblock).toHaveBeenCalled();
            });
        });
    });
});
