describe('VirginCreateWorkingEnvironmentDetailsStepAndCredentialsController', () => {
    'use strict';

    var j;    beforeEach(() => {        j = magnifispec.setup();    });

    beforeEach(inject(() => {
        j.$rootScope.currentUser = {tenantId: 2};

        magnifispec.mockWizardStepController('VirginCreateWorkingEnvironmentDetailsStepAndCredentialsController', {}, {
            manifestData: false,
            choice: 'VSA',
            workingEnvironmentType: 'VSA',
            workingEnvironmentTypeAndProvider: {type: 'VSA', providerName: 'AWS'},
            providerAccounts: {awsAccounts: [{publicId: '1'}], azureAccounts: [{publicId: '2'}]}
        });

        j.$rootScope.instancePlacement = {installLocation: 'AWS'};
    }));

    describe('initialize on conditions', () => {
        it('should initialize when the controller is created for the first time and the lastCloudProvider is undefined', () => {
            jest.spyOn(j.scope, 'initialize').mockImplementation(() => {});

            j.scope.lastCloudProvider = undefined;
            j.scope.wizardController.choice = 'newProviderName';
            j.scope.$digest(); //Triggers scope.$watch

            expect(j.scope.initialize).toHaveBeenCalled();
        });

        it('should initialize when the choice is changed', () => {
            jest.spyOn(j.scope, 'initialize').mockImplementation(() => {});
            j.scope.lastCloudProvider = 'AWS';

            j.scope.wizardController.choice = 'Azure';
            j.scope.wizardController.workingEnvironmentTypeAndProvider.providerName = 'Azure';
            j.scope.$digest(); //Triggers scope.$watch

            expect(j.scope.initialize).toHaveBeenCalled();
        });
    });

    describe('initialize', () => {
        it('should keep the last cloud provider name', () => {
            j.scope.initialize();
            expect(j.scope.lastCloudProvider).toEqual('AWS');
        });

        it('should initialize all the fields properly for AWS', () => {
            j.scope.initialize();
            expect(j.scope.isAWS).toEqual(true);
            expect(j.scope.regexName).toEqual('workingEnvironmentName');
            expect(j.scope.weNameRegex).toEqual(/^[a-zA-Z][0-9a-zA-Z_]{0,39}$/);
        });

        it('should initialize all the fields properly for Azure', () => {
            j.scope.wizardController.workingEnvironmentTypeAndProvider.providerName = 'Azure';
            j.scope.initialize();
            expect(j.scope.isAWS).toEqual(false);
            expect(j.scope.regexName).toEqual('azureWorkingEnvironmentName');
            expect(j.scope.weNameRegex).toEqual(/^[a-zA-Z][0-9a-zA-Z]{0,39}$/);
        });

        it('should initialize to empty custom tags if do not exist on the wizardController', () => {
            j.scope.wizardController.customTags = undefined;
            j.scope.initialize();
            expect(j.scope.wizardController.customTags).toEqual([]);
        });

        it('should initialize to empty custom tags if provider has been changed', () => {
            j.scope.wizardController.workingEnvironmentTypeAndProvider.providerName = 'Azure';
            j.scope.wizardController.detailsBeenHereBefore = 'AWS';
            j.scope.wizardController.customTags = [{tagKey: '123', tagValue: '123'}];
            j.scope.initialize();
            expect(j.scope.wizardController.customTags).toEqual([]);
        });

        it('should preserve custom tags if provider has not changed', () => {
            j.scope.wizardController.workingEnvironmentTypeAndProvider.providerName = 'AWS';
            j.scope.wizardController.detailsBeenHereBefore = 'AWS';
            j.scope.wizardController.customTags = [{tagKey: '123', tagValue: '123'}];
            j.scope.initialize();
            expect(j.scope.wizardController.customTags).toEqual([{tagKey: '123', tagValue: '123'}]);
        });
    });

    describe('validateWorkingEnvironmentNameUniqueness', () => {
        beforeEach(() => {
            j.scope.wizardController.workingEnvironmentName = 'we1';
            j.scope.getTagKeysAndPutOnWizardController = jest.fn();
        });

        it('should do nothing and return resolved promise if in recreate mode', () => {
            j.scope.wizardController.failedRequestType = 'Azure';

            expect(j.scope.validateWorkingEnvironmentNameUniqueness()).toBeResolved();
        });

        it('should be resolved if WE name is unique', () => {
            j.$httpBackend.expectGET(j.nocBaseUrl + '/working-environments/exists/we1').respond(false);

            expect(j.scope.validateWorkingEnvironmentNameUniqueness()).toBeResolved();

            expect(j.pidgeon.clear).toHaveBeenCalledWith('weName');
            expect(j.pidgeon.error).not.toHaveBeenCalled();
        });

        it('should be rejected with pidgeon if WE name is not unique', () => {
            j.$httpBackend.expectGET(j.nocBaseUrl + '/working-environments/exists/we1').respond(true);

            expect(j.scope.validateWorkingEnvironmentNameUniqueness()).toBeRejected();

            expect(j.pidgeon.clear).toHaveBeenCalledWith('weName');
            expect(j.pidgeon.error).toHaveBeenCalledWith('The name \'we1\' is already used by another working environment. Please use another one.', null, 'weName');
        });

        it('should be rejected if API fails', () => {
            j.$httpBackend.expectGET(j.nocBaseUrl + '/working-environments/exists/we1').respond(500);

            expect(j.scope.validateWorkingEnvironmentNameUniqueness()).toBeRejected();

            expect(j.pidgeon.clear).toHaveBeenCalledWith('weName');
            expect(j.pidgeon.error).toHaveBeenCalledWith('An error occurred while checking if the working environment name is already used', jasmine.any(Object), 'checking if the working environment name is already used');
        });
    });

    describe('validateResourceGroupNameUniqueness', () => {
        beforeEach(() => {
            j.scope.wizardController.resourceGroupName = 'we-name-rg';
            j.scope.wizardController.choice = 'AZURE';
            j.scope.wizardController.selectedAzureSubscription = {subscriptionId: '123'};
            j.scope.getTagKeysAndPutOnWizardController = jest.fn();
        });

        it('should do nothing and return resolved promise if in recreate mode', () => {
            j.scope.wizardController.failedRequestType = 'Azure';

            expect(j.scope.validateResourceGroupNameUniqueness()).toBeResolved();
        });

        it('should do nothing and return resolved promise if working environment is not in Azure', () => {
            j.scope.wizardController.choice = 'AWS';

            expect(j.scope.validateResourceGroupNameUniqueness()).toBeResolved();
        });

        it('should be resolved if resource group name is unique', () => {
            j.$httpBackend.expectGET(j.nocBaseUrl + '/azure/vsa/working-environments/resource-group-exists/we-name-rg?subscriptionId=123').respond(false);

            expect(j.scope.validateResourceGroupNameUniqueness()).toBeResolved();

            expect(j.pidgeon.clear).toHaveBeenCalledWith('resourceGroupName');
            expect(j.pidgeon.error).not.toHaveBeenCalled();
        });

        it('should be rejected with pidgeon if resource group name is not unique', () => {
            j.$httpBackend.expectGET(j.nocBaseUrl + '/azure/vsa/working-environments/resource-group-exists/we-name-rg?subscriptionId=123').respond(true);

            expect(j.scope.validateResourceGroupNameUniqueness()).toBeRejected();

            expect(j.pidgeon.clear).toHaveBeenCalledWith('resourceGroupName');
            expect(j.pidgeon.error).toHaveBeenCalledWith('The name \'we-name-rg\' is already used by another resource group. Please use another one.', null, 'resourceGroupName');
        });

        it('should be rejected if API fails', () => {
            j.$httpBackend.expectGET(j.nocBaseUrl + '/azure/vsa/working-environments/resource-group-exists/we-name-rg?subscriptionId=123').respond(500);

            expect(j.scope.validateResourceGroupNameUniqueness()).toBeRejected();

            expect(j.pidgeon.clear).toHaveBeenCalledWith('resourceGroupName');
            expect(j.pidgeon.error).toHaveBeenCalledWith('An error occurred while checking if the resource group name is already used', jasmine.any(Object), 'checking if the resource group name is already used');
        });
    });

    describe('next', () => {
        beforeEach(() => {
            j.scope.wizardController.next = jest.fn();
            j.scope.wizardController.isValid = jest.fn().mockReturnValue(true);
            j.scope.getTagKeysAndPutOnWizardController = jest.fn();
        });

        it('should validate the form', () => {
            j.scope.isValid.mockReturnValue(false);
            j.scope.next();
            expect(j.nocUI.handleFormValidity).toHaveBeenCalledWith(j.scope.VirginWizardForm);
        });

        it('should continue to "vpc" step for AWS VSA if the WE name does not exist', () => {
            j.scope.wizardController.workingEnvironmentTypeAndProvider.combined = 'VSA_AWS';
            jest.spyOn(j.scope, 'validateWorkingEnvironmentNameUniqueness').mockReturnValue(j.$q.when());
            jest.spyOn(j.scope, 'validateResourceGroupNameUniqueness').mockReturnValue(j.$q.when());

            j.scope.next();
            magnifispec.processPromise();

            expect(j.scope.wizardController.next).toHaveBeenCalledWith('.vpc');
        });

        it('should continue to "ha-vpc" step for AWS-HA VSA if the WE name does not exist', () => {
            j.scope.wizardController.workingEnvironmentTypeAndProvider.combined = 'HA_AWS';
            jest.spyOn(j.scope, 'validateWorkingEnvironmentNameUniqueness').mockReturnValue(j.$q.when());
            jest.spyOn(j.scope, 'validateResourceGroupNameUniqueness').mockReturnValue(j.$q.when());

            j.scope.next();
            magnifispec.processPromise();

            expect(j.scope.wizardController.next).toHaveBeenCalledWith('.choose-ha-deployment-model');
        });

        it('should continue to "vnet" step for Azure VSA if the resource group name and the  WE name does not exist', () => {
            j.scope.wizardController.workingEnvironmentTypeAndProvider = {combined: 'VSA_Azure'};
            jest.spyOn(j.scope, 'validateWorkingEnvironmentNameUniqueness').mockReturnValue(j.$q.when());
            jest.spyOn(j.scope, 'validateResourceGroupNameUniqueness').mockReturnValue(j.$q.when());

            j.scope.next();
            magnifispec.processPromise();

            expect(j.scope.wizardController.next).toHaveBeenCalledWith('.vnet');
        });

        it('should not continue if the we name already exists', () => {
            jest.spyOn(j.scope, 'validateWorkingEnvironmentNameUniqueness').mockReturnValue(j.$q.reject());
            jest.spyOn(j.scope, 'validateResourceGroupNameUniqueness').mockReturnValue(j.$q.when());

            j.scope.next();
            magnifispec.processPromise();

            expect(j.scope.wizardController.next).not.toHaveBeenCalled();
        });

        it('should not continue if the resource group name already exists', () => {
            jest.spyOn(j.scope, 'validateWorkingEnvironmentNameUniqueness').mockReturnValue(j.$q.when());
            jest.spyOn(j.scope, 'validateResourceGroupNameUniqueness').mockReturnValue(j.$q.reject());

            j.scope.next();
            magnifispec.processPromise();

            expect(j.scope.wizardController.next).not.toHaveBeenCalled();
        });
    });

    describe('destroy', () => {
        it('should do cleanup', () => {
            j.scope.wizardController.workingEnvironmentName = 'bla';
            j.scope.wizardController.svmPassword = 'bla';

            j.scope.$broadcast('$destroy');
            expect(j.parentScope.backupStepData).toHaveBeenCalledWith('create-detailsCredentials', {
                workingEnvironmentName: 'bla',
                svmPassword: 'bla',
                retypeSVMPassword: undefined,
                useDefaultResourceGroupName: undefined,
                resourceGroupName: undefined,
                selectedAWSAccountId: undefined,
                selectedAzureSubscription: undefined,
                cloudProviderAccount: undefined,
                detailsBeenHereBefore: undefined
            });
            expect(j.scope.wizardController.workingEnvironmentName).toEqual('');
        });

        it('should restore backed up data upon creation ', () => {
            expect(j.parentScope.getStepData).toHaveBeenCalledWith('create-detailsCredentials');
        });
    });
});
