describe('CreateWorkingEnvironmentPackagesStepController', () => {
    'use strict';

    var j;    beforeEach(() => {        j = magnifispec.setup();    });
    let ManifestMapper;

    beforeEach(inject((_ManifestMapper_, MetadataResource) => {
        ManifestMapper = _ManifestMapper_;
        jest.spyOn(MetadataResource, 'getPackages').mockReturnValue(j.$q.when([
            {
                name: 'tiered_package',
                diskType: 'gp2',
                diskSize: {size: 500, unit: 'GB', _identifier: '500 GB'},
                capacityTier: 'S3',
                licenseType: 'explore',
                instanceTypeMapping: [{region: 'default', instanceType: 'm4.large'}]
            }, {
                name: 'not_tiered_package',
                diskType: 'st1',
                diskSize: {size: 1, unit: 'TB', _identifier: '1 TB'},
                capacityTier: undefined,
                licenseType: 'standard',
                instanceTypeMapping: [{region: 'default', instanceType: 'm4.large'}]
            }
        ]));

        magnifispec.mockWizardStepController('CreateWorkingEnvironmentPackagesStepController', {}, {
            workingEnvironmentTypeAndProvider: {type: 'VSA', providerName: 'AWS', combined: 'VSA_AWS'},
            selectedRegion: 'ab-west-1',
            next: jest.fn(),
            setWizardStep: jest.fn(),
            path: {packages: 'path.to.packages.step'},
            clearStepData: jest.fn()
        });

        j.scope.wizardController.currentStateName = 'packages';
        j.scope.isAWS = true;

        jest.spyOn(ManifestMapper, 'getAvailableLicenses').mockReturnValue([{type: 'explore'}, {type: 'standard'}]);
        jest.spyOn(ManifestMapper, 'getAvailableVersions').mockReturnValue(['version0', 'version1']);
    }));

    describe('initialize', () => {
        it('should get licenses', () => {
            j.scope.initialize();

            expect(j.scope.licenses).toEqual([{type: 'explore'}, {type: 'standard'}]);
            expect(ManifestMapper.getAvailableLicenses).toHaveBeenCalledWith('ab-west-1', 'version0');
        });

        it('should get and initialize packages', () => {
            j.scope.initialize();
            magnifispec.processPromise();

            expect(j.scope.packages).toEqual([
                {
                    name: 'tiered_package',
                    diskType: 'gp2',
                    diskSize: {size: 500, unit: 'GB', _identifier: '500 GB'},
                    capacityTier: 'S3',
                    licenseType: 'explore',
                    instanceTypeMapping: [{region: 'default', instanceType: 'm4.large'}],
                    instanceType: 'm4.large',
                    _diskType: {
                        name: 'gp2',
                        shortName: 'gp2',
                        diskName: 'gp2',
                        display: 'General Purpose SSD (gp2)',
                        description: 'General purpose SSD volume that balances price and performance.',
                        supportedInHa: true,
                        supportedCapacityTiers: ['S3'],
                        default: true
                    },
                    _tierName: 'gp2+S3',
                    _license: {type: 'explore'},
                    _diskTypeCap: 'General Purpose SSD (gp2)'
                }, {
                    name: 'not_tiered_package',
                    diskType: 'st1',
                    diskSize: {size: 1, unit: 'TB', _identifier: '1 TB'},
                    capacityTier: undefined,
                    licenseType: 'standard',
                    instanceTypeMapping: [{region: 'default', instanceType: 'm4.large'}],
                    instanceType: 'm4.large',
                    _diskType: {
                        name: 'st1',
                        shortName: 'st1',
                        diskName: 'st1',
                        display: 'Throughput Optimized HDD (st1)',
                        description: 'Low cost HDD volume designed for frequently accessed, throughput-intensive workloads.',
                        supportedInHa: true,
                        supportedCapacityTiers: ['S3']
                    },
                    _tierName: 'st1',
                    _license: {type: 'standard'},
                    _diskTypeCap: 'Throughput Optimized HDD (st1)'
                }
            ]);
        });

        it('should disable packages with capacity tier if ONTAP encryption was selected', () => {
            j.scope.wizardController.encryptionType = 'ONTAP';

            j.scope.initialize();
            magnifispec.processPromise();

            expect(j.scope.packages).toEqual([
                jasmine.objectContaining({
                    name: 'tiered_package',
                    diskType: 'gp2',
                    capacityTier: 'S3',
                    disabled: true,
                    disabledText: 'This package is not supported with ONTAP encryption.'
                }), jasmine.objectContaining({
                    name: 'not_tiered_package',
                    diskType: 'st1',
                    capacityTier: undefined
                })
            ]);
        });

        it('should use BYOL as license for all packages, if BYOL selected', () => {
            j.scope.wizardController.selectedLicense = {type: 'byol-license'};

            j.scope.initialize();
            magnifispec.processPromise();

            expect(j.scope.packages).toEqual([
                jasmine.objectContaining({
                    _license: {type: 'byol-license'}
                }),
                jasmine.objectContaining({
                    _license: {type: 'byol-license'}
                })
            ]);
        });

        it('should re-select selected package in re-create mode', () => {
            j.scope.wizardController.failedRequestType = true;
            j.scope.wizardController.failedRequest = {packageName: 'not_tiered_package'};

            j.scope.initialize();
            magnifispec.processPromise();

            expect(j.scope.wizardController.selectedPackage).toEqual(jasmine.objectContaining({name: 'not_tiered_package'}));
        });
    });

    describe('getLatestOntapVersions', () => {
        it('should return the first version for this region', () => {
            expect(j.scope.getLatestOntapVersions()).toEqual('version0');

            expect(ManifestMapper.getAvailableVersions).toHaveBeenCalledWith('ab-west-1');
        });
    });

    describe('clearFutureStepsDataIfNeeded', () => {
        function expectToClearAndCollapse(selectedPackage, clickedPackage) {
            j.scope.wizardController.selectedPackage = selectedPackage;

            j.scope.clearFutureStepsDataIfNeeded(clickedPackage);

            expect(j.scope.wizardController.setWizardStep).toHaveBeenCalledWith('path.to.packages.step');
            expect(j.scope.wizardController.clearStepData).toHaveBeenCalledWith('volume');
        }

        function expectNotToClearAndCollapse(selectedPackage, clickedPackage) {
            j.scope.wizardController.selectedPackage = selectedPackage;

            j.scope.clearFutureStepsDataIfNeeded(clickedPackage);

            expect(j.scope.wizardController.setWizardStep).not.toHaveBeenCalled();
            expect(j.scope.wizardController.clearStepData).not.toHaveBeenCalled();
        }

        it('should clear data if is switching from custom package to pre-conf package', () => {
            expectToClearAndCollapse({name: 'aws_custom', isCustom: true}, {name: 'poc'});
        });

        it('should clear data if is switching from pre-conf package to custom package', () => {
            expectToClearAndCollapse({name: 'poc'}, {name: 'aws_custom', isCustom: true});
        });

        it('should not clear data if is selecting custom package for the first time', () => {
            expectNotToClearAndCollapse(undefined, {name: 'aws_custom', isCustom: true});
        });

        it('should not clear data if is selecting pre-conf package for the first time', () => {
            expectNotToClearAndCollapse(undefined, {name: 'poc'});
        });

        it('should clear data if switching from package with S3 tiering to package without', () => {
            expectToClearAndCollapse(
                {name: 'low-cost', _tierName: 'gp2+S3', capacityTier: 'S3'},
                {name: 'standard', _tierName: 'gp2', capacityTier: undefined}
            );
        });

        it('should clear data if switching from package without S3 tiering to package with it', () => {
            expectToClearAndCollapse(
                {name: 'standard', _tierName: 'gp2', capacityTier: undefined},
                {name: 'low-cost', _tierName: 'gp2+S3', capacityTier: 'S3'}
            );
        });

        it('should not clear data if switching between 2 different packages without S3 tier', () => {
            expectNotToClearAndCollapse({name: 'standard', _tier: {name: 'gp2'}}, {name: 'poc', _tier: {name: 'st1'}});
        });
    });

    describe('selectPackage', () => {
        beforeEach(() => {
            jest.spyOn(j.scope, 'next').mockImplementation(() => {});
            jest.spyOn(j.scope, 'clearFutureStepsDataIfNeeded').mockImplementation(() => {});
        });

        it('should do nothing for disabled package', () => {
            j.scope.wizardController.selectedPackage = 'old-package';

            j.scope.selectPackage({disabled: true});
            j.$timeout.flush();

            expect(j.scope.wizardController.selectedPackage).toEqual('old-package');
        });

        it('should set all properties according to package in AWS', () => {
            j.scope.isAWS = true;
            const awsPackage = {
                _diskType: 'tier1',
                diskSize: {size: 33, unit: 'TB'},
                instanceType: 'm4-large',
                _license: 'license1',
                instanceTenancy: 'default',
                writingSpeedState: 'NORMAL'
            };

            j.scope.selectPackage(awsPackage);
            j.$timeout.flush();

            expect(j.scope.wizardController.selectedPackage).toEqual(awsPackage);
            expect(j.scope.wizardController.selectedTier).toEqual('tier1');
            expect(j.scope.wizardController.providerVolumeSize).toEqual({size: 33, unit: 'TB'});
            expect(j.scope.wizardController.selectedInstanceType).toEqual('m4-large');
            expect(j.scope.wizardController.selectedLicense).toEqual('license1');
            expect(j.scope.wizardController.tenancy).toEqual({name: 'Shared', requestName: 'default'});
            expect(j.scope.wizardController.writingSpeedState).toEqual('NORMAL');
            expect(j.scope.wizardController.selectedOntapVersion).toEqual('version0');

            expect(j.scope.clearFutureStepsDataIfNeeded).toHaveBeenCalledWith(awsPackage);
        });

        it('should set all properties according to package in Azure', () => {
            j.scope.isAWS = false;
            const azurePackage = {
                diskSize: {size: 2, unit: 'TB', _identifier: '2 TB'}
            };

            j.scope.selectPackage(azurePackage);
            j.$timeout.flush();

            expect(j.scope.wizardController.providerVolumeSize).toEqual({size: 2, unit: 'TB', _identifier: '2 TB'});
            expect(j.scope.clearFutureStepsDataIfNeeded).toHaveBeenCalledWith(azurePackage);
        });

        it('should go to next step', () => {
            j.scope.selectPackage({});
            j.$timeout.flush();

            expect(j.scope.next).toHaveBeenCalled();
            expect(j.scope.clearFutureStepsDataIfNeeded).toHaveBeenCalledWith({});
        });
    });

    describe('selectCreateMyOwn', () => {
        it('should create custom package for AWS single node', () => {
            j.scope.selectCreateMyOwn();
            j.$timeout.flush();

            expect(j.scope.selectedPackage).toEqual({name: 'aws_custom', isCustom: true});
            expect(j.scope.wizardController.next).toHaveBeenCalledWith('.roles');
        });

        it('should create custom package for AWS HA', () => {
            j.scope.wizardController.workingEnvironmentTypeAndProvider = {
                type: 'HA',
                providerName: 'AWS',
                combined: 'HA_AWS'
            };

            j.scope.selectCreateMyOwn();
            j.$timeout.flush();

            expect(j.scope.selectedPackage).toEqual({name: 'aws_ha_custom', isCustom: true});
            expect(j.scope.wizardController.next).toHaveBeenCalledWith('.roles');
        });

        it('should create custom package for Azure', () => {
            j.scope.wizardController.workingEnvironmentTypeAndProvider = {
                type: 'VSA',
                providerName: 'Azure',
                combined: 'VSA_Azure'
            };

            j.scope.selectCreateMyOwn();
            j.$timeout.flush();

            expect(j.scope.selectedPackage).toEqual({name: 'azure_custom', isCustom: true});
            expect(j.scope.wizardController.next).toHaveBeenCalledWith('.roles');
        });

        it('should not move to next step if scrolled here from future step', () => {
            j.scope.wizardController.currentStateName = 'future-step';

            j.scope.selectCreateMyOwn();
            j.$timeout.flush();

            expect(j.scope.wizardController.next).not.toHaveBeenCalledWith();
        });
    });

    describe('next', () => {
        it('should call continueAfterLicenseOrPackagesStep', () => {
            jest.spyOn(j.scope, 'continueAfterLicenseOrPackagesStep').mockImplementation(() => {});

            j.scope.next();

            expect(j.scope.continueAfterLicenseOrPackagesStep).toHaveBeenCalledWith('.volume');
        });
    });

    describe('backing up data', () => {
        it('should clear fields upon destroy', () => {
            j.scope.wizardController.selectedPackage = 'value1';
            j.scope.wizardController.selectedTier = 'value2';
            j.scope.wizardController.providerVolumeSize = 'value3';
            j.scope.wizardController.selectedInstanceType = 'value4';
            j.scope.wizardController.selectedLicense = 'value5';
            j.scope.wizardController.tenancy = 'value6';
            j.scope.wizardController.writingSpeedState = 'value7';
            j.scope.wizardController.selectedOntapVersion = 'value8';

            j.$rootScope.$broadcast('$destroy');
            expect(j.parentScope.backupStepData).toHaveBeenCalledWith('packages', {
                selectedPackage: 'value1',
                selectedTier: 'value2',
                providerVolumeSize: 'value3',
                selectedInstanceType: 'value4',
                selectedLicense: 'value5',
                tenancy: 'value6',
                writingSpeedState: 'value7',
                selectedOntapVersion: 'value8'
            });
            expect(j.scope.wizardController.selectedPackage).toBeNull();
            expect(j.scope.wizardController.selectedTier).toBeNull();
            expect(j.scope.wizardController.providerVolumeSize).toBeNull();
            expect(j.scope.wizardController.selectedInstanceType).toBeNull();
            expect(j.scope.wizardController.selectedLicense).toBeNull();
            expect(j.scope.wizardController.tenancy).toBeNull();
            expect(j.scope.wizardController.writingSpeedState).toBeNull();
            expect(j.scope.wizardController.selectedOntapVersion).toBeNull();
        });

        it('should restore backed up data upon creation ', () => {
            expect(j.parentScope.getStepData).toHaveBeenCalledWith('packages');
        });
    });
});
