describe('Volume Step Controller', () => {
    const defaultSnapshotPolicies = [{name: 'policy1', count: 1, description: 'policy1 description'}, {
        name: 'policy2',
        count: 2,
        description: 'policy2 description'
    }];

    var j;    beforeEach(() => {        j = magnifispec.setup();    });

    beforeEach(inject((_TieringMetadataResource_) => {
        jest.spyOn(_TieringMetadataResource_, 'getSupportedCapacityTiers').mockReturnValue(j.$q.when());

        j.$httpBackend.expectGET(j.nocBaseUrl + '/vsa/metadata/default-snapshot-policies').respond(200, {});
        magnifispec.mockWizardStepController('VirginCreateWorkingEnvironmentVolumeStepController', {}, {
            getDefaultExportPolicyIps: jest.fn().mockReturnValue([]),
            workingEnvironmentTypeAndProvider: {type: 'VSA', providerName: 'AWS'},
            selectedLicense: {type: 'cot-standard-paygo', name: 'Cloud Volumes ONTAP Standard'},
            createVpcSelectObjects: {
                vpc: {cidrBlock: ''}
            },
            selectedTier: {name: 'gp2'},
            pathPrefix: '/vsa/'
        });

        j.$httpBackend.flush();
        j.scope.snapshotPolicies = defaultSnapshotPolicies;
        j.$rootScope.capacityFeature = true;
        j.$state.go = jest.fn();
    }));

    describe('initialize with VSA', () => {
        it('should get the snapshot policies and update them to the scope on success', () => {
            j.scope.wizardController.workingEnvironmentType = 'VSA';
            j.$httpBackend.expectGET(j.nocBaseUrl + '/vsa/metadata/default-snapshot-policies').respond([{name: 'policy1FromServer'}, {name: 'policy2FromServer'}]);
            j.scope.initialize();
            j.$httpBackend.flush();
            expect(j.scope.snapshotPolicies).toEqual([{name: 'policy1FromServer'}, {name: 'policy2FromServer'}]);
            expect(j.nocUI.unblock).toHaveBeenCalled();
        });

        it('should get the snapshot policies and push an alert on error', () => {
            j.scope.wizardController.workingEnvironmentType = 'VSA';
            j.$httpBackend.expectGET(j.nocBaseUrl + '/vsa/metadata/default-snapshot-policies').respond(400, j.errorResponse);
            j.scope.initialize();
            j.$httpBackend.flush();
            expect(j.scope.snapshotPolicies).toEqual(defaultSnapshotPolicies);
            expect(j.pidgeon.error).toHaveBeenCalled();
            expect(j.nocUI.unblock).toHaveBeenCalled();
        });

        it('should remove all loading symbols from screen when creating VSA', () => {
            j.scope.wizardController.workingEnvironmentType = 'VSA';
            j.$httpBackend.expectGET(j.nocBaseUrl + '/vsa/metadata/default-snapshot-policies').respond([{name: 'policy1FromServer'}, {name: 'policy2FromServer'}]);
            j.scope.initialize();
            j.$httpBackend.flush();
            //Due to the call in the creation of the controller
            j.expectUItoBlockAndUnblock(2);
        });

        it('should not disable "No Storage Efficiency" in case the tiering enabled checkbox was selected', () => {
            j.scope.wizardController.tieringEnabled = true;

            j.scope.initialize();

            expect(j.scope.disabledUsageProfile).toEqual(undefined);
        });
    });
    describe('skip handling', () => {
        beforeEach(() => {
            jest.spyOn(j.scope, 'nextStep').mockImplementation(() => {});
        });

        it('should set the volumeSkipped to true, delete vol name and size, and call handle next pressed when skip is called', () => {
            j.scope.volumeRequest = {
                name: 'vol1',
                sizeGb: 2,
                usageProfile: {display: 'No Storage Efficiency'},
                snapshotPolicyName: 'lala'
            };
            j.scope.wizardController.volumeSkipped = false;
            j.scope.skip();
            expect(j.scope.volumeRequest.name).toEqual('');
            expect(j.scope.volumeRequest.sizeGb).toEqual('');
            expect(j.scope.wizardController.volumeSkipped).toEqual(true);
            expect(j.scope.volumeRequest.usageProfile.display).toEqual('Storage Efficiency');
            expect(j.scope.volumeRequest.snapshotPolicyName).toEqual('policy1');
            expect(j.scope.nextStep).toHaveBeenCalled();
        });
        it('should set the volumeSkipped to false and call handle next pressed when next is called when user filled vol name', () => {
            j.scope.volumeRequest.name = 'vol1';
            j.scope.wizardController.volumeSkipped = true;
            j.scope.next();
            expect(j.scope.wizardController.volumeSkipped).toEqual(false);
            expect(j.scope.nextStep).toHaveBeenCalled();
        });
        it('should set the volumeSkipped to false and call handle next pressed when next is called when user filled vol size', () => {
            j.scope.volumeRequest.sizeGb = '44';
            j.scope.wizardController.volumeSkipped = true;
            j.scope.next();
            expect(j.scope.wizardController.volumeSkipped).toEqual(false);
            expect(j.scope.nextStep).toHaveBeenCalled();
        });
    });

    describe('get snapshot by policy name', () => {
        it('should return the right policy from the list according to name', () => {
            expect(j.scope.getSnapshotPolicyByName('policy2')).toEqual(defaultSnapshotPolicies[1]);
        });
    });

    describe('userPressedEnter', () => {
        it('should skip if no volume name and size entered', () => {
            jest.spyOn(j.scope, 'skip').mockImplementation(() => {});
            j.scope.wizardController.volumeSkipped = 'some_value';
            j.scope.volumeRequest.name = '';
            j.scope.volumeRequest.sizeGb = '';
            j.scope.userPressedEnter();
            expect(j.scope.skip).toHaveBeenCalled();
            expect(j.scope.wizardController.volumeSkipped).not.toEqual(false);
        });
        it('should call handleFormValidity if the form is not valid', () => {
            j.scope.volumeRequest.name = 'vol1';
            j.scope.isValid = jest.fn().mockReturnValue(false);
            j.scope.userPressedEnter();
            expect(j.$state.go).not.toHaveBeenCalled();
            expect(j.nocUI.handleErrorsInWizard).toHaveBeenCalledWith(true);
        });
        it('should go to the next step is form is valid', () => {
            j.scope.volumeRequest.name = 'vol1';
            j.scope.volumeRequest.sizeGb = '1';
            j.scope.isValid = jest.fn().mockReturnValue(true);
            j.scope.userPressedEnter();
            expect(j.pidgeon.clear).toHaveBeenCalled();
            expect(j.$state.go).toHaveBeenCalledWith('.tiering-policy');
            expect(j.nocUI.handleFormValidity).not.toHaveBeenCalled();
        });
    });

    describe('next', () => {
        it('Should go to next step', () => {
            jest.spyOn(j.scope, 'nextStep').mockImplementation(() => {});
            j.scope.wizardController.volumeSkipped = 'some_value';

            j.scope.next();

            expect(j.scope.wizardController.volumeSkipped).toEqual(false);
            expect(j.scope.nextStep).toHaveBeenCalled();
        });
    });

    describe('set wizard step', () => {
        beforeEach(() => {
            j.scope.setWizardStep = jest.fn();
            j.scope.wizardController.path = {volume: 'path.to.volume.step'};
        });

        it('should set the wizard step to "volume" if the user skipped this step and then scrolled up from further step and changed vol name', () => {
            j.scope.wizardController.volumeSkipped = true;
            j.scope.currentStepPath = 'path.to.review.step';
            j.scope.$apply(() => {
                j.scope.volumeRequest.name = 'volvo';
            });

            expect(j.scope.setWizardStep).toHaveBeenCalledWith('path.to.volume.step');
        });

        it('should set the wizard step to "volume" if the user skipped this step and then scrolled up from further step and changed vol size', () => {
            j.scope.wizardController.volumeSkipped = true;
            j.scope.currentStepPath = 'path.to.review.step';
            j.scope.$apply(() => {
                j.scope.volumeRequest.sizeGb = 5;
            });

            expect(j.scope.setWizardStep).toHaveBeenCalledWith('path.to.volume.step');
        });

        it('should do nothing if called when volume step is current step', () => {
            j.scope.wizardController.volumeSkipped = true;
            j.scope.currentStepPath = 'path.to.volume.step';
            j.scope.$apply(() => {
                j.scope.volumeRequest.name = 'volvo';
            });

            expect(j.scope.setWizardStep).not.toHaveBeenCalled();
        });

        it('should do nothing if volume step was not skipped', () => {
            j.scope.wizardController.volumeSkipped = false;
            j.scope.currentStepPath = 'path.to.review.step';
            j.scope.$apply(() => {
                j.scope.volumeRequest.name = 'volvo';
            });

            expect(j.scope.setWizardStep).not.toHaveBeenCalled();
        });
    });

    describe('nextStep', () => {
        const expectToGoToNextStep = () => {
            expect(j.pidgeon.clear).toHaveBeenCalled();
            expect(j.nocUI.unblock).toHaveBeenCalled();
            expect(j.nocUI.clearFormValidity).toHaveBeenCalled();
        };

        it('should go to review step if valid', () => {
            j.scope.isValid = jest.fn().mockReturnValue(true);

            j.scope.nextStep();

            expectToGoToNextStep();
            expect(j.$state.go).toHaveBeenCalledWith('.tiering-policy');
        });

        it('should go to review step if volumeSkipped, even if form is not valid', () => {
            j.scope.isValid = jest.fn().mockReturnValue(false);
            j.scope.wizardController.volumeSkipped = true;

            j.scope.nextStep();

            expectToGoToNextStep();
            expect(j.$state.go).toHaveBeenCalledWith('.review');
        });

        it('should not go to review step if not valid', () => {
            j.scope.isValid = jest.fn().mockReturnValue(false);
            j.scope.wizardController.volumeSkipped = false;

            j.scope.nextStep();

            expect(j.$state.go).not.toHaveBeenCalledWith('.tiering-policy');
            expect(j.nocUI.handleErrorsInWizard).toHaveBeenCalledWith(true);
        });
    });

    describe('getDefaultExportPolicyIps', () => {
        it('should return the selected vpc cidr for AWS', () => {
            j.scope.wizardController.workingEnvironmentTypeAndProvider.providerName = 'AWS';
            j.scope.wizardController.createVpcSelectObjects.vpc = {cidrBlock: '1.1.1.1/1'};
            expect(j.scope.getDefaultExportPolicyIps()).toEqual(['1.1.1.1/1']);
        });

        it('should return the selected selected subnet cidr for Azure', () => {
            j.scope.wizardController.workingEnvironmentTypeAndProvider.providerName = 'Azure';
            j.scope.wizardController.selectedVnet = {cidr: {cidr: '1.1.1.1/2'}};
            expect(j.scope.getDefaultExportPolicyIps()).toEqual(['1.1.1.1/2']);
        });

        it('should return an empty array for any other case (not aws not azure)', () => {
            j.scope.wizardController.workingEnvironmentTypeAndProvider.providerName = '';
            expect(j.scope.getDefaultExportPolicyIps()).toEqual([]);
        });
    });
});
