describe('VirginCloudOntapCredentialsStepController', () => {
    'use strict';

    let j;
    beforeEach(() => {
        j = magnifispec.setup();
    });

    let recoverUrl, AzureRecoverUrl, HaRecoverUrl, nssUrl, _nocSelectedWorkspaceOrTenantService;

    beforeEach(inject((nocSelectedWorkspaceOrTenantService) => {
        magnifispec.mockWizardStepController('VirginCloudOntapCredentialsStepController', {}, {
            selectedWorkingEnvironment: {
                _pathPrefix: '/vsa/',
                workingEnvironmentPublicId: '1',
                name: 'we1',
                clusterAddress: '1.1.1.1',
                vpcId: 'vpc1',
                stackId: 'stack1',
                type: 'VSA'
            },
            workingEnvironmentTypeAndProvider: {type: 'VSA', providerName: 'AWS'}
        });

        recoverUrl = j.nocBaseUrl + '/vsa/recover';
        nssUrl = j.nocBaseUrl + '/accounts';
        AzureRecoverUrl = j.nocBaseUrl + '/azure/vsa/recover';
        HaRecoverUrl = j.nocBaseUrl + '/aws/ha/discovery/recover';

        _nocSelectedWorkspaceOrTenantService = nocSelectedWorkspaceOrTenantService;
        jest.spyOn(_nocSelectedWorkspaceOrTenantService, 'getPublicId').mockReturnValue('tenant1');
    }));

    describe('recover', () => {
        let data;

        beforeEach(() => {
            j.scope.setPathToPreviousOrRoot = jest.fn();
            j.scope.wizardController.selectedRegion = 'west-1';
            j.scope.fetched = true;
            j.scope.wizardController.svmPassword = 'pass';
            j.scope.wizardController.selectedAzureSubscription = {name: 'dev', subscriptionId: '12345'};

            j.scope.selectedTenant = {publicId: 'tenant1'};
            data = {
                tenantId: 'tenant1',
                region: 'west-1',
                stackId: 'stack1',
                clusterCredentials: {userName: 'admin', password: 'pass'}
            };
        });

        it('should post the right data and on success and call proceed function', () => {
            j.scope.isValid = jest.fn().mockReturnValue(true);
            j.$httpBackend.expectGET(nssUrl).respond({});
            j.$httpBackend.expectPOST(recoverUrl, data).respond(200);
            j.scope.recover();
            j.$httpBackend.flush();

            expect(j.nocUI.loading).toHaveBeenCalled();
            expect(j.scope.setPathToPreviousOrRoot).toHaveBeenCalled();
            expect(j.nocUI.unloading).toHaveBeenCalled();
        });

        it('should should give an error upon error response', () => {
            j.scope.isValid = jest.fn().mockReturnValue(true);
            j.$httpBackend.expectGET(nssUrl).respond({});
            j.$httpBackend.expectPOST(recoverUrl, data).respond(400, j.errorResponse);
            j.scope.recover();
            j.$httpBackend.flush();

            expect(j.nocUI.loading).toHaveBeenCalled();
            expect(j.pidgeon.error).toHaveBeenCalled();
            expect(j.nocUI.unloading).toHaveBeenCalled();
        });

        it('should call ha vsa if it is an HA instance', () => {
            j.scope.selectedWorkingEnvironment.type = 'HA Cloud Volumes ONTAP';
            j.scope.isValid = jest.fn().mockReturnValue(true);
            j.$httpBackend.expectGET(nssUrl).respond({});
            j.$httpBackend.expectPOST(HaRecoverUrl, data).respond(400, j.errorResponse);

            j.scope.recover();
            j.$httpBackend.flush();

            expect(j.nocUI.loading).toHaveBeenCalled();
            expect(j.pidgeon.error).toHaveBeenCalled();
            expect(j.nocUI.unloading).toHaveBeenCalled();
        });

        it('should post correct data for Azure', () => {
            j.scope.wizardController.workingEnvironmentTypeAndProvider.providerName = 'Azure';

            _.assignIn(j.scope.wizardController.selectedWorkingEnvironment, {
                name: 'AzureWE',
                id: 'id1',
                resourceGroup: 'resourceGroup1'
            });
            j.scope.isValid = jest.fn().mockReturnValue(true);

            j.$httpBackend.expectGET(nssUrl).respond({});
            j.$httpBackend.expectPOST(AzureRecoverUrl, {
                tenantId: 'tenant1',
                region: 'west-1',
                instanceName: 'AzureWE',
                id: 'id1',
                resourceGroupName: 'resourceGroup1',
                clusterCredentials: {
                    userName: 'admin',
                    password: 'pass'
                },
                providerAccount: '12345'
            }).respond(200, {});
            j.scope.recover();
            j.$httpBackend.flush();

            expect(j.nocUI.loading).toHaveBeenCalled();
            expect(j.scope.setPathToPreviousOrRoot).toHaveBeenCalled();
            expect(j.nocUI.unloading).toHaveBeenCalled();
        });

        it('should call handle form validity if the form is not valid', () => {
            j.scope.isValid = jest.fn().mockReturnValue(false);
            j.scope.recover();
            expect(j.nocUI.handleErrorsInWizard).toHaveBeenCalledWith(true);
        });
    });

    describe('backing up data', () => {
        it('should backup data and clear fields upon destroy', () => {
            j.scope.wizardController.svmPassword = 'Netapp234';
            j.$rootScope.$broadcast('$destroy', () => {
                expect(j.parentScope.backupStepData).toHaveBeenCalledWith('credentials', {
                    svmPassword: 'Netapp234'
                });
                expect(j.scope.svmPassword).toEqual('');
            });
        });

        it('should restore backed up data upon creation ', () => {
            expect(j.parentScope.getStepData).toHaveBeenCalledWith('credentials');
        });
    });
});
